/******************************************************
                  DirectShow .NET
		      netmaster@swissonline.ch
*******************************************************/
//					DsUtils
// DirectShow utility classes, partial from the SDK Common sources

using System;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace DShowNET
{

		[ComVisible(false)]
	public class DsUtils
	{

		public static bool IsCorrectDirectXVersion()
		{
			return File.Exists( Path.Combine( Environment.SystemDirectory, @"dpnhpast.dll" ) );
		}

		public static bool ShowPinDialog(IBaseFilter pFilter, IntPtr hwnd )
		{
			int hr;
			object comObj = null;
			ISpecifyPropertyPages pProp = null;
			DsCAUUID cauuid = new DsCAUUID();

			try 
			{
		
		
				comObj = pFilter;
				pProp = pFilter as ISpecifyPropertyPages;
				if( pProp == null )
					return false;

				hr = pProp.GetPages( out cauuid );
				hr = OleCreatePropertyFrame( hwnd, // Parent window 
					                           30, // x (Reserved)
											   30, // y (Reserved)
											 null, // Caption for the dialog box
												1, // Number of filters
									   ref comObj, // Pointer to the filter 
									cauuid.cElems, // Number of property pages
									cauuid.pElems, // Pointer to property page CLSIDs
												0, // Locale identifier
												0, // Reserved
									 IntPtr.Zero ); // Reserved

				return true;
			}
			catch( Exception ee )
			{
				Trace.WriteLine( "!Ds.NET: ShowPinDialog " + ee.Message );
				return false;
			}
			finally
			{
				if( cauuid.pElems != IntPtr.Zero )
					Marshal.FreeCoTaskMem( cauuid.pElems );
				
				pProp = null;
				if( comObj != null )
					//Marshal.ReleaseComObject( comObj ); 
					comObj = null;
			}
		}

		public static bool ShowPrevPinDialog( ICaptureGraphBuilder2 bld, IBaseFilter flt, IntPtr hwnd )
		{
			int hr;
			object comObj = null;
			ISpecifyPropertyPages	spec = null;
			DsCAUUID cauuid = new DsCAUUID();

			try 
			{

				// If media type is video, try to get interface IAMVideoProcAmp
				Guid cat = PinCategory.Preview;
				Guid type = MediaType.Video;
				Guid iid = typeof(IAMVideoProcAmp).GUID;
				hr = bld.FindInterface(ref cat, ref type, flt, ref iid, out comObj);
				if( hr != 0 )
				{
					return false;
				}
				spec = comObj as ISpecifyPropertyPages;
				if( spec == null )
					return false;
				
				hr = spec.GetPages( out cauuid );
				hr = OleCreatePropertyFrame( hwnd, 30, 30, null, 1,
					ref comObj, cauuid.cElems, cauuid.pElems, 0, 0, IntPtr.Zero );
				return true;

			}
			catch( Exception ee )
			{
				Console.WriteLine( "!Ds.NET: ShowPrevPinDialog " + ee.Message );
				return false;
			}
			finally
			{
				if( cauuid.pElems != IntPtr.Zero )
					Marshal.FreeCoTaskMem( cauuid.pElems );
				
				spec = null;
				if( comObj != null )
					Marshal.ReleaseComObject( comObj ); comObj = null;
			}
		}
		

		public static bool AutoCapPinDialog( ICaptureGraphBuilder2 bld, IBaseFilter flt, IntPtr hwnd )
		{
			int hr;
			object comObjPrev = null;
			object comObjCap = null;

			try 
			{

				Guid catCap  = PinCategory.Capture;
				Guid catPrev  = PinCategory.Preview;
				Guid type = MediaType.Interleaved;
				Guid iid = typeof(IAMStreamConfig).GUID;

//				type = MediaType.Interleaved;
//				hr = bld.FindInterface( ref catPrev, ref type, flt, ref iid, out comObjPrev );
//				if( hr != 0 )
//				{
//					type = MediaType.Video;
//					hr = bld.FindInterface( ref catPrev, ref type, flt, ref iid, out comObjPrev );
//					if( hr != 0 )
//						return false;
//				}

				hr = bld.FindInterface( ref catCap, ref type, flt, ref iid, out comObjCap );
				if( hr != 0 )
				{
					type = MediaType.Video;
					hr = bld.FindInterface( ref catCap, ref type, flt, ref iid, out comObjCap );
					
					if( hr != 0 )
						return false;
				}
				
		
				//Instead of calling OleCreatePropertyFrame do ...
				IAMStreamConfig streamConfigCap=null;
				IAMStreamConfig streamConfigPrev=null;
				streamConfigCap=comObjCap as IAMStreamConfig;
				streamConfigPrev=comObjPrev as IAMStreamConfig;
				if(streamConfigCap == null || streamConfigPrev == null) 
				{
					Console.WriteLine("IAMStreamConfig not present");
					return false;
				}

				int CAPTURE_FRAMERATE = 30;
				AMMediaType media = new AMMediaType();
				//streamConfigCap.GetFormat(out media);
				VideoInfoHeader videoInfoHeader = (VideoInfoHeader) Marshal.PtrToStructure( media.formatPtr, typeof(VideoInfoHeader) );
				videoInfoHeader.BitRate=10; 
				videoInfoHeader.BmiHeader.Width=768; 
				videoInfoHeader.BmiHeader.Height=576;
				videoInfoHeader.BmiHeader.ImageSize=1327104; 
				videoInfoHeader.AvgTimePerFrame = (long)(10000000 / CAPTURE_FRAMERATE);
				Marshal.StructureToPtr(videoInfoHeader, media.formatPtr, true);
				streamConfigCap.SetFormat(media); 
				//streamConfigCap.GetFormat(out media);
				Marshal.PtrToStructure( media.formatPtr, typeof(VideoInfoHeader) );

				CAPTURE_FRAMERATE = 5;
				//streamConfigPrev.GetFormat(out media);
				videoInfoHeader = (VideoInfoHeader) Marshal.PtrToStructure( media.formatPtr, typeof(VideoInfoHeader) );
				videoInfoHeader.AvgTimePerFrame = (long)(10000000 / CAPTURE_FRAMERATE);
				streamConfigPrev.SetFormat(media); 
				return true;
				// end
			}
			catch( Exception ee )
			{
				Console.WriteLine( "!Ds.NET: AutoCapPinDialog " + ee.Message );
				return false;
			}
			finally
			{
				if( comObjCap != null )
					Marshal.ReleaseComObject( comObjCap ); comObjCap = null;
				if( comObjPrev != null )
					Marshal.ReleaseComObject( comObjPrev ); comObjPrev = null;
			}
		}
		public static bool ShowCapPinDialog( ICaptureGraphBuilder2 bld, IBaseFilter flt, IntPtr hwnd )
		{
			int hr;
			object comObj = null;
			ISpecifyPropertyPages	spec = null;
			DsCAUUID cauuid = new DsCAUUID();

			try {
				Guid cat  = PinCategory.Capture;
				Guid type = MediaType.Interleaved;
				Guid iid = typeof(IAMStreamConfig).GUID;
				hr = bld.FindInterface( ref cat, ref type, flt, ref iid, out comObj );
				if( hr != 0 )
				{
					type = MediaType.Video;
					hr = bld.FindInterface( ref cat, ref type, flt, ref iid, out comObj );
					if( hr != 0 )
						return false;
				}
				spec = comObj as ISpecifyPropertyPages;
				if( spec == null )
					return false;

				hr = spec.GetPages( out cauuid );
				hr = OleCreatePropertyFrame( hwnd, 30, 30, null, 1,
						ref comObj, cauuid.cElems, cauuid.pElems, 0, 0, IntPtr.Zero );
				return true;
			}
			catch( Exception ee )
			{
				Console.WriteLine( "!Ds.NET: ShowCapPinDialog " + ee.Message );
				return false;
			}
			finally
			{
				if( cauuid.pElems != IntPtr.Zero )
					Marshal.FreeCoTaskMem( cauuid.pElems );
					
				spec = null;
				if( comObj != null )
					Marshal.ReleaseComObject( comObj ); comObj = null;
			}
		}

		public static bool ShowTunerPinDialog( ICaptureGraphBuilder2 bld, IBaseFilter flt, IntPtr hwnd )
		{
			int hr;
			object comObj = null;
			ISpecifyPropertyPages	spec = null;
			DsCAUUID cauuid = new DsCAUUID();

			try {
				Guid cat  = PinCategory.Capture;
				Guid type = MediaType.Interleaved;
				Guid iid = typeof(IAMTVTuner).GUID;
				hr = bld.FindInterface( ref cat, ref type, flt, ref iid, out comObj );
				if( hr != 0 )
				{
					type = MediaType.Video;
					hr = bld.FindInterface( ref cat, ref type, flt, ref iid, out comObj );
					if( hr != 0 )
						return false;
				}
				spec = comObj as ISpecifyPropertyPages;
				if( spec == null )
					return false;

				hr = spec.GetPages( out cauuid );
				hr = OleCreatePropertyFrame( hwnd, 30, 30, null, 1,
						ref comObj, cauuid.cElems, cauuid.pElems, 0, 0, IntPtr.Zero );
				return true;
			}
			catch( Exception ee )
			{
				Trace.WriteLine( "!Ds.NET: ShowCapPinDialog " + ee.Message );
				return false;
			}
			finally
			{
				if( cauuid.pElems != IntPtr.Zero )
					Marshal.FreeCoTaskMem( cauuid.pElems );
					
				spec = null;
				if( comObj != null )
					Marshal.ReleaseComObject( comObj ); comObj = null;
			}

			
		}

		// from 'DShowUtil.cpp' does not work in C#, if you have some ideas how to realize this send it to ...@fhbb.ch
//		private int SaveGraphFile(IGraphBuilder pGraph, string wszPath)
//		{

//			const string StreamName = "ActiveMovieGraph"; 
//			int hr;
//			UCOMIStorage pStorage;
//
//			// First, create a document file which will hold the GRF file
//			hr = StgCreateDocfile(
//				wszPath,
//				STGM_CREATE | STGM_TRANSACTED | STGM_READWRITE | STGM_SHARE_EXCLUSIVE,
//				0,  out pStorage);
//			if(hr < 0) 
//			{
//				return hr;
//			}
//
//			// Next, create a stream to store.
//			UCOMIStream pStream;
//			//IStream pStream;
//			hr = pStorage.CreateStream(
//				StreamName,
//				STGM_WRITE | STGM_CREATE | STGM_SHARE_EXCLUSIVE,
//				0, 0, &pStream);
//			if (hr < 0) 
//			{   
//				return hr;
//			}
//
//			// The IPersistStream converts a stream into a persistent object.
//			IPersistStream pPersist;
//			Marshal.ReleaseComObject(pGraph); pGraph = null;
//			//pGraph->QueryInterface(IID_IPersistStream, reinterpret_cast<void**>(&pPersist));
//			hr = pPersist.Save(pStream, TRUE);
//			//pStream->Release();
//			//pPersist->Release();
//			if (hr == 0) 
//			{
//				hr = pStorage.Commit(STGC_DEFAULT);
//			}
//			//pStorage->Release();
//			return hr;
//		}

	
		public bool SaveGraphtoFile(IGraphBuilder pGraph, ushort wszPath)
		{
			//SavetoGraph.class1.SaveGraphFile((IGraphBuilder*)&pGraph, &wszPath);
			int hr = SaveGraphFile(pGraph,wszPath);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
			return true;

		}

		[DllImport("SavetoGraph.dll", ExactSpelling=true) ]
		private static extern int SaveGraphFile(IGraphBuilder pGraph, ushort wszPath);

		// from 'DShowUtil.cpp'
//		public int SeekNextFrame( IMediaSeeking pSeeking, double FPS, long Frame )
//		{
//			// try seeking by frames first
//			//
//			REFERENCE_TIME Pos = 0;
//			long UNITS = 1;
//			if( hr < 0 )
//			{
//				pSeeking.GetCurrentPosition(Pos);
//				Pos++;
//			}
//			else
//			{
//				// couldn't seek by frames, use Frame and FPS to calculate time
//				//
//				Pos = REFERENCE_TIME((double)(Frame * UNITS) / FPS);
//
//				// add a half-frame to seek to middle of the frame
//				//
//				Pos += REFERENCE_TIME((double)UNITS * 0.5 / FPS);
//			}
//
//			hr = pSeeking.SetPositions(Pos, AM_SEEKING_AbsolutePositioning, 
//				NULL, AM_SEEKING_NoPositioning);
//			return hr;
//
//		}

		// from 'DShowUtil.cpp'
		public static int GetPin( IBaseFilter filter, PinDirection directionReq, int num, out IPin foundPin )
		{	
			foundPin = null;	
			IEnumPins pinEnum;	
			int hr = filter.EnumPins( out pinEnum );	
			if( (hr < 0) || (pinEnum == null) )		
				return hr;		

			IPin[] pins = new IPin[1];	
			int f;	
			PinDirection dir;	
			do	
			{		
				hr = pinEnum.Next( 1, pins, out f );		
				if( (hr != 0) || (pins[0] == null) )			
					break;		
				dir = (PinDirection) 3;		
				hr = pins[0].QueryDirection( out dir );		
				if( (hr == 0) && (dir == directionReq) )		
				{			
					if( num == 0 )			
					{				
						foundPin = pins[0];				
						pins[0] = null;				
						break;			
					}			
					num--;		
				}		
				Marshal.ReleaseComObject( pins[0] ); pins[0] = null;	
			}	
			while( hr == 0 );	
			Marshal.ReleaseComObject( pinEnum ); 
			pinEnum = null;	
			return hr;
		}

		public static int NextPin( IBaseFilter filter, PinDirection directionReq, out int nextPin)
		{		
			nextPin = 0;
			IEnumPins pinEnum;
			int hr = filter.EnumPins( out pinEnum );	
			if( (hr < 0) || (pinEnum == null) )		
				return hr;		

			IPin[] pins = new IPin[1];	
			int f;	
			PinDirection dir;	
			do	
			{		
				hr = pinEnum.Next( 1, pins, out f );		
				if( (hr != 0) || (pins[0] == null) )			
					break;		
				dir = (PinDirection) 3;		
				hr = pins[0].QueryDirection( out dir );		
				if( (hr == 0) && (dir == directionReq) )		
				{			
					nextPin++;
				}		
				Marshal.ReleaseComObject( pins[0] ); pins[0] = null;	
			}	
			while( hr == 0 );	
			Marshal.ReleaseComObject( pinEnum ); 
			pinEnum = null;	
			return hr;
		}

		public static int EnumFilters (IFilterGraph pGraph) 
		{
			IEnumFilters pEnum = null;
			IBaseFilter[] pFilter = new IBaseFilter[1];
			int cFetched;

			try
			{
				int hr = pGraph.EnumFilters(out pEnum);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				do	
				{
					hr = pEnum.Next(1, pFilter, out cFetched);
					if( (hr != 0) || (pFilter[0] == null) )			
						break;	

					FilterInfo fInfo;
					hr = pFilter[0].QueryFilterInfo(out fInfo);
					if( hr < 0 )
					{
						Console.WriteLine("Could not get the filter info");
						continue;  // Maybe the next one will work.
					}
													
					Console.WriteLine("Filter Name: " + fInfo.achName );
				

					// The FILTER_INFO structure holds a pointer to the Filter Graph
					// Manager, with a reference count that must be released.
					if (fInfo.pUnk != null)
					{
						Marshal.ReleaseComObject(fInfo.pUnk);
					}
					Marshal.ReleaseComObject(pFilter[0]); pFilter[0]= null;
				}	
				while( hr == 0 );

				Marshal.ReleaseComObject(pEnum); pEnum=null;
				return hr;
			}	
			catch( Exception ee )
			{
				Trace.WriteLine("Could not get the filters info" + ee.Message);
				return -1; 
			}
			
		}

		/// <summary> 
		///  Free the nested structures and release any 
		///  COM objects within an AMMediaType struct.
		/// </summary>
		public static void FreeAMMediaType(AMMediaType mediaType)
		{
			if ( mediaType.formatSize != 0 )
				Marshal.FreeCoTaskMem( mediaType.formatPtr );
			if ( mediaType.unkPtr != IntPtr.Zero ) 
				Marshal.Release( mediaType.unkPtr );
			mediaType.formatSize = 0;
			mediaType.formatPtr = IntPtr.Zero;
			mediaType.unkPtr = IntPtr.Zero;
		}

		[DllImport("olepro32.dll", CharSet=CharSet.Unicode, ExactSpelling=true) ]
		private static extern int OleCreatePropertyFrame( IntPtr hwndOwner, int x, int y,
			string lpszCaption, int cObjects,
			[In, MarshalAs(UnmanagedType.Interface)] ref object ppUnk,
			int cPages,	IntPtr pPageClsID, int lcid, int dwReserved, IntPtr pvReserved );
	}


// ---------------------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
public struct DsPOINT		// POINT
{
	public int		X;
	public int		Y;
}


// ---------------------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
public struct DsRECT		// RECT
{
	public int		Left;
	public int		Top;
	public int		Right;
	public int		Bottom;
}


// ---------------------------------------------------------------------------------------

	[StructLayout(LayoutKind.Sequential, Pack=2), ComVisible(false)]
	public struct BitmapInfoHeader
	{
		public int      Size;
		public int      Width;
		public int      Height;
		public short    Planes;
		public short    BitCount;
		public int      Compression;
		public int      ImageSize;
		public int      XPelsPerMeter;
		public int      YPelsPerMeter;
		public int      ClrUsed;
		public int      ClrImportant;
	}




// ---------------------------------------------------------------------------------------

		[ComVisible(false)]
	public class DsROT
	{
			// Adds a DirectShow filter graph to the Running Object Table,
			// allowing GraphEdit to "spy" on a remote filter graph.
		public static bool AddGraphToRot( object graph, out int cookie )
		{
			cookie = 0;
			int hr = 0;
			UCOMIRunningObjectTable rot = null;
			UCOMIMoniker mk = null;
			try {
				hr = GetRunningObjectTable( 0, out rot );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				int id = GetCurrentProcessId();
				IntPtr iuPtr = Marshal.GetIUnknownForObject( graph );
				int iuInt = (int) iuPtr;
				Marshal.Release( iuPtr );
				string item = string.Format( "FilterGraph {0} pid {1}", iuInt.ToString("x8"), id.ToString("x8") );
				hr = CreateItemMoniker( "!", item, out mk );
				// Use the ROTFLAGS_REGISTRATIONKEEPSALIVE to ensure a strong reference
				// to the object.  Using this flag will cause the object to remain
				// registered until it is explicitly revoked with the Revoke() method.
				//
				// Not using this flag means that if GraphEdit remotely connects
				// to this graph and then GraphEdit exits, this object registration 
				// will be deleted, causing future attempts by GraphEdit to fail until
				// this application is restarted or until the graph is registered again.
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				
				rot.Register( ROTFLAGS_REGISTRATIONKEEPSALIVE, graph, mk, out cookie );
				return true;
			}
			catch( Exception )
			{
				return false;
			}
			finally
			{
				if( mk != null )
					Marshal.ReleaseComObject( mk ); mk = null;
				if( rot != null )
					Marshal.ReleaseComObject( rot ); rot = null;
			}
		}

			// Removes a filter graph from the Running Object Table
		public static bool RemoveGraphFromRot( ref int cookie )
		{
			UCOMIRunningObjectTable rot = null;
			try {
				int hr = GetRunningObjectTable( 0, out rot );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				rot.Revoke( cookie );
				cookie = 0;
				return true;
			}
			catch( Exception )
			{
				return false;
			}
			finally
			{
				if( rot != null )
					Marshal.ReleaseComObject( rot ); rot = null;
			}
		}

		private const int ROTFLAGS_REGISTRATIONKEEPSALIVE	= 1;

		[DllImport("ole32.dll", ExactSpelling=true) ]
		private static extern int GetRunningObjectTable( int r,
			out UCOMIRunningObjectTable pprot );

		[DllImport("ole32.dll", CharSet=CharSet.Unicode, ExactSpelling=true) ]
		private static extern int CreateItemMoniker( string delim,
			string item, out UCOMIMoniker ppmk );

//		[DllImport("ole32.dll", CharSet=CharSet.Unicode, ExactSpelling=true) ]
//		private static extern int OpenStream( string delim,
//			string item, out UCOMIStream ppostr );

		[DllImport("kernel32.dll", ExactSpelling=true) ]
		private static extern int GetCurrentProcessId();
	}


// ---------------------------------- ocidl.idl ------------------------------------------------

	[ComVisible(true), ComImport,
	Guid("B196B28B-BAB4-101A-B69C-00AA00341D07"),
	InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
public interface ISpecifyPropertyPages
{
		[PreserveSig]
	int GetPages( out DsCAUUID pPages );
}

[StructLayout(LayoutKind.Sequential), ComVisible(false)]
public struct DsCAUUID		// CAUUID
{
	public int		cElems;
	public IntPtr	pElems;
}
// ---------------------------------------------------------------------------------------


	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
public class DsOptInt64
{
	public DsOptInt64( long Value )
	{
		this.Value = Value;
	}
	public long		Value;
}


	[StructLayout(LayoutKind.Sequential), ComVisible(false)]
public class DsOptIntPtr
{
	public IntPtr	Pointer;
}



} // namespace DShowNET
