using System;
using System.Collections;
using System.Runtime.Serialization;

namespace MT.DataStructures
{
	/// <summary>
	/// A Featurepoint Frame contains all Featurepoints of one single Frame
	/// It also contains a FeaturePointFrameColor Class for each available eColorType
	/// which will be given to a tracker.
	/// </summary>
	public class FeaturePointFrame: FeaturePointFrameColor 
	{
		private FeaturePointFrameColor	_red;
		private FeaturePointFrameColor	_green;
		private FeaturePointFrameColor	_blue;

		public override int Sync
		{
			get {	return base._Sync; }
			set {	base._Sync = value; 
					_red.Sync = value;
					_green.Sync = value;
					_blue.Sync = value;
				}
		}


		public FeaturePointFrameColor Pred
		{
			get { return _red; }
		}

		public FeaturePointFrameColor Pgreen
		{
			get { return _green; }
		}

		public FeaturePointFrameColor Pblue
		{
			get { return _blue; }
		}

		public FeaturePointFrame(int FrameNumber, string cid): base(eColorCode.none,FrameNumber, cid)
		{
			_red = new FeaturePointFrameColor(eColorCode.R,FrameNumber,cid);
			_green = new FeaturePointFrameColor(eColorCode.G,FrameNumber,cid);
			_blue = new FeaturePointFrameColor(eColorCode.B,FrameNumber,cid);


		}
		public FeaturePointFrame(string input)
		{	

			//Temporary Array to split out Headerdata from message
			string[] header = new string[5];

			
			header = input.Split(new char[1] {';'},5);

			//Extract Framenumber and CameraID to initialize FPFrameColors
			base._fid = int.Parse(header[0]);
			base._cid = header[1];


			_red = new FeaturePointFrameColor(eColorCode.R,base._fid, _cid);
			_green = new FeaturePointFrameColor(eColorCode.G,base._fid, _cid);
			_blue = new FeaturePointFrameColor(eColorCode.B,base._fid, _cid);

			int FPcount = int.Parse(header[2]);
			this.Sync = int.Parse(header[3]);
			
			SortedList Debug = new SortedList();
			if (FPcount>0)
			{
				//Temporary Arrary to split out Featurepoints from Message
				string[] FPs = new string[FPcount];

				FPs = header[4].Split(new char[1] {';'},FPcount);

				FeaturePoint P;

				for (int i =0; i< FPcount;i++)
				{
					P = new FeaturePoint(FPs[i]);
//					try
//					{
//						Debug.Add(P.tid, P);
//					}
//					catch
//					{
//						Console.WriteLine("very bad");
//					}
					Add(P);
				}
			}

			//			_X = info.GetSingle("_X");
			//			_Y = info.GetSingle("_Y");
			//
			//			_tid = info.GetInt32("_tid");
			//			_fid = info.GetInt32("_fid");
			//			_type = (eFPType)info.GetValue("_type",_type.GetType());
			//			_C = (eColorCode)info.GetValue("_C",_C.GetType());
		}

		public override void Add(FeaturePoint FP)
		{
			//Add FP to the base collection
			base.Add(FP);

			switch (FP.Color)
			{
				case eColorCode.R:
					_red.Add(FP);
					break;
				
				case eColorCode.G:
					_green.Add(FP);
					break;
			
				case eColorCode.B:
					_blue.Add(FP);
					break;
			}
		}
		public void Add(FeaturePointFrameColor frameColor)
		{
			foreach(FeaturePoint FP in frameColor.Values)
			{
				Add(FP);
			}
		}


		public override string ToString()
		{
				
			//Store Class Information
			string r = "";
			
			r += base._fid + ";";
			r += base._cid + ";";	
			r += base.Count.ToString()+ ";";
			r += base._Sync.ToString();

			//Store Collection Members
			foreach(FeaturePoint FP in base.Values)
			{
				r += ";" + FP.ToString();
			}

			return r;

		}

	}
}
