using System;
using System.Collections;
using MT.MathBase;
using OpenGL;

namespace MT.DataStructures
{
	/// <summary>
	/// Summary description for clsBPConnector.
	/// </summary>
	public class clsBPConnector
	{
		string			_name;
		clsBPStream		_BP1a = new clsBPStream("",eColorCode.none);
		clsBPStream		_BP2a = new clsBPStream("",eColorCode.none);
		clsBPStream		_BP1b = new clsBPStream("",eColorCode.none);
		clsBPStream		_BP2b = new clsBPStream("",eColorCode.none);
		int				_BPs;
		

		//Each Bodyconnector has a given maximum Elasticity.
		//This is the maximum allowed discrepance in percent of the baselength.
		//which allows us to check
		//the quality of its Base clsBPStreams. If the elasticity maximum is exceeded
		//the ConnectorQuality Function will return a value greater than 1
		//This supports the TUK Controller in determining if a given BP position is realistical
		//p.E if a body connector simulates a bone,
		//its base elasticity is zero, including the fact that the exoscelett with the LEDs is placed
		//not on bones but on the movable skin, it results in elasticity values around 10%
		
		float			_elasticity;
		float			_baselength=0;
		float			_mh=1;
		float			_ml=-1;

		public string	Name
		{
			get { return _name; }
		}

		public float	Elasticity
		{
			get { return _elasticity; }
		}

		public clsBPConnector(string name, clsBPStream BP1, clsBPStream BP2, float elasticity)
		{
			_name = name;
			_BP1a = BP1;
			_BP2a = BP2;
			_BPs = 2;
			_elasticity = elasticity;
			
			//register on all connected clsBPStreams
			_BP1a.newConnector(this);
			_BP2a.newConnector(this);
		}
		public clsBPConnector(string name, clsBPStream BP1, clsBPStream BP2a, clsBPStream BP2b, float elasticity)
		{
			_name = name;
			_BP1a = BP1;
			_BP2a = BP2a;
			_BP2b = BP2b;
			_BPs = 3;
			_elasticity = elasticity;
				
			//register on all connected clsBPStreams
			_BP1a.newConnector(this);
			_BP2a.newConnector(this);
			_BP2b.newConnector(this);

		}
		public clsBPConnector(string name, clsBPStream BP1a, clsBPStream BP1b, clsBPStream BP2a, clsBPStream BP2b, float elasticity)
		{
			_name = name;
			_BP1a = BP1a;
			_BP1b = BP1b;
			_BP2a = BP2a;
			_BP2b = BP2b;
			_BPs = 4;
			_elasticity = elasticity;
			
			//register on all connected clsBPStreams
			_BP1a.newConnector(this);
			_BP1b.newConnector(this);
			_BP2a.newConnector(this);
			_BP2b.newConnector(this);

		}


		public clsBPConnector(string Input, SortedList _clsBPStreams)
		{
			string[] Header;

			Header = Input.Split(";".ToCharArray(),4);

			_name = Header[0];
			_elasticity = float.Parse(Header[1]);
			_BPs = int.Parse(Header[2]);

			string[] BPNames;

			BPNames= Header[3].Split(";".ToCharArray(), 4);

			switch(_BPs)
			{
				case 2:
					_BP1a = (clsBPStream)_clsBPStreams[BPNames[0]];
					_BP2a = (clsBPStream)_clsBPStreams[BPNames[2]];
					break;
				case 3:
					_BP1a = (clsBPStream)_clsBPStreams[BPNames[0]];
					_BP2a = (clsBPStream)_clsBPStreams[BPNames[2]];
					_BP2b = (clsBPStream)_clsBPStreams[BPNames[3]];
					break;
				case 4:
					_BP1a = (clsBPStream)_clsBPStreams[BPNames[0]];
					_BP1b = (clsBPStream)_clsBPStreams[BPNames[1]];
					_BP2a = (clsBPStream)_clsBPStreams[BPNames[2]];
					_BP2b = (clsBPStream)_clsBPStreams[BPNames[3]];
					break;
			}
		}

		public override string ToString()
		{

			string splitchar = ";";
			return string.Concat(_name, splitchar, 
								 _elasticity.ToString(), splitchar, 
								 _BPs.ToString(), splitchar, 
				                 _BP1a.Name, splitchar, 
				                 _BP1b.Name, splitchar,
				                 _BP2a.Name, splitchar, 
				                 _BP2b.Name);
		}


		/// <summary>
		/// When called, calculates the current length of the connector
		/// </summary>
		/// <param name="FrameID">Frame ID to use for length calculation</param>
		/// <returns>returns a value > 0 if calculation was successful, else it returns -1</returns>
		public float calcLength(int FrameID)
		{
			vector3f vec1;
			vector3f vec2;

			vector3f a;
			vector3f b;
			switch(_BPs)
			{
				case 2:
					vec1 = _BP1a[FrameID]/1000.0f;
					if (vec1.Type == eVecType.Invalid) {return -1;}
					vec2 = _BP2a[FrameID]/1000.0f;
					if (vec2.Type == eVecType.Invalid) {return -1;}
					break;
				case 3:
					vec1 = _BP1a[FrameID]/1000.0f;
					if (vec1.Type == eVecType.Invalid) {return -1;}
					a = _BP2a[FrameID]/1000.0f;
					if (a.Type == eVecType.Invalid) {return -1;}
					b = _BP2b[FrameID]/1000.0f;
					if (b.Type == eVecType.Invalid) {return -1;}
					vec2 = a + (b-a)/2;
					break;
				case 4:
					a = _BP1a[FrameID]/1000.0f;
					if (a.Type == eVecType.Invalid) {return -1;}
					b = _BP1b[FrameID]/1000.0f;
					if (b.Type == eVecType.Invalid) {return -1;}
					vec1 = a + (b-a)/2;
					a = _BP2a[FrameID]/1000.0f;
					if (a.Type == eVecType.Invalid) {return -1;}
					b = _BP2b[FrameID]/1000.0f;
					if (b.Type == eVecType.Invalid) {return -1;}
					vec2 = a + (b-a)/2;
					break;
				default:
					vec1 = _BP1a[FrameID]/1000.0f;
					if (vec1.Type == eVecType.Invalid) {return -1;}
					vec2 = _BP2a[FrameID]/1000.0f;
					if (vec2.Type == eVecType.Invalid) {return -1;}
					break;



					//				case 2:
					//					vec1 = _BP1a.get3DCoord(FrameID)/1000.0f;
					//					if (vec1.Type == eVecType.Invalid) {return -1;}
					//					vec2 = _BP2a.get3DCoord(FrameID)/1000.0f;
					//					if (vec2.Type == eVecType.Invalid) {return -1;}
					//					break;
					//				case 3:
					//					vec1 = _BP1a.get3DCoord(FrameID)/1000.0f;
					//					if (vec1.Type == eVecType.Invalid) {return -1;}
					//					a = _BP2a.get3DCoord(FrameID)/1000.0f;
					//					if (a.Type == eVecType.Invalid) {return -1;}
					//					b = _BP2b.get3DCoord(FrameID)/1000.0f;
					//					if (b.Type == eVecType.Invalid) {return -1;}
					//					vec2 = a + (b-a)/2;
					//					break;
					//				case 4:
					//					a = _BP1a.get3DCoord(FrameID)/1000.0f;
					//					if (a.Type == eVecType.Invalid) {return -1;}
					//					b = _BP1b.get3DCoord(FrameID)/1000.0f;
					//					if (b.Type == eVecType.Invalid) {return -1;}
					//					vec1 = a + (b-a)/2;
					//					a = _BP2a.get3DCoord(FrameID)/1000.0f;
					//					if (a.Type == eVecType.Invalid) {return -1;}
					//					b = _BP2b.get3DCoord(FrameID)/1000.0f;
					//					if (b.Type == eVecType.Invalid) {return -1;}
					//					vec2 = a + (b-a)/2;
					//					break;
					//				default:
					//					vec1 = _BP1a.get3DCoord(FrameID)/1000.0f;
					//					if (vec1.Type == eVecType.Invalid) {return -1;}
					//					vec2 = _BP2a.get3DCoord(FrameID)/1000.0f;
					//					if (vec2.Type == eVecType.Invalid) {return -1;}
					//					break;
			}

			return (vec1-vec2).length;
		}
		
		private void setBaseLength(int frameID)
		{
			//Calculate BaseLength with the first frame data, 
			//throw an exception if this is not possible because of 
			//incomplete BP-3D data
			float length = calcLength(0);
			if (length>=0) {_baselength=length;} 
			//else {throw new Exception("BodyConnector->Konstruktor: Not every intconnected clsBPStream had a valid 3D Position when the BodyConnector was created.");}
			
			try
			{
				//				_mh = 1/((1+_elasticity)*_baselength);
				//				_ml = -1/(_baselength/(1+_elasticity));
				_mh =  1/(_baselength*_elasticity);
				_ml = -1/(_baselength*(1-1/_elasticity));

			}
			catch{}

		}

		/// <summary>
		/// Calculates a Quality value. 
		/// A Value over 1 indicates that the base elasticity has been exceeded.
		/// A Value of -1 indicates that no BoneQuality has been calculated because of missing BP-3D Data.
		/// Values between 0 and 1 are better, the closer to 0 they are.
		/// </summary>
		/// <param name="FrameID">Frame to use for calculation</param>
		/// <returns>Quality Value, the lower the better ;)</returns>
		public float ConnectorQuality(int FrameID)
		{
			float length = calcLength(FrameID);
			if (length <0) { return -1;}
			else { return (Math.Max(_ml*length-_ml*_baselength,_mh*length-_mh*_baselength));}
		}

		public void Paint(int FrameID)
		{	
			vector3f vec1;
			vector3f vec2;

			vector3f a;
			vector3f b;
			switch(_BPs)
			{
				case 2:
					vec1 = _BP1a[FrameID]/1000.0f;
					if (vec1.Type == eVecType.Invalid) {return;}
					vec2 = _BP2a[FrameID]/1000.0f;
					if (vec2.Type == eVecType.Invalid) {return;}
					break;
				case 3:
					vec1 = _BP1a[FrameID]/1000.0f;
					if (vec1.Type == eVecType.Invalid) {return;}
					a = _BP2a[FrameID]/1000.0f;
					if (a.Type == eVecType.Invalid) {return;}
					b = _BP2b[FrameID]/1000.0f;
					if (b.Type == eVecType.Invalid) {return;}
					vec2 = a + (b-a)/2;
					break;
				case 4:
					a = _BP1a[FrameID]/1000.0f;
					if (a.Type == eVecType.Invalid) {return;}
					b = _BP1b[FrameID]/1000.0f;
					if (b.Type == eVecType.Invalid) {return;}
					vec1 = a + (b-a)/2;
					a = _BP2a[FrameID]/1000.0f;
					if (a.Type == eVecType.Invalid) {return;}
					b = _BP2b[FrameID]/1000.0f;
					if (b.Type == eVecType.Invalid) {return;}
					vec2 = a + (b-a)/2;
					break;
				default:
					vec1 = _BP1a[FrameID]/1000.0f;
					if (vec1.Type == eVecType.Invalid) {return;}
					vec2 = _BP2a[FrameID]/1000.0f;
					if (vec2.Type == eVecType.Invalid) {return;}
					break;
			}


			WGL.drawCylinder(0.03f,vec1,vec2,16,new vector3f(0,0,0));
			
		}
	}
}
