using System;
using System.IO;
using System.Collections;

namespace MT.DataStructures
{
	/// <summary>
	/// Summary description for clsBPData.
	/// </summary>
	public class clsBPData: clsBPDataColor
	{
		public event EventHandler Loaded;		//Raised after a load call

		private clsBPDataColor	_red   = new clsBPDataColor(eColorCode.R);
		private clsBPDataColor	_green = new clsBPDataColor(eColorCode.G);
		private clsBPDataColor	_blue  = new clsBPDataColor(eColorCode.B);

		private Hashtable	_Connectors = new Hashtable();


		public int FirstFrame
		{
			get
			{
				int result =-1;
				foreach (clsBPStream BP in this.Values)
				{
					if (result == -1)
						result = (int)BP.GetKey(0);
					else
						result = Math.Min(result,(int)BP.GetKey(0));
				}

				return result;
			}
		}

		public int LastFrame
		{
			get
			{
				int result =-1;
				foreach (clsBPStream BP in this.Values)
				{
					if (result == -1)
						result = (int)BP.GetKey(BP.Count-1);
					else
						result = Math.Max(result,(int)BP.GetKey(BP.Count-1));
				}

				return result;

			}
		}

		public clsBPData(): base(eColorCode.none)
		{
		}

		public clsBPData(string Filename): base(eColorCode.none)
		{
			Load(Filename);
		}

		public void Load(string Filename)
		{
			Clear();

			StreamReader input = new StreamReader(Filename);	

			//int	NrOfFrames = int.Parse(input.ReadLine());
			int NrOfStreams = int.Parse(input.ReadLine());
			int NrOfConnectors = int.Parse(input.ReadLine());
			

			for(int i = 0; i<NrOfStreams; i++)
			{
				Add(new clsBPStream(input.ReadLine()));
			}

			clsBPConnector BPConnector;

			for(int i = 0; i<NrOfConnectors; i++)
			{
				BPConnector = new clsBPConnector(input.ReadLine(),this);
				_Connectors.Add(BPConnector.Name,BPConnector);
			}

			input.Close();

			OnLoaded();	//Raise Event
		}

		private void OnLoaded()
		{
			if (Loaded != null)
				Loaded(this, new EventArgs());
		}

		public override void Clear()
		{
			base.Clear();

			_Connectors.Clear();

			_red.Clear();
			_green.Clear();
			_blue.Clear();
		}


		public void Save(string Filename)
		{
			StreamWriter output = new StreamWriter(Filename);

			output.WriteLine(base.Count.ToString());
			output.WriteLine(_Connectors.Count.ToString());

			foreach(clsBPStream BPStream in base.Values)
			{
				output.WriteLine(BPStream.ToString());
			}

			foreach(clsBPConnector BPConnector in _Connectors.Values)
			{
				output.WriteLine(BPConnector.ToString());
			}
		
			output.Close();
		}

		public clsBPDataColor DataRed
		{
			get { return _red; }
		}

		public clsBPDataColor DataGreen
		{
			get { return _green; }
		}

		public clsBPDataColor DataBlue
		{
			get { return _blue; }
		}

		public override void Add(clsBPStream BPStream)
		{
			base.Add (BPStream);

			switch (BPStream.Color)
			{
				case eColorCode.R:
					_red.Add(BPStream);
					break;
				
				case eColorCode.G:
					_green.Add(BPStream);
					break;
			
				case eColorCode.B:
					_blue.Add(BPStream);
					break;
			}

		}

		public void Add(clsBPConnector BPConnector)
		{	_Connectors.Add(BPConnector.Name,BPConnector);
		}	

		public override void Paint(int frameID)
		{
			_red.Paint(frameID);
			_green.Paint(frameID);
			_blue.Paint(frameID);

			foreach(clsBPConnector BPConnector in _Connectors.Values)
			{
				BPConnector.Paint(frameID);
			}
		}

	}
}
