using System;
using System.Collections;
using MT.MathBase;
using OpenGL;



namespace MT.DataStructures
{
	/// <summary>
	/// Summary description for clsBPStream.
	/// </summary>
	public class clsBPStream: SortedList
	{
		private string _Name;
		private eColorCode _Color;

		//Adjacent Bodyconnectorlist
		//this list is used to consult the Bodyconnectors about
		//the propability of a 3D tracking under consideration
		//of bone physics (p.E. an ellbow is always at the same 
		//distance from a hand.
		Hashtable			_BodyConnectors= new Hashtable();


		/// <summary>
		/// Gets Name of BPStream
		/// </summary>
		public string Name
		{
			get { return _Name; }
		}

		/// <summary>
		/// Gets Color of BPStream
		/// </summary>
		public eColorCode Color
		{	get { return _Color; }	 }


		public clsBPStream(string Name, eColorCode Color)
		{
			_Name = Name;
			_Color = Color;

		}
		public clsBPStream(string Name, eColorCode Color, float x, float y, float z)
		{
			_Name = Name;
			_Color = Color;

			Add(new BodyPoint(x,y,z,Color,0));
		}

		public clsBPStream(string Input)
		{
			string[] Header;

			Header = Input.Split(new char[] {char.Parse(";")},4);

			_Name = Header[0];
			_Color = (eColorCode)int.Parse(Header[1]);
			int FrameCount = int.Parse(Header[2]);

			if(FrameCount>0)
			{
				string[] Frames;

				Frames= Header[3].Split(";".ToCharArray(), FrameCount);

				foreach(string Frame in Frames)
				{
					Add(new BodyPoint(Frame,_Color));
				}
			}
		}


		public override string ToString()
		{
			string splitchar = ";";
			string result = string.Concat(_Name,splitchar,((int)_Color).ToString(),splitchar, base.Count.ToString());
			foreach(BodyPoint BP in base.Values)
			{
				result += splitchar + BP.ToString();
			}

			return result;
		}


		public void		newConnector(clsBPConnector BC)
		{
			_BodyConnectors.Add(BC.Name,BC);
		}

		//Assign Color
		public void Add(BodyPoint BP)
		{
			//Verify correct Color
			if (BP.Color == eColorCode.none)
				BP.Color = _Color;
	
			//Verify correct frame ID
			if (BP.fid == -1)
				BP.fid = this.Count;

			base.Add (BP.fid, BP);
		}
	
		public override void Add(object key, object value)
		{
			BodyPoint BP = value as BodyPoint;

			if (BP!=null)
				base.Add (BP.fid, BP);
		}


		public new BodyPoint this[object key]
		{
			get
			{ if (base.Contains(key))
				return (BodyPoint)base[key];
				else
				  return new BodyPoint(int.Parse(key.ToString()));

			}
			set
			{
				base[key] = value;
			}
		}


		public void Paint(int FrameID)
		{
			GL.glPushMatrix();

			//Draw Bodypoint
			if (this.ContainsKey(FrameID))
				this[FrameID].Paint();

			GL.glPopMatrix();
		}


	}
}
