using System;
using System.Runtime.Serialization;
using MT.MathBase;
using OpenGL;

namespace MT.DataStructures
{
	/// <summary>
	/// Point Class, stores a 3-Tupel for the X, Y and Z Axis
	/// and the color of the Bodypoint
	/// </summary>
	public class BodyPoint: vector3f, IComparable //, ISerializable
	{
		private eColorCode	_C = eColorCode.none;
	
		private int			_fid = -1;

		/// <summary>
		/// Returns Frame ID of this Featurepoint
		/// </summary>
		public	int			fid
		{
			get { return _fid;  }
			set { _fid = value; }
		}

//		public	BodyPoint() {}
//		public	BodyPoint(float X, float Y, float Z) : base(X,Y,Z)
//		{
//			_C = eColorCode.none;
//	
//		}
		public BodyPoint(int fid)
		{
			_fid = fid;
		}

		public BodyPoint(float X, float Y, float Z, int fid): base(X,Y,Z)
		{
			_fid = fid;
		}
		public 	BodyPoint(float X, float Y, float Z, eColorCode Color, int fid) : base(X,Y,Z)
		{
			_C = Color;
			_fid = fid;
		}
		public BodyPoint(string Input, eColorCode Color)
		{
			_C = Color;
			string[] Data;
			Data = Input.Split(":".ToCharArray(),5);

			_fid = int.Parse(Data[0]);
			_X = float.Parse(Data[1]);
			_Y = float.Parse(Data[2]);
			_Z = float.Parse(Data[3]);
			_Type = (eVecType)int.Parse(Data[4]);
		}


		public	eColorCode	Color
		{
			get { return _C;  }
			set { _C = value; }
		}


		public override string ToString()
		{
			string splitchar = ":";
			return string.Concat(new string[] {_fid.ToString(), splitchar,
											   _X.ToString() ,splitchar, 
											   _Y.ToString(), splitchar, 
											   _Z.ToString(), splitchar,
											   ((int)Type).ToString()}
								);
		}

		public void Paint()
		{

			if (base.Type==eVecType.Valid) 
			{
				GL.glTranslatef( base.X/1000, base.Y/1000, base.Z/1000 );
				WGL.drawSphere(0.06f, 12, 12); //Pingpongball
			}

		}


		#region IComparable Members

		public int CompareTo(object obj)
		{

			// TODO:  Add BodyPointComparer.Compare implementation
			BodyPoint P1 = this;
			BodyPoint P2 = (BodyPoint)obj;

			if (P1.X == P2.X && P1.Y == P2.Y && P1.Z ==P2.Z)
				return 0;
			else
			{
				if(P1.X*10000000000 + P1.Y*100000 + P1.Z > P2.X*10000000000 + P2.Y*100000 + P2.Z)
					return 1;
				else
					return -1;
			}
		
		}

		#endregion


	}

}
