using System;
using System.Runtime.InteropServices;
using System.Drawing.Imaging;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;
using DShowNET;
using MT.FPSource;

namespace GraphController
{

	public class SampleGrabber: ISampleGrabberCB, IPictureGrabber
	{
		/// <summary> buffer for bitmap data. </summary>
		private	byte[]					_savedArray;

		/// <summary> structure describing the bitmap to grab. </summary>
		private	VideoInfoHeader			_videoInfoHeader;
		private	bool					_captured;
		private	bool					_captureDone;

		/// <summary> capture graph builder interface. </summary>
		private ISampleGrabber			_sampGrabber;
//		private PictureBox				_grabImage;

		/// <summary> event when callback has finished (ISampleGrabberCB.BufferCB). </summary>
//		public delegate void CaptureDone();
		
		public delegate void EmptyDelegate();

		public event EmptyDelegate BeginCapture;
		public event EmptyDelegate EndCapture;



		public SampleGrabber(ISampleGrabber sampGrabber) //, PictureBox grabImage)
		{
//			_grabImage = grabImage;
			_sampGrabber = sampGrabber;

			Initialize();
		}


		public void Initialize()
		{
			//*************************************************
			//SampleGrabber Final
			//*************************************************
		
			_savedArray = null;

			AMMediaType media = new AMMediaType();
			int hr = _sampGrabber.GetConnectedMediaType( media );
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
			if( (media.formatType != FormatType.VideoInfo) || (media.formatPtr == IntPtr.Zero) )
				throw new NotSupportedException( "Unknown Grabber Media Format" );
			
			_videoInfoHeader = (VideoInfoHeader) Marshal.PtrToStructure( media.formatPtr, typeof(VideoInfoHeader) );
			Marshal.FreeCoTaskMem( media.formatPtr ); media.formatPtr = IntPtr.Zero;

			hr = _sampGrabber.SetBufferSamples( false );
			if( hr == 0 )
				hr = _sampGrabber.SetOneShot( false );
			if( hr == 0 )
				hr = _sampGrabber.SetCallback( null, 0 );
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );

			if ( media != null )
				DsUtils.FreeAMMediaType( media ); media = null;
			//*************************************************
			// END SampleGrabber Final 
			//*************************************************

		}

		/// <summary> Destructor. Dispose of resources. </summary>
		~SampleGrabber()
		{
			//Dispose();
		}

		public Bitmap  GrabImage()
		{
			BeginCapture();

			Console.WriteLine( "!!BTN: toolBarBtnGrab" );

			if( _savedArray == null )
			{
				int size = _videoInfoHeader.BmiHeader.ImageSize;
				if( (size < 1000) || (size > 16000000) )
					return null;
				_savedArray = new byte[ size + 64000 ];
			}

//			Image old = _grabImage.Image;
//			_grabImage.Image = null;
//			if( old != null )
//				old.Dispose();

			_captured = false;
			int hr = _sampGrabber.SetCallback(this, 1 );

			//Block Thread until the capture process has finished.
			while(!_captureDone)
				System.Threading.Thread.Sleep(50);

			return OnCaptureDone();


		}

		int ISampleGrabberCB.SampleCB( double SampleTime, IMediaSample pSample )
		{
			Console.WriteLine( "!!CB: ISampleGrabberCB.SampleCB" );
			return 0;
		}

		/// <summary> buffer callback, COULD BE FROM FOREIGN THREAD. </summary>
		int ISampleGrabberCB.BufferCB( double SampleTime, IntPtr pBuffer, int BufferLen )
		{
			if( (_captured || _savedArray == null) )
			{
				Console.WriteLine( "!!CB: ISampleGrabberCB.BufferCB" );
				return 0;
			}

			_captured = true;

			Console.WriteLine( "!!CB: ISampleGrabberCB.BufferCB  !GRAB! size = " + BufferLen.ToString() );

			if( (pBuffer != IntPtr.Zero) && (BufferLen > 1000) && (BufferLen <= _savedArray.Length) )
				Marshal.Copy( pBuffer, _savedArray, 0, BufferLen );
			else
				Console.WriteLine( "    !!!GRAB! failed " );
			
//			_grabImage.Invoke( new EmptyDelegate( OnCaptureDone ) );
			_captureDone = true;
		
			return 0;
		}

		private Bitmap OnCaptureDone()
		{
			Bitmap r = null;
			Console.WriteLine( "!!DLG: OnCaptureDone" );
			try 
			{	
				//Reset CaptureDone Flag
				_captureDone = false;

				//toolBarBtnGrab.Enabled = true;
				if( _sampGrabber == null )
					return null;
				
				int hr = _sampGrabber.SetCallback( null, 0 );

				int w = _videoInfoHeader.BmiHeader.Width;
				int h = _videoInfoHeader.BmiHeader.Height;
				if( ((w & 0x03) != 0) || (w < 32) || (w > 4096) || (h < 32) || (h > 4096) )
					return null;
				int stride = w * 3;

				GCHandle handle = GCHandle.Alloc( _savedArray, GCHandleType.Pinned );
				int scan0 = (int) handle.AddrOfPinnedObject();
				scan0 += (h - 1) * stride;
				Bitmap b = new Bitmap( w, h, -stride, PixelFormat.Format24bppRgb, (IntPtr) scan0 );
				handle.Free();
				_savedArray = null;

//				NewPicture.BeginInvoke(new Bitmap(b),null,null);
				r = b;

				EndCapture();

//				Image old = _grabImage.Image;
//				_grabImage.Image = b;
//				if( old != null )
//					old.Dispose();
			}
			catch( Exception ee )
			{
				Console.WriteLine("Could not grab picture\r\n" + ee.Message, "DirectShow.NET");
			}

			return r;
		}	


		#region IPictureGrabber Member

		public event NewPictureDelegate NewPicture;

		public Bitmap GrabPicture()
		{
			Bitmap b =GrabImage();
			if (NewPicture!=null) NewPicture(b);

			return GrabImage();
		}

		#endregion

	}

}
