using System;
using System.Runtime.InteropServices;
using System.Drawing.Imaging;
using System.Drawing;
using System.Windows.Forms;
using DShowNET;
using System.Threading;
using MT.FPSource;
namespace GraphController
{


	/// <summary>
	/// Zusammenfassung fr SampleGrabber.
	/// </summary>
	//public delegate void VideoCaptureFrameEventHandler(object sender, VideoCaptureFrameEventArgs e);
	public class VideoGrabber: ISampleGrabberCB, IPictureGrabber
	{
		public 	event NewPictureDelegate NewPicture;
 
		public delegate void HandleVideoFrame(System.Drawing.Bitmap BM);
		//		public event HandleVideoFrame FrameCaptured;
		/// <summary> buffer for bitmap data. </summary>
		private	byte[]					_savedArray;

		/// <summary> structure describing the bitmap to grab. </summary>
		private	VideoInfoHeader			_videoInfoHeader;

		/// <summary> capture graph builder interface. </summary>
		private ISampleGrabber			_sampGrabber;
		//		private FPDetector _fpDetector;

		/// <summary> event when callback has finished (ISampleGrabberCB.BufferCB). </summary>
		private delegate void CaptureDone();


		private bool			_isEnabled = false;

		public bool isEnabled
		{
			get {return _isEnabled;}
		}

		public VideoGrabber(ISampleGrabber sampGrabber)
		{
			_sampGrabber = sampGrabber;
			Initialize();

		}

		public void Initialize()
		{
			lock(this)
			{
				bool wasEnabled = _isEnabled;
	
				int hr;
			
				hr = _sampGrabber.SetBufferSamples( false );
				if( hr == 0 )
					hr = _sampGrabber.SetOneShot( false );
				if( hr == 0 )
					hr = _sampGrabber.SetCallback( null, 0 );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				Disable();

				 _savedArray = null;
				//*************************************************
				//SampleGrabber Final
				//*************************************************
				AMMediaType media = new AMMediaType();
				hr = _sampGrabber.GetConnectedMediaType( media );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				if( (media.formatType != FormatType.VideoInfo) || (media.formatPtr == IntPtr.Zero) )
					throw new NotSupportedException( "Unknown Grabber Media Format" );
			
				_videoInfoHeader = (VideoInfoHeader) Marshal.PtrToStructure( media.formatPtr, typeof(VideoInfoHeader) );
				Marshal.FreeCoTaskMem( media.formatPtr ); media.formatPtr = IntPtr.Zero;

				if( _savedArray == null )
				{
					int size = _videoInfoHeader.BmiHeader.ImageSize;
					if( (size < 1000) || (size > 16000000) )
						return;
					_savedArray = new byte[ size + 64000 ];
				}

				if ( media != null )
					DsUtils.FreeAMMediaType( media ); media = null;
				//*************************************************
				// END SampleGrabber Final 
				//*************************************************

				if(wasEnabled)
					Enable();

			}
		}

		/// <summary> Destructor. Dispose of resources. </summary>
		~VideoGrabber()
		{
			//Dispose();
		}

		public void Enable()
		{

			if( _sampGrabber == null )
				return;
			int hr = _sampGrabber.SetCallback( this, 1 );
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );

			_isEnabled = true;
		}




		public void Disable()
		{
			if (_isEnabled)
			{
				if( _sampGrabber == null )
					return;
				int hr = _sampGrabber.SetCallback( null, 0 );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
			}
			_isEnabled = false;
		}

		//		public void OnSample(double SampleTime, IntPtr pBuffer, int BufferLen  )
		//		{
		//			lock (this)
		//			{
		//				if(FrameCaptured != null)
		//				{
		//					FrameCaptured(this, new VideoCaptureFrameEventArgs(SampleTime, pBuffer, BufferLen, _savedArray, _videoInfoHeader));
		//				}
		//			}		
		//		}

		int ISampleGrabberCB.SampleCB( double SampleTime, IMediaSample pSample )
		{
			lock (this)
			{
				//SetMediaSample=pSample;
				Console.WriteLine( "!!CB: ISampleGrabberCB.SampleCB" );
				
			}
			return 0;
		}

		/// <summary> buffer callback, COULD BE FROM FOREIGN THREAD. </summary>
		int ISampleGrabberCB.BufferCB( double SampleTime, IntPtr pBuffer, int BufferLen )
		{
			//
			//			Console.WriteLine( "!!CB: ISampleGrabberCB.BufferCB" );
			lock (this)
			{
				if( (pBuffer != IntPtr.Zero) && (BufferLen > 1000) && (BufferLen <= _savedArray.Length) )
					Marshal.Copy( pBuffer, _savedArray, 0, BufferLen );
				else
					Console.WriteLine( "    !!!GRAB! failed " );
				//			Console.WriteLine("ThreadHashcode: " + System.Threading.Thread.CurrentThread.GetHashCode().ToString());
				//			OnSample(SampleTime, pBuffer, BufferLen);

				//_b = null;
				int w = _videoInfoHeader.BmiHeader.Width;
				int h = _videoInfoHeader.BmiHeader.Height;
				if( ((w & 0x03) != 0) || (w < 32) || (w > 4096) || (h < 32) || (h > 4096) )
					return 0;
				int stride = w * 3;

				GCHandle handle = GCHandle.Alloc( _savedArray, GCHandleType.Pinned );
				int scan0 = (int) handle.AddrOfPinnedObject();
				scan0 += (h - 1) * stride;
				Bitmap b = new Bitmap( w, h, -stride, PixelFormat.Format24bppRgb, (IntPtr) scan0);
				//				Bitmap c = new Bitmap(b);
				handle.Free();
				//				b.Dispose();
				//				b=null;
				//				c = _fpDetector.Detection(c);
				SetBitmap=b;
			}
			return 0;
		}

		private System.Drawing.Bitmap SetBitmap
		{
			set
			{
				//				FrameCaptured(value);
				NewPicture(new Bitmap(value));
			}
		}

		#region IPictureGrabber Member

		//public event MT.FPSource.NewPictureDelegate NewPicture;

		public Bitmap GrabPicture()
		{
			return null;
		}

		#endregion
	}

}
