using System;
using MT.Filter.Server;
using MT.DataStructures;
using DShowNET;
using System.Runtime.InteropServices;

namespace GraphController
{
	/// <summary>
	/// clsFPnetReceiver inherits the MT.Filter.Server modul and automatically parses
	/// Featurepoints from it. It also inherits the IFPsource Interface and can therefor
	/// server as a Featurepoint Source for the clsTracker Modul of the MTClient
	/// </summary>
	public class clsFPnetReceiver: Server, IFPsource
	{

		ISWFilter _SWFilter;
//		//declare Event Delegates
//		public delegate void NewFrameDelegate(FeaturePointFrame FPframe);

		//declare Events
		//public event NewFrameDelegate NewFrame ;
		public event NewFrameDelegate NewFrame;


		public clsFPnetReceiver(int Port,ISWFilter SWFilter): base(Port)
		{
			_SWFilter = SWFilter;

			//Connect necessary ServerEvents
			base.NewMessage += new Server.NewMessageDelegate(Parse);

		}

		/// <summary>
		/// Event Delegate - a new Servermessage with a string message to parse has Arrived
		/// </summary>
		/// <param name="ConID"></param>
		/// <param name="message"></param>
		private void Parse(string ConID, string message)
		{
			int frame = -1;			//Stores Parsed Framenumber
			int zonecount = -1;		//Stores Number of Points in loaded FrameFile
			int sync = -1;			//Stores serverside Syncsignal

			System.Threading.Thread.Sleep(5);
			//Temporary Array to split out Headerdata from message
			string[] header = new string[4];

			header = message.Split(new char[1] {';'},4);

			frame = int.Parse(header[0]);
			zonecount = int.Parse(header[1]);
			sync = int.Parse(header[2]);

			//Temporary Arrary to split out Zones from Message
			string[] zones = new string[zonecount];

			zones = header[3].Split(new char[1] {';'},zonecount);
			
			//Temporary Array to split out Zonedata from each Zone
			string[] zone = new string[8];

			//Create new FP Frame Class
			FeaturePointFrame	newFrame = new FeaturePointFrame(frame,"");
			newFrame.Sync = sync;

			FeaturePoint P;

			for(int i = 0; i<zonecount;i++)
			{
				zone = zones[i].Split(new char[1] {'\t'},8);

				//Create a new Feature Point with Coords out of the parsed Values of myarray
				// ToDo: May cause Problems that we hardcoded Picturesize (480)
				P = new FeaturePoint(float.Parse(zone[1]),					
									 480.0f-float.Parse(zone[2]),
									 (eColorCode)(int)zone[6].ToCharArray(0,1)[0],
									 i+1);
				

				//Store new Point into all List
				newFrame.Add(P);
			}

			//Raise Event to announce new parsed Frame
			if(NewFrame!=null)
				NewFrame(newFrame);
					
		}
//		#region IFPsource Member
//
//		event MTClient.NewFrameDelegate MTClient.IFPsource.NewFrame;
//
//		#endregion

		#region IFPsource Member

		public void Activate()
		{
			SWFilterParams	parset;

			//Deactivate Networkcommunication of SWFilter
			int hr = _SWFilter.GetParams(out parset);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
					
			parset.intNet = 1;

			hr = _SWFilter.SetParams(out parset);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
		}

		public void Deactivate()
		{
			SWFilterParams	parset;

			//Deactivate Networkcommunication of SWFilter
			int hr = _SWFilter.GetParams(out parset);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
					
			parset.intNet = 0;

			hr = _SWFilter.SetParams(out parset);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );		}

		#endregion
	}
}
