using System;
using System.Collections;
using MT.DataStructures;

namespace MT.Tracker
{

	/// <summary>
	/// Summary description for FeaturePointDataManager.
	/// </summary>
	public class RGBTracker
	{
		public delegate void NewHypothesisesDelegate(HypothesisSuperSet HS_R, HypothesisSuperSet HS_G,HypothesisSuperSet HS_B);
		public 	event NewHypothesisesDelegate NewHypothesises;

		private	Tracker					_TrackerR;	//Contains a Tracker for each color
		private	Tracker					_TrackerG;	
		private	Tracker					_TrackerB;	

		private	bool		_ConfigMode=false;		//Puts the tracker into a state where it raises events 
													//whenever a HypothesisSet has been created

		//Tracker Settings
		private	int			_Vmax = 40;			//Maximum allowed Speed in Pixels
		private	int			_Vmin = 5;			//Minimal Speed so that AngleChanges are considered
		private	int			_Vnrm = 10;			//Norm Speed from which on the normal weight in cost function is considered
		private	float		_Gmax = 0.8f;		//Maximum allowed Costs

		private	int			_Vbase;				// = _Vnrm-_Vmin;

		//Gamma Settings
		private float _w1 = 0.9f;	//Angleweight
		private float _w2b = 0.1f;	//Speedweight
		private float _w2a = 0.0001f;


		#region "Properties"

		public int Vmax
		{
			get {	return _Vmax;	}
			set 
			{
					_Vmax = value;	
				_TrackerR.Vmax = value;
				_TrackerG.Vmax = value;
				_TrackerB.Vmax = value;
			}
		}

		public int Vmin
		{
			get {	return _Vmin;	}
			set 
			{
				_Vmin = value;	
				_TrackerR.Vmin = value;
				_TrackerG.Vmin = value;
				_TrackerB.Vmin = value;
				calcVbase();
			}
		}

		public int Vnrm
		{
			get {	return _Vnrm;	}
			set 
			{
				_Vnrm = value;	
				_TrackerR.Vnrm = value;
				_TrackerG.Vnrm = value;
				_TrackerB.Vnrm = value;
				calcVbase();
			}

		}

		public float GammaMax
		{
			get { return _Gmax;		}
			set 
			{
				_Gmax = value;	
				_TrackerR.GammaMax = value;
				_TrackerG.GammaMax = value;
				_TrackerB.GammaMax = value;
			}
		}

		public float AngleWeight
		{
			get { return _w1;		}
			set 
			{
				_w1 = value;	
				_TrackerR.AngleWeight = value;
				_TrackerG.AngleWeight = value;
				_TrackerB.AngleWeight = value;
			}		
		}

		public float SpeedWeight
		{
			get { return _w2b;		}
			set 
			{
				_w2b = value;	
				_TrackerR.SpeedWeight = value;
				_TrackerG.SpeedWeight = value;
				_TrackerB.SpeedWeight = value;
			}		
		}

		public float StartSpeedWeight
		{
			get { return _w2a;		}
			set 
			{
				_w2a = value;	
				_TrackerR.StartSpeedWeight = value;
				_TrackerG.StartSpeedWeight = value;
				_TrackerB.StartSpeedWeight = value;
			}		
		}

		public bool	 ConfigMode
		{
			get { return _ConfigMode; }
			set 
			{
				_ConfigMode = value;	
				_TrackerR.ConfigMode = value;
				_TrackerG.ConfigMode = value;
				_TrackerB.ConfigMode = value;
			}		
		}


		#endregion 


		public RGBTracker()
		{	
			//Create Tracker instanzes
			_TrackerR = new Tracker("red");
			_TrackerG = new Tracker("green");
			_TrackerB = new Tracker("blue");

			_TrackerR.NewHypothesises +=new MT.Tracker.NewHypothesisesDelegate(OnNewHypothesisesR);
			_TrackerG.NewHypothesises +=new MT.Tracker.NewHypothesisesDelegate(OnNewHypothesisesG);
			_TrackerB.NewHypothesises +=new MT.Tracker.NewHypothesisesDelegate(OnNewHypothesisesB);
		}

		private HypothesisSuperSet _HS_R;
		private HypothesisSuperSet _HS_G;
		private HypothesisSuperSet _HS_B;

		private void OnNewHypothesisesR(HypothesisSuperSet HS)
		{
			_HS_R = HS;
		}
		private void OnNewHypothesisesG(HypothesisSuperSet HS)
		{
			_HS_G = HS;
		}
		private void OnNewHypothesisesB(HypothesisSuperSet HS)
		{
			_HS_B = HS;
			OnNewHypothesises();
		}

		private void OnNewHypothesises()
		{
			if (NewHypothesises!=null && _HS_R!=null && _HS_G!=null && _HS_B!=null)
			{
//				foreach(Delegate D in NewHypothesises.GetInvocationList())
//				{
//					NewHypothesises.BeginInvoke
//				}
				NewHypothesises(_HS_R,_HS_G,_HS_B);
				_HS_R = null;
				_HS_G = null;
				_HS_B =null;
			}
		}
	
		private void	calcVbase()
		{
			_Vbase = _Vnrm-_Vmin;
		}


		public bool initialize(FeaturePointFrame oldFrame, FeaturePointFrame curFrame, FeaturePointFrame newFrame)
		{
			bool r,g,b;
			r = _TrackerR.initialize(oldFrame.Pred,curFrame.Pred,newFrame.Pred);			
			g = _TrackerG.initialize(oldFrame.Pgreen,curFrame.Pgreen,newFrame.Pgreen);			
			b = _TrackerB.initialize(oldFrame.Pblue,curFrame.Pblue,newFrame.Pblue);			
			return r && g && b;
		}

		public bool processDoubleFrame(FeaturePointFrame curFrame, FeaturePointFrame newFrame)
		{
			bool r,g,b;
			r = _TrackerR.processDoubleFrame(curFrame.Pred,newFrame.Pred);
			g = _TrackerG.processDoubleFrame(curFrame.Pgreen,newFrame.Pgreen);
			b = _TrackerB.processDoubleFrame(curFrame.Pblue,newFrame.Pblue);
			return r && g && b;
		}

		public bool processFrame(FeaturePointFrame oldFrame, FeaturePointFrame curFrame, FeaturePointFrame newFrame)
		{
			bool r,g,b;
			r = _TrackerR.processFrame(oldFrame.Pred,curFrame.Pred,newFrame.Pred);
			g = _TrackerG.processFrame(oldFrame.Pgreen,curFrame.Pgreen,newFrame.Pgreen);
			b = _TrackerB.processFrame(oldFrame.Pblue,curFrame.Pblue,newFrame.Pblue);
			return r && g && b;
		}

		public bool postProcessFrame(FeaturePointFrame oldFrame)
		{
			bool r,g,b;
			r = _TrackerR.postProcessFrame(oldFrame.Pred);
			g = _TrackerG.postProcessFrame(oldFrame.Pgreen);
			b = _TrackerB.postProcessFrame(oldFrame.Pblue);
			return r && g && b;
		}


	}

//	{
//		int _DoubleFrame = 0;		//Stores Number of dropped doubleframes
//		private FeaturePointData	_A;
//		private FeaturePointData	_R;
//		private FeaturePointData	_G;
//		private FeaturePointData	_B;
//
//		private Tracker				_TrackerR;
//		private Tracker				_TrackerG;
//		private Tracker				_TrackerB;
//
//		private int					_CameraID;
//
////		FrameLoader _Loader; 
////		FrameSaver _Saver;
//
//
//
//		public FeaturePointData Data
//		{
//			get { return _A; }
//		}
//
//
//		public FeaturePointData DataR
//		{
//			get { return _R; }
//		}
//
//		public FeaturePointData DataG
//		{
//			get { return _G; }
//		}
//
//		public FeaturePointData DataB
//		{
//			get {return _B; }
//		}
//
//		public int DoubleFrame
//		{
//			get { return _DoubleFrame; }
//		}
//
//		public TrackerMain(FeaturePointData A, FeaturePointData R, FeaturePointData G, FeaturePointData B,int CameraID)
//		{
//			//LoadPath = @"d:\Projekte\200310 Motion Tracking mit Webcams\sh_tracking\listen\skelett2_compress\";
//			//LoadName = "liste";
//
//			_CameraID = CameraID;
//			_A = A;
//			_R = R;
//			_G = G;
//			_B = B;
//			//Initalize 3 Trackers each for one color
//			_TrackerR = new Tracker(_R,"R", _CameraID);
//			_TrackerG = new Tracker(_G,"G", _CameraID);
//			_TrackerB = new Tracker(_B,"B", _CameraID);
//		
//			//_Loader = new FrameLoader(LoadPath,LoadName,"txt",5,this) ;
//			//_Saver = new FrameSaver(@"d:\Projekte\200310 Motion Tracking mit Webcams\sh_tracking\listen\Listen_logitech_white_10fps(35,20)\","liste_processed","txt",3) ;
//		
//		}
//
//		/// <summary>
//		/// Called by the parent, each time when a new Frame is available.
//		/// </summary>
//		public void TrackFrame()
//		{
//
//			int FrameCount = _A.CountFrames();
//
//			//The Tracker runs always one frame after the Frameloader
//			int curFrame = _A.CountFrames()-2;		
//
//			// ToDo: muss von der Controllerklasse verwaltet werden.
//			//_Loader.LoadFrame(++_LastRealFrame);
//
//			//First we calculate the least square error between the new frame and the previous frame
//			//The idea is to detect a phenomenon called double frame, which occurs when the computer
//			//hat a bufferoverflow during the recordingphase. Since the Trajectories are interrupted 
//			//by Doubleframes, we connect them manually
//			const float MIN_ERROR = 1f;
//
//			double error = 1000000.0;
//			//ToDo: Add Compare Method
//
//			if (FrameCount > 3)
//			{	//Calculate complete LS Error between current and previous frame
//				error = 0.0;
//				error += CalcLeastSquareError(_R.GetFrame(curFrame+1),_R.GetFrame(curFrame));
//				error += CalcLeastSquareError(_G.GetFrame(curFrame+1),_G.GetFrame(curFrame));
//				error += CalcLeastSquareError(_B.GetFrame(curFrame+1),_B.GetFrame(curFrame));
//
//			}
//
//
//			//Whenever we add a frame, check what can and has to be
//			//done by the trackers:
//			//For Framenumbers higher than 3 we process the Frame
//			//For Framenumbers equal 3 we initalize the tracker
//			if (FrameCount >3)
//			{
//				//Only if the error between 2 pictures is larger than our threshold
//				//we will analyse the new frame with the tracker else it is considered 
//				//a doubleframe and we manually connect identical Featurepoints.
//				if (error > MIN_ERROR)
//				{
//
//					_TrackerR.processFrame(curFrame);
//					_TrackerG.processFrame(curFrame);
//					_TrackerB.processFrame(curFrame);
//				}
//				else
//				{	//Manually connect Identical points
//					_TrackerR.processDoubleFrame(curFrame);
//					_TrackerG.processDoubleFrame(curFrame);
//					_TrackerB.processDoubleFrame(curFrame);
//					_DoubleFrame++;
//				}
//
//			}
//			else
//			{
//				if(FrameCount==3)
//				{
//					_TrackerR.initialize();
//					_TrackerG.initialize();
//					_TrackerB.initialize();
//				}
//			}
//
//			//Make sure, all FPs (even the unconnected ones) have a trajectoryID
//			//to allow correct classification by the TUK Tracker later.
//			_TrackerR.postProcessFrame(curFrame-1);
//			_TrackerG.postProcessFrame(curFrame-1);
//			_TrackerB.postProcessFrame(curFrame-1);
//
//		}
//
//		public void Finish()
//		{
//			_TrackerR.Finish();
//			_TrackerG.Finish();
//			_TrackerB.Finish();
//
//		}
//
//
//		//Calculates LeastSquare of 2 FeaturePointSets
//		private  double CalcLeastSquareError (SortedList a,  SortedList b)
//		{
//			SortedList S;
//			SortedList s;
//
//			int SCount;
//			int sCount;
//
//			double r = 0.0f;
//			int dP;					//Difference in FeaturePoints between a and b
//			float error=0.0f;
//			
//			if (a.Count > b.Count)
//			{
//				S= a;
//				s= b;
//			}
//			else
//			{
//				s= a;
//				S= b;
//			}
//
//			SCount = S.Count;
//			sCount = s.Count;
//
//			dP = SCount - sCount;
//
//			FeaturePoint P;
//			FeaturePoint p;
//
//			
//
//			for (int i = 0;i<sCount; i++)
//			{
//				p = (FeaturePoint)s.GetByIndex(i);
//
//				error = 1000000.0f;
//				for (int j=0; j<=dP; j++)
//				{
//					P = (FeaturePoint)S.GetByIndex(i+j);
//					error = System.Math.Min(error,(P.X -  p.X) * (P.X -  p.X)+ (P.Y -  p.Y) * (P.Y -  p.Y));
//				}
//			
//				r+= error;
//
//			}
//
//			return r;
//		
//		}
//	}
}
