using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

using System.Diagnostics;
using System.Drawing.Imaging;
using System.IO;
using tsai;
using GraphController;
using MT.DataStructures;
using MT.FPSource;
using MT.Network;


namespace MTClient
{
	/// <summary>
	/// Zusammenfassung fr clsCalibration.
	/// </summary>
	public class clsCalibration
	{
		IPictureGrabber			_IPgrabber;			// Interface um ein Bild zu grabben
//		PictureBox				_pbox_Bitmap;		// PictureBox in welche das gegrabbte Bitmap geladen wird 
		Bitmap	_bitmapIn;							// Originalbild
		Bitmap	_bitmapOut;							// Bearbeitetes Bild
		bool	_pictureloaded	= false;			// Ist ein Bild geladen?
		
		float	_zoomfactorX	= 1;
		float	_zoomfactorY	= 1;
		tsai_converter _tsai;						// Tsai
		bool	_calibrated		= false;			// Wurde schon kalibriert?


		static int _Squares		= 5;				// Anzahl Quadrate pro Zeile - 1
		static int _intersections = (_Squares+1)*(_Squares+1);			// Anzahl Quadrate pro Wrfelseite

		

		int[,]	_axis			= new int[60,6];	// Weltkoordinatenachsen
		int[,]	_cube			= new int[8,2];		// Wrfel
		
		double[] _cameraposition= new double[3];	// Kameraposition
		int		_cameratype		= 1;				// Kameratyp: 1=Logitech, 2=Sony DV
		int		_chessboard		= 1;				// Wrfelseite 1 bis 5
		bool[]	_calculated		= new bool[5];		// Ist diese Wrfelseite schon kalibriert?
		float[,] _Bodypoint		= new float[13,2];	// Zuordnung FP-BP
		float[,] _corner		= new float[(int)(5*_intersections),5]; // Punkteliste mit x, y, z, u, v
		float[,] _point			= new float[4,2];	// 4 Eckpunkte eines Schachbretts
		
		float	_deltaXleft		= 0;				// 
		float	_deltaXright	= 0;
		float	_deltaYleft		= 0;
		float	_deltaYright	= 0;

		int		_ImageX			= 0;				// Berechneter Weltkoordinatenpunkt
		int		_ImageY			= 0;

		private FilterGraphController		_filterGraphController;
		private clsFPnetSink				_Network;



		public delegate void UpdateCornerDelegate(float[,] corner);
		public event UpdateCornerDelegate UpdateCorner;

		public delegate void UpdateCameraPositionDelegate(double[] cameraposition);
		public event UpdateCameraPositionDelegate UpdateCameraPosition;

		public delegate void UpdatePictureDelegate(Bitmap b);
		public event UpdatePictureDelegate UpdatePicture;

		public delegate void CalibrationDoneDelegate();
		public event CalibrationDoneDelegate CalibrationDone;

		#region "Properties"


		public float[,] Bodypoint
		{
			get {return _Bodypoint; }
			set {_Bodypoint = value;}
		}

		public float[,] Point
		{
			get {return _point; }
			set {_point = value;}
		}

		public float ZoomfactorX
		{
			get {return _zoomfactorX; }
			set {_zoomfactorX = value;}
		}

		public float ZoomfactorY
		{
			get {return _zoomfactorY; }
			set {_zoomfactorY = value;}
		}

		public int Intersections
		{
			get {return _intersections; }
			set {_intersections = value;}
		}

		public int ImageX
		{
			get {return _ImageX; }
			set {_ImageX = value;}
		}

		public int ImageY
		{
			get {return _ImageY; }
			set {_ImageY = value;}
		}

		public Bitmap BitmapIn
		{
			get { return _bitmapIn; }
		}

		public double[] Cameraposition
		{
			get {return _cameraposition; }
		}


		public bool Calibrated
		{
			get {return _calibrated; }
//			set {_calibrated = value;}
		}

		public bool Pictureloaded
		{
			get {return _pictureloaded; }
//			set {_pictureloaded = value;}
		}

		public bool[] Calculated
		{
			get {return _calculated; }
//			set {_calculated = value;}
		}

		public int[,] Axis
		{
			get {return _axis; }
		}

		public int[,] Cube
		{
			get {return _cube; }
		}

		public float[,] Corner
		{
			get {return _corner; }
		}

		public int Cameratype
		{
			get {return _cameratype; }
			set {_cameratype = value;}
		}

		public int Chessboard
		{
			get {return _chessboard; }
			set
			{	
				_chessboard = value;
				for (int i=0 ; i<_point.GetLength(0); i++)
				{ 
					_point[i,0] = -1;
					_point[i,1] = -1;
				}
			}
		}
		
		#endregion

		public clsCalibration(IPictureGrabber IPgrabber, clsFPnetSink Network)
		{
			_IPgrabber = IPgrabber;
			_Network = Network;
			_Network.NewConnection +=new NetEventHandler(_Network_NewConnection);
		}

		public void Initalize(FilterGraphController filterGraphController, clsTracker Tracker)
		{
			_filterGraphController = filterGraphController;

		}

		public void ResetGrid()
		{
			for (int i = 0 ; i < _point.GetLength(0); i++)
			{ 
				_point[i,0] = -1;
				_point[i,1] = -1;
			}
			for (int i = 0 ; i < _intersections; i++)
			{ 
				_corner[i+(_chessboard-1)*_intersections,3] = -1;
				_corner[i+(_chessboard-1)*_intersections,4] = -1;
			}

			_calculated[_chessboard-1] = false;
		}

		public void InitializeVariables()
		{
			for (int i=0; i<_Bodypoint.GetLength(0); i++)
			{
				_Bodypoint[i,0] = -1;
				_Bodypoint[i,1] = -1;
			}

			for (int i=0; i<_calculated.GetLength(0); i++)
			{
				_calculated[i] = false;
			}

			for (int i = 0 ; i < _point.GetLength(0); i++)
			{ 
				_point[i,0] = -1;
				_point[i,1] = -1;
			}


			for (int i = 0 ; i < _axis.GetLength(0); i++)
			{ 
				_axis[i,0] = 0;
				_axis[i,1] = 0;
				_axis[i,2] = 0;
				_axis[i,3] = 0;
				_axis[i,4] = 0;
				_axis[i,5] = 0;
			}
			for (int i = 0 ; i < _cube.GetLength(0); i++)
			{ 
				_cube[i,0] = 0;
				_cube[i,1] = 0;
			}
			
			int k=0;
			// Weltkoordinaten der einzelnen Schachbrettzentren
			for (int z=1100; z>=100; z-=200)
			{
				for (int y=1100; y>=100; y-=200)
				{
					//Rot
					_corner[k,0]=0;		
					_corner[k,1]=y;		
					_corner[k,2]=z;
					_corner[k,3]=-1;	
					_corner[k,4]=-1;	

					//Grn
					_corner[k+1*_intersections,0]=1200-y;
					_corner[k+1*_intersections,1]=0;		
					_corner[k+1*_intersections,2]=z;
					_corner[k+1*_intersections,3]=-1;	
					_corner[k+1*_intersections,4]=-1;	

					//Rot
					_corner[k+2*_intersections,0]=1200;	
					_corner[k+2*_intersections,1]=1200-y;
					_corner[k+2*_intersections,2]=z;
					_corner[k+2*_intersections,3]=-1;	
					_corner[k+2*_intersections,4]=-1;	

					//Grn
					_corner[k+3*_intersections,0]=y;		
					_corner[k+3*_intersections,1]=1200;	
					_corner[k+3*_intersections,2]=z;
					_corner[k+3*_intersections,3]=-1;	
					_corner[k+3*_intersections,4]=-1;	

					//Blau
					_corner[k+4*_intersections,0]=z;		
					_corner[k+4*_intersections,1]=y;		
					_corner[k+4*_intersections,2]=1200;
					_corner[k+4*_intersections,3]=-1;	
					_corner[k+4*_intersections,4]=-1;	
					k++;					
				}						  
			}
			
			// Offset vom Weltkoordinatenursprung zum Wrfel
			for (int i=0; i<_corner.GetLength(0); i++)
			{
				//_corner[i,0]+= 2400;
				//_corner[i,1]+= 1200;
				_corner[i,2]+=   59;
			}
		}




		public void LoadPicture(string Filename)
		{
			_bitmapIn   = new Bitmap(Filename);
			_bitmapOut	= (Bitmap)_bitmapIn.Clone();


			BitmapData bmData   = _bitmapIn.LockBits(new Rectangle(0, 0, _bitmapIn.Width, _bitmapIn.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb); 
			_bitmapIn.UnlockBits(bmData);

			if (UpdatePicture !=null)
				UpdatePicture(new Bitmap(_bitmapIn));

			for (int i = 0 ; i < _point.GetLength(0); i++)
			{ 
				_point[i,0] = -1;
				_point[i,1] = -1;
			}

			_pictureloaded			 = true;
		}


		public void LoadCalibFile(string Filename, eTsaiOrientation Orientation)
		{
			_tsai		= new tsai_converter(Filename, Orientation);
			_calibrated = true;	
			ResetPicture();
			calc3D();
			OnCalibrationDone();
		}

		private void OnCalibrationDone()
		{
			//Announce Calibration
			if (CalibrationDone!=null) {CalibrationDone();}

			_Network.Send(new clsNetTsaiConverter(_Network.ConnectionID,_tsai));

		}

		public bool readyToCalibrate()
		{
			int k=0;
			int i=0;
			for (i=0; i<_calculated.GetLength(0); i++)
			{
				if (_calculated[i]==true){k++;}
			}
			if (k<2){return false;}
			else	{return true;}
		}


		public void Calibrate()
		{
			int i = 0;
			int k = 0;
			string filenameCd	= @"c:\CalibData.dat";	//Liste mit Kaliebrierungspunkten (World & Image)
			string filenameTsai	= @"c:\Vtsai.exe";		//Tsai-Programm
			string filenameCp	= @"c:\CalibParam.dat";	//Kalibrierungsparameter
			StreamWriter input	= new StreamWriter(filenameCd);


			//			for (i=0; i<_axis.GetLength(0); i++)
			//			{ 
			//				_axis[i,0] = 0;
			//				_axis[i,1] = 0;
			//				_axis[i,2] = 0;
			//				_axis[i,3] = 0;
			//				_axis[i,4] = 0;
			//				_axis[i,5] = 0;
			//			}

			for (i=0; i<_corner.GetLength(0); i++)
			{
				if (_corner[i,3]!=-1 && k<500)
				{
					input.WriteLine("{0:f0} {1:f0} {2:f0} {3:f2} {4:f2}", _corner[i,0], _corner[i,1], _corner[i,2], _corner[i,3], _corner[i,4]);
					k++;
				}
			}
			input.Flush();
			input.Close();
		
			Process cal = new Process();
	
			cal.StartInfo.FileName = filenameTsai;
			cal.StartInfo.Arguments = String.Concat(filenameCd, " ", _cameratype.ToString());
			cal.StartInfo.UseShellExecute = false;
			cal.StartInfo.RedirectStandardOutput = true;
			System.Console.WriteLine(String.Concat(filenameTsai, " ",filenameCd, " ", _cameratype.ToString())); //argumente
			cal.Start();
			cal.WaitForExit();

			StreamWriter output = new StreamWriter(filenameCp);
			string data			= cal.StandardOutput.ReadLine();

			while (data != null ) 
			{	
				output.WriteLine(data);
				data = cal.StandardOutput.ReadLine();
			}

			output.Flush();
			output.Close();

			_tsai = new tsai_converter(filenameCp, eTsaiOrientation.x);
		
			_calibrated = true;
			calc3D();

			//Announce Calibration
			OnCalibrationDone();
		}



		public void calculateCorners()
		{

			// Unabhngige Reihenfolge der Punkteingabe
			//			int ul, ll, ur, lr;
			//			int[,] _newpoint = new int[4,2];
			//
			//			sort(_point, 0); // Nach X sortieren
			//
			//			if (_point[0,1]<_point[1,1]) {ul=0; ll=1;}
			//			else						 {ul=1; ll=0;}
			//			if (_point[2,1]<_point[3,1]) {ur=2; lr=3;}
			//			else						 {ur=3; lr=2;}
			//			
			//			_newpoint[0,0]=_point[ul,0];
			//			_newpoint[0,1]=_point[ul,1];
			//			_newpoint[1,0]=_point[ur,0];
			//			_newpoint[1,1]=_point[ur,1];
			//			_newpoint[2,0]=_point[lr,0];
			//			_newpoint[2,1]=_point[lr,1];
			//			_newpoint[3,0]=_point[ll,0];
			//			_newpoint[3,1]=_point[ll,1];
			//			_point=_newpoint;
							


			// Ausgehend von den Ecken des Wrfels werden die Schwerpunkte der usseren vier Quadrate berechnet
			float deltaX1 = (_point[3,0]-_point[1,0])/((_Squares+1)*2);
			float deltaY1 = (_point[3,1]-_point[1,1])/((_Squares+1)*2);
			float deltaX2 = (_point[2,0]-_point[0,0])/((_Squares+1)*2);
			float deltaY2 = (_point[2,1]-_point[0,1])/((_Squares+1)*2);

			_point[0,0]+=deltaX2;
			_point[0,1]+=deltaY2;
			_point[1,0]+=deltaX1;
			_point[1,1]+=deltaY1;
			_point[2,0]-=deltaX2;
			_point[2,1]-=deltaY2;
			_point[3,0]-=deltaX1;
			_point[3,1]-=deltaY1;


			if(_pictureloaded)
			{
				//pbox_Bitmap.Image = null;
				AutomaticThreshold(_bitmapIn, _point);
				//pbox_Bitmap.Image = _bitmapOut;
			}


			_deltaXleft  = (_point[3,0]-_point[0,0])/(_Squares);
			_deltaYleft  = (_point[3,1]-_point[0,1])/(_Squares);
			_deltaXright = (_point[2,0]-_point[1,0])/(_Squares);
			_deltaYright = (_point[2,1]-_point[1,1])/(_Squares);

			int k=0;
			float dxh,dyh;

			// Linear interpolierte SChnittpunkte
			for (int y=0; y<=_Squares; y++)//Horizontale Geraden
			{
				dxh = ((_point[1,0]+(y*_deltaXright))-(_point[0,0]+(y*_deltaXleft)))/(_Squares);
				dyh = ((_point[1,1]+(y*_deltaYright))-(_point[0,1]+(y*_deltaYleft)))/(_Squares);
				
				for (int x=0; x<=_Squares; x++)//Vertikale Geraden
				{
					_corner[k+(_chessboard-1)*_intersections,3] = _point[0,0]+(x*dxh)+(y*_deltaXleft);
					_corner[k+(_chessboard-1)*_intersections,4] = _point[0,1]+(x*dyh)+(y*_deltaYleft);					
					k++;
				}
			}

			float cx = new float();
			float cy = new float();
			
			// Genaue Position (Schwerpunkt) berechnen
			for (k=0;k<_intersections; k++)
			{
				centre(_bitmapOut, _corner[k+(_chessboard-1)*_intersections,3], _corner[k+(_chessboard-1)*_intersections,4], ref cx, ref cy);
				_corner[k+(_chessboard-1)*_intersections,3]=cx;
				_corner[k+(_chessboard-1)*_intersections,4]=cy;
			}

			// Falls zwei unterchiedliche Weltpunkte den gleichen Kamerapunkt beekommen, werden sie gelscht
			for (int i=0; i<_intersections; i++)
			{
				for (int j=i+1; j<_intersections; j++)
				{
					if ( Math.Abs(_corner[i+(_chessboard-1)*_intersections,3] -
						_corner[j+(_chessboard-1)*_intersections,3]) < 2 &&
						Math.Abs(_corner[i+(_chessboard-1)*_intersections,4] -
						_corner[j+(_chessboard-1)*_intersections,4]) < 2)
					{
						_corner[i+(_chessboard-1)*_intersections,3]=-1;
						_corner[i+(_chessboard-1)*_intersections,4]=-1;
						_corner[j+(_chessboard-1)*_intersections,3]=-1;
						_corner[j+(_chessboard-1)*_intersections,4]=-1;
					}
				}
			}

			UpdateCorner(_corner);

			_calculated[_chessboard-1] = true;
			readyToCalibrate();
		}


		public void ResetPicture()
		{
			if(_pictureloaded)
			{
				for (int i = 0 ; i < _axis.GetLength(0); i++)
				{ 
					_axis[i,0] = 0;
					_axis[i,1] = 0;
					_axis[i,2] = 0;
					_axis[i,3] = 0;
					_axis[i,4] = 0;
					_axis[i,5] = 0;
				}
				for (int i = 0 ; i < _cube.GetLength(0); i++)
				{ 
					_cube[i,0] = 0;
					_cube[i,1] = 0;
				}

				_ImageX = 0;
				_ImageY = 0;

				UpdatePicture(_bitmapIn);
//				pbox_Bitmap.Image       = _bitmapIn;
//				this.Refresh();
			}
		}

		public void calc3D()
		{
			int i=0;
			double x = new double();
			double y = new double();

			for (i = 0 ; i < _point.GetLength(0); i++)
			{ 
				_point[i,0] = -1;
				_point[i,1] = -1;
			}

			//Kameraposition berechnen
			_tsai.get_camera_position(ref _cameraposition[0], ref _cameraposition[1], ref _cameraposition[2]);
			UpdateCameraPosition(_cameraposition);

			System.Console.WriteLine(String.Concat("Kameraposition: ",  ((int)_cameraposition[0]).ToString(), " / ",
				((int)_cameraposition[1]).ToString(), " / ",
				((int)_cameraposition[2]).ToString() ));

			// Koordinatenachsen
			for (i=0; i<_axis.GetLength(0); i++)
			{
				_tsai.world_coord_to_image_coord(i*200-3000, 0, 0, ref x, ref y);
				if (x>0)
				{
					_axis[i,0]=(int)x; _axis[i,1]=(int)y;
				}
					
				_tsai.world_coord_to_image_coord(0, i*200-3000, 0, ref x, ref y);
				if (x>0)
				{
					_axis[i,2]=(int)x; _axis[i,3]=(int)y;
				}
					
				_tsai.world_coord_to_image_coord(0, 0, i*200, ref x, ref y);
				if (x>0)
				{
					_axis[i,4]=(int)x; _axis[i,5]=(int)y;
				}
			}

			// Wrfel
			_tsai.world_coord_to_image_coord(0,		  0,   55, ref x, ref y); _cube[0,0]=(int)x; _cube[0,1]=(int)y;
			_tsai.world_coord_to_image_coord(1200,    0,   55, ref x, ref y); _cube[1,0]=(int)x; _cube[1,1]=(int)y;
			_tsai.world_coord_to_image_coord(1200, 1200,   55, ref x, ref y); _cube[2,0]=(int)x; _cube[2,1]=(int)y;
			_tsai.world_coord_to_image_coord(0,	   1200,   55, ref x, ref y); _cube[3,0]=(int)x; _cube[3,1]=(int)y;
			_tsai.world_coord_to_image_coord(0,		  0, 1255, ref x, ref y); _cube[4,0]=(int)x; _cube[4,1]=(int)y;
			_tsai.world_coord_to_image_coord(1200,    0, 1255, ref x, ref y); _cube[5,0]=(int)x; _cube[5,1]=(int)y;
			_tsai.world_coord_to_image_coord(1200, 1200, 1255, ref x, ref y); _cube[6,0]=(int)x; _cube[6,1]=(int)y;
			_tsai.world_coord_to_image_coord(0,	   1200, 1255, ref x, ref y); _cube[7,0]=(int)x; _cube[7,1]=(int)y;
		}


		private Bitmap AutomaticThreshold(Bitmap bitmapIn, float[,] a)
		{
			_bitmapOut		    = (Bitmap)bitmapIn.Clone();
			BitmapData bmData   = _bitmapOut.LockBits(new Rectangle(0, 0, _bitmapOut.Width, _bitmapOut.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb); 
			int stride		    = bmData.Stride;		// bytes in a row 3*b.Width
			System.IntPtr Scan0 = bmData.Scan0;
			int iImgWidth		= _bitmapOut.Width;		// Bildbreite
			float[,] point		= (float[,])a.Clone();	// Histogramm in diesem Viereck
			float[,] square		= new float[4,2];		// Histogramm in diesem Viereck
			float[] histogramm	= new float[266];
			float[] histogramm2	= new float[266];
			int i = 0;
			int threshold = 0;	// Automatischer Schwellwert
			int max1 = 0;		// erstes lokales Maximum
			int max2 = 0;		// zweites lokales Maximum
			int max  = 0;		// temp. Maximum
			float min= 9999999; // Minimum

			for(i=0;i<266;i++)
			{
				histogramm[i]  = 0;
				histogramm2[i] = 0;
			}

			// Berechnung der Eckpunkte fr den Ausschnitt fr das Histogramm
			sort(point,0);
			square[0,0] = square[3,0] = point[1,0];
			square[1,0] = square[2,0] = point[2,0];

			sort(point,1);
			square[0,1] = square[1,1] = point[1,1];
			square[2,1] = square[3,1] = point[2,1];


			unsafe 
			{ 
				byte * p		= (byte *)(void *)Scan0;
				byte * p2		= (byte *)(void *)Scan0;
				int nOffset		= stride - _bitmapOut.Width*3;

				//Histogramm bilden
				for(int y=0;y < _bitmapOut.Height;++y)
				{
					for(int x=0; x<_bitmapOut.Width; ++x )
					{
						// Bereich innerhalb des Vierecks?
						if ( y>square[0,1] && y<square[2,1] && x>square[0,0] && x<square[1,0] )
						{
							//Histogramm von 5-260 statt von 0-255 wegen dem Randproblem beim gltten, deshalb +5
							histogramm[(int)(0.299*p[2] + 0.587*p[1] + 0.114*p[0])+5]++;
						}
						p += 3;
					}
					p += nOffset;
				}

				// Histogramm gltten
				for (i=5;i<261;i++)
				{
					histogramm2[i] = (	histogramm[i-5]+
						histogramm[i-4]+
						histogramm[i-3]+
						histogramm[i-2]+
						histogramm[i-1]+
						histogramm[i+0]+
						histogramm[i+1]+
						histogramm[i+2]+
						histogramm[i+3]+
						histogramm[i+4]+
						histogramm[i+5])/11;
				}

				for (i=5;i<261;i++)
				{
					histogramm[i] = (	histogramm2[i-5]+
						histogramm2[i-4]+
						histogramm2[i-3]+
						histogramm2[i-2]+
						histogramm2[i-1]+
						histogramm2[i+0]+
						histogramm2[i+1]+
						histogramm2[i+2]+
						histogramm2[i+3]+
						histogramm2[i+4]+
						histogramm2[i+5])/11;
				}

				for (i=5;i<261;i++)
				{
					histogramm2[i] = (	histogramm[i-5]+
						histogramm[i-4]+
						histogramm[i-3]+
						histogramm[i-2]+
						histogramm[i-1]+
						histogramm[i+0]+
						histogramm[i+1]+
						histogramm[i+2]+
						histogramm[i+3]+
						histogramm[i+4]+
						histogramm[i+5])/11;
				}

				for (i=5;i<261;i++)
				{
					histogramm[i] = (	histogramm2[i-5]+
						histogramm2[i-4]+
						histogramm2[i-3]+
						histogramm2[i-2]+
						histogramm2[i-1]+
						histogramm2[i+0]+
						histogramm2[i+1]+
						histogramm2[i+2]+
						histogramm2[i+3]+
						histogramm2[i+4]+
						histogramm2[i+5])/11;
				}

				// erstes lokales Maximum suchen
				max=0;
				for (i=5;i<261;i++)
				{
					if (histogramm[i]>histogramm[i-1] && histogramm[i]>histogramm[i+1] && histogramm[i]>histogramm[max]) 
					{max1=max=i;}
				}

				// zweites lokales Maximum suchen
				max=0;
				for (i=5;i<261;i++)
				{
					if (histogramm[i]>histogramm[i-1] && histogramm[i]>histogramm[i+1] && histogramm[i]<histogramm[max1] && histogramm[i]>histogramm[max]) {max2=max=i;}
				}

				// Minimum zwischen den beiden Maxima suchen
				//				if (max2==0){max2=1;}
				//				if (max1==255){max1=254;}
				
				for (i=max2;i<max1;i++)
				{
					if (histogramm[i]<min) {threshold=i-5; min=histogramm[i];}
				}
				

				// Binrbild erstellen
				for(int y=0;y < _bitmapOut.Height;++y)
				{
					for(int x=0; x<_bitmapOut.Width; ++x )
					{
						if ((0.299*p2[2] + 0.587*p2[1] + 0.114*p2[0])<threshold){p2[0] = p2[1] = p2[2] =   0;}
						else													{p2[0] = p2[1] = p2[2] = 255;}
						p2 += 3;
					}
					p2+= nOffset;
				}
			}
			_bitmapOut.UnlockBits(bmData);
			return _bitmapOut;
		}


		private bool centre(Bitmap bitmap, float x, float y, ref float cx, ref float cy)
		{
			int ix				= (int)x;
			int iy				= (int)y;
			Bitmap mybitmap	    = (Bitmap)bitmap.Clone();
			BitmapData bmData   = mybitmap.LockBits(new Rectangle(0, 0, mybitmap.Width, mybitmap.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb); 
			int stride		    = bmData.Stride;	// bytes in a row 3*b.Width
			System.IntPtr Scan0 = bmData.Scan0;
			int iImgWidth		= mybitmap.Width;	// Bildbreite
			float sumX			= 0;				// Aufsummierte X-Koordinaten des Umfangs
			float sumY			= 0;				// Aufsummierte Y-Koordinaten des Umfangs
			float z				= 0;				// Zhler fr Anzahl Pixel des Umfangs
			int   dx			= 0;				// Entfernung in X-Richtung zum Startpunkt (Spiralsuche)
			int   dy			= 0;				// Entfernung in Y-Richtung zum Startpunkt (Spiralsuche)
			int   vDir			= 0;				// Richtung
			int[] vOffset		= new int[8];		// Offset fr eine bestimmte Richtung
			int k = 0;
						
			vOffset[3] = -iImgWidth*3-3;	vOffset[2] = -iImgWidth*3;		vOffset[1] = -iImgWidth*3+3;
			vOffset[4] = -3;												vOffset[0] = 3;
			vOffset[5] = iImgWidth*3-3;		vOffset[6] = iImgWidth*3;		vOffset[7] = iImgWidth*3+3;		
			
			unsafe 
			{ 
				byte * p		= (byte *)(void *)Scan0;
				byte * vActual	= (byte *)(void *)Scan0;
				byte * vStart	= (byte *)(void *)Scan0;
				int nOffset		= stride - _bitmapOut.Width*3;
				
				//Randpixel suchen
				vStart += 3*(iy*iImgWidth+ix);
				vDir = 4;
				
				//Spiralfrmige Suche im Urzeigersinn. Start nach unten.
				while (*vStart != 0 && k<400)
				{
					vStart+=vOffset[vDir];
					if (vDir==4){dx-=1;}
					if (vDir==2){dy-=1;}
					if (vDir==0){dx+=1;}
					if (vDir==6){dy+=1;}
					if (-(dx+1)==dy && dx<0){vDir=2;}
					if (dx==dy && dx<0)		{vDir=0;}
					if (-dx==dy && dx>0)	{vDir=6;}
					if (dx==dy && dx>0)		{vDir=4;}
					k++;
				}
				

				//Startpunkt suchen
				k=0;
				while (*vStart == 0 && k<50)
				{
					vStart+=3;
					k++;
				}
				
				vStart-=3;
				vActual = vStart;				
				vDir = 0;
				k=0;
				
				do
				{
				while ( *(vActual+vOffset[vDir]) != 0 && k<10) 
				{ 
					if (vDir == 7)	{vDir=0;}
					else			{vDir++;}
					k++;
				}
					k=0;
					vActual = vActual + vOffset[vDir];

					if ( *vActual != 0){cx=-1; cy=-1; return false;}
					
					sumY+=(int)(((vActual-p)/3)/iImgWidth); //X und Y Koordinaten der Randpixel aufsummieren
					sumX+=(int)(((vActual-p)/3)%iImgWidth);
					z++;

					if (z>200) {cx=-1; cy=-1; return false;} //Irgend etwas ist schief gelaufen

					vDir = (vDir+5)%8;

				} while ( vActual != vStart);

				if (z<5) {cx=-1; cy=-1; return false;} //Irgend etwas ist schief gelaufen
			}
			cx=sumX/z; // Schwerpunkt bestimmen
			cy=sumY/z;
			mybitmap.UnlockBits(bmData);
			return true;
		}


		private Bitmap ConvertToBinary(Bitmap bitmapIn, int threshold)
		{
			// GDI+ return format is BGR, NOT RGB. 
			_bitmapOut		    = (Bitmap)bitmapIn.Clone();
			BitmapData bmData   = _bitmapOut.LockBits(new Rectangle(0, 0, _bitmapOut.Width, _bitmapOut.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb); 
			int stride		    = bmData.Stride; // bytes in a row 3*b.Width
			System.IntPtr Scan0 = bmData.Scan0;
			double t			= (double)threshold/10+0.1;
			
			unsafe 
			{ 
				byte * p = (byte *)(void *)Scan0;
				byte red, green, blue;
				int nOffset = stride - _bitmapOut.Width*3;
				for(int y=0;y < _bitmapOut.Height;++y)
				{
					for(int x=0; x<_bitmapOut.Width; ++x )
					{
						blue  = p[0];
						green = p[1];
						red   = p[2];

						//						if(red/t>green && red/t>blue){p[0]=p[1]=0; p[2]=255;}
						//						else{p[0] = p[1] = p[2] = 0;}

						//						if(green/t>red && green/t>blue){p[0]=p[2]=0; p[1]=255;}
						//						else{p[0] = p[1] = p[2] = 0;}

						//						if(blue/t>red && blue/t>green){p[1]=p[2]=0; p[0]=255;}
						//						else{p[0] = p[1] = p[2] = 0;}

						p[0] = (byte)(0.299*red + 0.587*green + 0.114*blue);
						 
						if (p[0]<threshold){p[0] = p[1] = p[2] =   0;}
						else			   {p[0] = p[1] = p[2] = 255;}
						p += 3;
					}
					p += nOffset;
				}
			}
			_bitmapOut.UnlockBits(bmData);
			return _bitmapOut;
		}


		private int max(int[,] a, int column)
		{
			int value = 0;
			int index = 0;

			for (int i=0; i<a.GetLength(0); i++)
			{
				if (a[i,column] > value)
				{
					value = a[i,column];
					index = i;
				}
			}
			return index;
		}


		private int min(int[,] a, int column)
		{
			int value = 99999999;
			int index = 99999999;

			for (int i=0; i<a.GetLength(0); i++)
			{
				if (a[i,column] < value)
				{
					value = a[i,column];
					index = i;
				}
			}
			return index;
		}


		private void sort(float[,] a, int column)
		{
			int i, j;
			float tmpX, tmpY;

			for(i=0; i<a.GetLength(0); i++)
			{
				for(j=0; j<a.GetLength(0); j++)
				{
					if(a[i,column] < a[j,column])
					{
						tmpX = a[i,0];											
						a[i,0] = a[j,0];								
						a[j,0] = tmpX;
						tmpY = a[i,1];
						a[i,1] = a[j,1];
						a[j,1] = tmpY;						
					}									
				}
			}
		}


		public void GrabPicture()
		{
			//_IPgrabber.NewPicture += new GraphController.NewPictureDelegate(this.HandleGrabImage);
			HandleGrabImage(_IPgrabber.GrabPicture());
			
			//_IPgrabber.NewPicture -= new GraphController.NewPictureDelegate(this.HandleGrabImage);
		
		}
	

		private void HandleGrabImage(System.Drawing.Bitmap e)
		{

			Image old = _bitmapIn;

			_bitmapIn = e;

			//Create Thumbnail
			BitmapData bmData   = _bitmapIn.LockBits(new Rectangle(0, 0, _bitmapIn.Width, _bitmapIn.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb); 
			_bitmapIn.UnlockBits(bmData);

			//pbox_Bitmap.Size = new System.Drawing.Size(_bitmapIn.Width, _bitmapIn.Height);

			_bitmapOut = (Bitmap)_bitmapIn.Clone();
				
								
			for (int i = 0 ; i < _point.GetLength(0); i++)
			{ 
				_point[i,0] = -1;
				_point[i,1] = -1;
			}
			_pictureloaded			 = true;


			//Announce New Picture
			if (UpdatePicture!=null)
				UpdatePicture(_bitmapIn);

			if( old != null )
				old.Dispose();

		}

		private void _Network_NewConnection(object sender, NetEventArgs e)
		{
			if (_calibrated)
			{
				System.Threading.Thread.Sleep(1000);
				_Network.Send(new clsNetTsaiConverter(e.ConID,_tsai));
			}
		}
	}
}
