using System;
using MT.Network;
using GraphController;

namespace MTClient
{
	/// <summary>
	/// Zusammenfassung fr clsNetworkClient.
	/// </summary>
	public class clsNetworkClient: IDisposable
	{
		public event NewConnectionStateDelegate NewConnectionState;
		
		public delegate void NewSyncSignalDelegate(int Value);
		public event NewSyncSignalDelegate NewSyncSignal;

		public delegate void ServerListDelegate(strConInfo[] ConInfos);
		public event ServerListDelegate ServerList;

		public delegate void RecordDelegate(bool enable);
		public delegate void PlayDelegate(bool enable);

		public event RecordDelegate Record;
		public event PlayDelegate Play;

		private clsFPnetSink	_Network;
		private strConInfo[]	_ConInfos = new strConInfo[0];

		private string			_ConnectionState = "Filter: Nicht verbunden ugaaahhh";

		public string ConnectionState
		{
			get { return _ConnectionState; }
		}

			public IFPsink FPsink
		{	
			get { return _Network; }
		}


		public clsNetworkClient()
		{
			_Network = new clsFPnetSink();
		
			//Connect Eventdelegates
			this._Network.Listening			+=new MT.Network.clsNetSemiAutoClient.ListeningDelegate(_Network_Listening);
			this._Network.Connecting		+=new MT.Network.NetEventHandler(_Network_Connecting);
			this._Network.NewConnection		+=new MT.Network.NetEventHandler(_Network_NewConnection);
			this._Network.NewMessageObject	+=new MT.Network.NetObjectHandler(_Network_NewMessageObject);
			this._Network.ClosedConnection	+=new MT.Network.NetEventHandler(_Network_ClosedConnection);
			this._Network.ServerList		+=new MT.Network.clsNetSemiAutoClient.ServerListDelegate(OnNewServerList);	

			if (NewConnectionState !=null)
				NewConnectionState(_ConnectionState);


		}

		private void OnNewServerList(strConInfo[] List)
		{
			if (_ConInfos.Length != List.Length)
			{
				_ConInfos = List;

				if (ServerList!=null)
				{
					ServerList(_ConInfos);
				}
			}
		}

		public bool Connect(strConInfo ConInfo)
		{
			_Network.SetConInfo(ConInfo);
			return true;
		}

		private void _Network_Listening(string conID)
		{
			_ConnectionState = "C<->S: Suche Server";
			if (NewConnectionState !=null)
				NewConnectionState(_ConnectionState);
		}

		private void _Network_Connecting(object sender, NetEventArgs e)
		{
			_ConnectionState = "C<->S: Verbinde...";
			if (NewConnectionState !=null)
				NewConnectionState(_ConnectionState);
		}

		private void _Network_NewConnection(object sender, NetEventArgs e)
		{
			_ConnectionState = "C<->S: Verbunden";
			if (NewConnectionState !=null)
				NewConnectionState(_ConnectionState);
		}

		private void _Network_ClosedConnection(object sender, NetEventArgs e)
		{
			//ClosedConnection(ConID);
			_ConnectionState = "C<->S: Nicht verbunden";
			if (NewConnectionState !=null)
				NewConnectionState(_ConnectionState);
		}

		#region IDisposable Member


		/// <summary>
		/// Shuts down Client Connection and frees all resources.
		/// </summary>
		public void Dispose()
		{
			if (_Network!=null) 
			{
				_Network.CloseConnection();
				_Network.Dispose();
			}		
		}

		#endregion

		private void _Network_NewMessageObject(object sender, NetObjectEventArgs e)
		{
			clsNetSyncSignal	SyncSignal;
			clsNetPlay			NetPlay ;
			clsNetRecord		NetRecord;

			if ((SyncSignal = e.NetObject as clsNetSyncSignal)!=null)
			{
				if (NewSyncSignal!=null) 
				{
					NewSyncSignal(SyncSignal.Message);
				}
			}
			else if ((NetPlay = e.NetObject as clsNetPlay)!=null)
			{
				if (Play  != null) { Play(NetPlay.Enable);}
			}
			else if ((NetRecord = e.NetObject as clsNetRecord)!=null)
			{
				if (Record  != null) { Record(NetRecord.Enable);}
			}
		}
	}
}
