using System;
using MT.Network;
using tsai;
using MT.DataStructures;
using MT.Synchronization;

namespace MTServer
{
	/// <summary>
	/// Zusammenfassung fr clsClientStatus.
	/// </summary>
	public class clsClient: Synchronizer
	{
		public		delegate void TotalFramesCountedDelegate(int totalFramesCounted, int FPBuffer);
		public		delegate void ErrorMessageDelegate(string errorMessage);
		public		delegate void MessageDelegate(string Message);
		public      delegate void CalibratedClientDelegate(string Message);

		//declare Events
		public		event TotalFramesCountedDelegate TotalFramesCounted;
		public		event CalibratedClientDelegate CalibratedClient;
		public		event ErrorMessageDelegate ErrorMessage ;
		public		event MessageDelegate Message ;


		private string							_conID;		//Connection ID
		private string							_Host;		//Hostname of the Client
		private int								_Port;		//Port number of the Client
		private string							_errorMessage;
		private string							_Message;
		private MT.Network.clsBroadcastServer	_Server;
		private int								_totalFramesCounted = 0;
		private clsNetClient					_NetClient;
		protected tsai_converter				_TSAI;
		protected FeaturePointData				_FPData;

		public FeaturePointData FPData
		{
			get { return _FPData;}
		}

		public tsai_converter Tsai
		{
			get { return _TSAI; }
		}
		public clsClient(clsNetClient NetClient, MT.Network.clsBroadcastServer Server, WorkerThread wt): base(wt)
		{
			_conID = NetClient.ConnectionID;
			_Host  = NetClient.Host;
			_Port  = NetClient.Port;
			_Server = Server;
			_NetClient = NetClient;
			_FPData = new FeaturePointData(NetClient.Host);

			_NetClient.NewMessageObject +=new NetObjectHandler(_NetClient_NewMessageObject);
			_NetClient.NewErrorMessage += new NetMessageHandler (_NetClient_NewErrorMessage);
			_NetClient.NewMessage += new NetMessageHandler (_NetClient_NewMessage);
			
			
			
		}

		private void _NetClient_NewMessageObject(object sender, NetObjectEventArgs e)
		{
			clsNetFeaturePointFrame frame  = e.NetObject as clsNetFeaturePointFrame;
			clsNetTsaiConverter tc;

			if (frame !=null)
			{
				if (frame.Frame != null)
				{
					_FPData.AddFrame(frame.Frame.fid, frame.Frame);
					
					_totalFramesCounted++;
					//Console.WriteLine("Received Frame {0}",frame.Frame.fid);
					if(TotalFramesCounted != null)
						TotalFramesCounted(_totalFramesCounted, _FPData.CountFrames());
				}
					
			}
			else if ((tc = e.NetObject as clsNetTsaiConverter)!=null)
			{
				_TSAI = tc.tsai_converter;

				Console.WriteLine("Received Calib information from Client {0}!",_Host);
				if(CalibratedClient != null)
					CalibratedClient(e.ConID);
//					foreach(Delegate Del in CalibratedClient.GetInvocationList())
//					{
//						Del.DynamicInvoke(new object[]{conID});
//						//	CalibratedClient.BeginInvoke((conID);
//					}
			}

		}

		private void _NetClient_NewErrorMessage(object sender, NetMessageEventArgs e)
		{
			_errorMessage = e.Message;

			if(ErrorMessage != null)
				ErrorMessage(_errorMessage);
		}

		private void _NetClient_NewMessage(object sender, NetMessageEventArgs e)
		{
			_Message = e.Message;

			if(Message != null)
				Message(_Message);
		}

		

		public string ConnectionID
		{
			get{return _conID;}
		}
		
		public string Host
		{
			get{return _Host;}
		}
		
		public int Port
		{
			get{return _Port;}
		}

	}
}
