using System;
using System.Collections;
using System.Timers;
using MT.Network;
using MT.Synchronization;
using MT.DataStructures;

namespace MTServer
{
	public delegate void FrameDelegate(object Sender, FrameEventArgs e);

	public class FrameEventArgs: EventArgs
	{
		private clsFPFrameArray _Frame;

		public clsFPFrameArray Frame
		{
			get {return _Frame;}
		}

		public FrameEventArgs(clsFPFrameArray frame)
		{
			_Frame = frame;
		}
	}

	/// <summary>
	/// Zusammenfassung fr clsServerController.
	/// </summary>
	public class clsServerController: Synchronizer
	{
		public event FrameDelegate newFrame;

		private		MT.Network.clsBroadcastServer	_Server;
//		private		Hashtable						_Data;
		private		clsCameras						_Cameras;
//		private		clsAssociationRGB				_Association;
		private		cls3DView						_3DView=null;
//		private		clsClientSynchronizer			_clientSynchronizer;
		private		clsFPFrameArrayContainer		_FPFrameArrayContainer;
		private		clsBPData						_BPData;
		private		TUKMain							_TUKMain;
//		private		clsCameras						_Cameras = new clsCameras();		//Collection which is filled with all cameras that are ready

		private int							_framenr = 1;


		//Flags
		private bool						_initialized = false;	//Is false as long as tracker has not been initialised
		private bool						_ready = false;			//setting this value to true, will gracefully stop the synchronizer
		private bool						_active = false;		//Is set to true by clicking the associate button
		private int							_syncSignal = 0;


		//Taktgeber
		private		System.Timers.Timer				_Clock;


		public TUKMain tukMain
		{
			get{ return _TUKMain;}
			set
			{
				_TUKMain = value;
			}
		}



		public clsServerController()
		{
			//
			// TODO: Fgen Sie hier die Konstruktorlogik hinzu
			//
		}

		public void Initialize()
		{
			//Create and Start Server
			_Server = new MT.Network.clsBroadcastServer(2347,1000);

			//Create SynchronizerContainer for FPData
			_FPFrameArrayContainer = new clsFPFrameArrayContainer();

			_BPData = new clsBPData();
			
			_Cameras = new clsCameras(base.m_WorkerThread);
			_Cameras.Initialize(_Server);

//			_clientSynchronizer = new clsClientSynchronizer(_Cameras,_Server);
//			_clientSynchronizer.Initialize(_FPFrameArrayContainer);

//			_Association = new clsAssociationRGB(_Cameras);
//			_Association.Initialize(_Association, _clientSynchronizer);

			_TUKMain = new MTServer.TUKMain(_Cameras,_BPData);
			_3DView = new cls3DView(_TUKMain);

			//ToDo: SH Event implementieren
			_TUKMain.Associated +=new EventHandler(_TUKMain_Associated);
			_Clock = new Timer(200);
			_Clock.Elapsed +=new ElapsedEventHandler(_Clock_Elapsed);
			_Clock.AutoReset = true;
			_Clock.Start();

		}

		private void _Clock_Elapsed(object sender, ElapsedEventArgs e)
		{
			if (this.InvokeRequired)
			{
				this.BeginInvoke(new ElapsedEventHandler(_Clock_Elapsed),new object[]{sender, e});
			}
			else
			{
				bool ready = CheckClients();
				//Reset Sync Signal when all clients are ready
				if (ready && !_ready)
				{
					resetSynchroSignal();
					_TUKMain.Reset();
					_initialized = false;

					//Fasten Up Synchronizer
					_Clock.Interval = 10;
					_ready = true;
				}
				else if(!ready)
				{
					_ready = false;
					_initialized = false;
					_Clock.Interval = 200;
				}
				else if (_initialized)
				{
					if (Synchronize())
					{
						_Clock.Interval = 10;
					}
					else
					{
						_Clock.Interval = 2;
					}
				}
				else if (!_initialized)
				{
					_initialized = InitSynchronize();
					_Clock.Interval = 5;

				}

				System.Threading.Thread.Sleep(1);
			}
		}

		public void StartSynchronizer()
		{	_ready = false;
			_Clock.Start();
			_active = true;
		}
		public void StopSynchronizer()
		{
			_active = false;
			_Clock.Stop();
		}


		public clsCameras ClientCollection
		{
			get{return _Cameras;}
		}

//		public clsAssociationRGB Association
//		{
//			get{return _Association;}
//		}

		public cls3DView DView
		{
			get{return _3DView;}
		}
		public clsBroadcastServer Server
		{
			get { return _Server; }
		}

		public clsBPData BPData
		{
			get { return _BPData; }
		}



//		private void _Server_NewMessageObject(string conID, clsNet message)
//		{
//			clsNetFeaturePointFrame frame  = message as clsNetFeaturePointFrame;
//			if (frame !=null)
//			{
//				if (frame.Frame != null)
//					Console.WriteLine("Received Frame {0}",frame.Frame.fid);
//			}
//		}
		

		private bool CheckClients()
		{
			bool allready = true;

			foreach(clsCamera camera in _Cameras.Values)
			{
				allready = allready && camera.Tsai!=null;
			}

			return allready;

		}


		//This function is called to guarantee that all clients are at sync signal -1
		private bool InitSynchronize()
		{
			FeaturePointFrame frame;

			foreach (clsCamera camera in _Cameras.Values)
			{

				//Try to get a certain Frame from a Camera, do nothing if unsuccessful
				if ((frame = camera.FPData.GetFrame(camera.CurFrameID)) == null)
				{
					return false;
				}


				//WP: the required frame of the camera does exist
				do
				{
					if(frame.Sync != -1)
					{
						camera.CurFrameID++;
					}
				} while((frame = camera.FPData.GetFrame(camera.CurFrameID)) != null && frame.Sync != -1);
				
				if (frame == null) {return false;}

			}

			return true;
		}

		//This Function is called to get a FeaturePointFrameArray of synchronized Frames
		private bool Synchronize()
		{	
			FeaturePointFrame[] frames = new FeaturePointFrame[_Cameras.Count];

			int i = 0;
			int maxSync=-1;

			foreach (clsCamera camera in _Cameras.Values)
			{

				//Try to get a Frame from each camera - if this fails, stop and try again later
				if ((frames[i] = camera.FPData.GetFrame(camera.CurFrameID)) == null)
				{
					return false;
				}

				i++;

			}

			//WP we got a frame from each camera and can synchronize now
			foreach(FeaturePointFrame frame in frames)
			{
				if (frame.Sync > maxSync) {maxSync = frame.Sync;}

			}

			i =0;
			bool abort = false;
			
			//Increase curFrame on all cameras that sent a frame wich has a lower sync than maxsync
			foreach (clsCamera camera in _Cameras.Values)
			{
				if (frames[i].Sync<maxSync)
				{ 
					camera.CurFrameID++; 
					abort = true;
				}

				i++;
				
			}

			if (abort) { return false; }

			//WP: we now got synchronised frames from each camera
			clsFPFrameArray  FPFrameArray = new clsFPFrameArray(_framenr++);
		

			for(i =0; i<frames.Length; i++)
			{
				//Add FPFrame to the _FPFrameArray
				FPFrameArray.Add(_Cameras.GetByIndex(i).Name,frames[i]);
			}

			if(_framenr%30 == 0)
				sendSynchroSignal();

			_FPFrameArrayContainer.Add(FPFrameArray.fid,FPFrameArray); //_FPFrame.fid
			

			OnNewFrame(FPFrameArray);

			//Increase curFrame on all cameras 
			foreach (clsCamera camera in _Cameras.Values)
			{
				camera.CurFrameID++; 				
			}

			return true;
			
		}

		private void OnNewFrame(clsFPFrameArray Frame)
		{
			if (newFrame != null)
			{
				newFrame(this, new FrameEventArgs(Frame));
			}

			if (_active)
				_TUKMain.TrackFrame(Frame);
		}

		private void resetSynchroSignal()
		{
			//Also reset the curFrames for the new sync
			foreach(clsCamera camera in _Cameras.Values)
			{
				camera.CurFrameID = camera.FPData.LastFrameID;
			}
			_syncSignal = 0;
			_Server.Send(new clsNetSyncSignal("",-1));
			
		}

		private void sendSynchroSignal()
		{
			_Server.Send(new clsNetSyncSignal("",_syncSignal++)); 
		}

		private void _TUKMain_Associated(object sender, EventArgs e)
		{
			resetSynchroSignal();
//			sendSynchroSignal();
		}

		public void ShutDown()
		{
		}
	}
}
