using System;
using tsai;
using System.Collections;
using OpenGL;
using System.IO;
using MT.DataStructures;
using MT.MathBase;
//using MT.BP3D.DataSturcutres;


namespace MTServer
{
	
	public delegate void TrackerInformationDelegate(object sender, TrackerEventArgs e);

	public class TrackerEventArgs: EventArgs
	{
		private string _info;

		public string Info
		{
			get { return _info; }
		}

		public TrackerEventArgs(string Info)
		{
			_info = Info;
		}
	}

	public class TUKElement
	{
		public event TrackerInformationDelegate TrackerInformation;


		int					_calculating;							// zeigt den Zustand der Funktion calc
		// -1 = sauber
		// n  = aktuelle FrameNr
		Hashtable			_intersectionpoints = new Hashtable();  // Schnittpunkte aller Kameras
		string				_name;
		MT.DataStructures.eColorCode			_color;
//		tsai_converter[]	_cameras;
//		vector3f[]			_cameracoords;
		int[]				_lastFedFrameID;
		int[]				_badIntersections;					//Stores number of bad intersections for each camera -> used to find out, if we received a bad featurepoint(food) from a camera

		double				_QualityThreshold = 400;	//<2cm distance and the trajectory assoziation will be broken
		
		vector2f[]			_correctionvector;
		Hashtable			_featurepoints = new Hashtable();
		clsBPStream			_BPStream;	// Contains clean 3D Points ready to be drawn
		Hashtable			_BPStreamRaw   = new Hashtable();	/* Contains raw calculated 3D Points without any
																 * cleaning for tracking quality checks
																 */
		bool				_dirty;									
//		FeaturePointData[]	_FPdata;


// ToDo: SH Remove after migrating to new Datastructure -> this is now implemented in BPStream
		//Adjacent Bodyconnectorlist
		//this list is used to consult the Bodyconnectors about
		//the propability of a 3D tracking under consideration
		//of bone physics (p.E. an ellbow is always at the same 
		//distance from a hand.
		Hashtable			_BodyConnectors= new Hashtable();

		//Debug Output
//		StreamWriter		_Console;

		clsCameras			_cameras;
//		private frm3DView	_GL3DView;

		public string Name
		{
			get{return _name;}
		}


//		public TUKElement(string name, eColorCode color, tsai_converter[] cameras, vector3f[] cameracoords)//, FeaturePointData[] FPdata)
//		{
//			_name			  = name;
//			_color			  = color;
//			_cameras		  = cameras;
//			_cameracoords	  = cameracoords;
//			//_FPdata			  = FPdata;	
//			_lastFedFrameID	  = new int[cameras.Length];
//			_correctionvector = new vector2f[cameras.Length];
//			_calculating	  = -1;
//			
//			for (int i=0; i<cameras.Length; i++)
//			{			
//				_correctionvector[i] = new vector2f(0, 0);
//				_lastFedFrameID[i]   = -1;
//			}
//			
//			//Initialize Debug Out Writer
//			//_Console = new StreamWriter(@"C:\2LED_TUKElement_" + _color + "_" + _name + ".txt");
//		}


		public TUKElement(clsBPStream BPStream, clsCameras Cameras) //string name, eColorCode color, clsCameras cameras, clsBPStreams BPStreams)
		{
			_name			  = BPStream.Name;
			_color			  = (MT.DataStructures.eColorCode)BPStream.Color;
			_cameras		  = Cameras;
//			_GL3DView         = GL3DView;
			_BPStream		  = BPStream;

			_lastFedFrameID	  = new int[_cameras.Count];
			_badIntersections = new int[_cameras.Count];
			_correctionvector = new vector2f[_cameras.Count];
			_calculating	  = -1;
			
			for (int i=0; i<_cameras.Count; i++)
			{			
				_correctionvector[i] = new vector2f(0, 0);
				_lastFedFrameID[i]   = -1;
			}
		}

		/// <summary>
		/// Calculates possible TUKElement Position.
		/// </summary>
		/// <param name="FrameID"></param>
		/// <returns>Returns number of Cameras which currently see this bodypoint</returns>
		public int calc(int FrameID)
		{
			string strInfo = "";
			int result = -1;

			if (_calculating != -1 && _calculating != FrameID) {throw new Exception("Kat.5 AAAARGH");}
			_calculating = FrameID;
			_intersectionpoints.Clear();
			_dirty = false;

			//reset badIntersections
			_badIntersections = new int[_cameras.Count];

			
			if(_featurepoints.ContainsKey(FrameID) == true) // Es ist ein Eintrag zu diesem Frame vorhanden
			{
				FeaturePoint[] curArray = (FeaturePoint[])_featurepoints[FrameID];
				
				// k = Anzahl Kameras die den Bodypoint erkannt haben
				int k=0;
				int cam = 0;
				for (int i=0; i<curArray.GetLength(0); i++)
				{
					if (curArray[i]!=null) {k++; cam=i;}
				}
				//if (k==0) {Console.WriteLine("Punkte von 0 Kameras. Berechnung nicht mglich."); return 0;} 
				if (k==0) 
				{	
					strInfo += String.Format("{0:f0} von 0 Kameras. Berechnung nicht mglich.\n",_name) ; 
					result =0;
				} 
				//if (k==1) {Console.WriteLine("Punkte von 1 Kameras. Berechnung nicht mglich."); return 1;} 
				if (k==1) 
				{
					strInfo += String.Format("{0:f0} nur {1:f0}. Berechnung nicht mglich.\n\n",_name,((clsCamera)_cameras.GetByIndex(cam)).Name);
					result = 1;} 
				if (k>=2)
				{
					//Console.WriteLine("Punkte von {0:f0} Kameras.", k);
					strInfo += String.Format("\n{0:f0} von {1:f0} Kameras.\n", _name, k);
					int m=0;
//					double xci = new double(); double yci = new double(); double zci = new double(); // Kamera 1
//					double xcj = new double(); double ycj = new double(); double zcj = new double(); // Kamera 2
//					double xwi = new double(); double ywi = new double(); double zwi = new double(); // Bodypoint in Kamera 1
//					double xwj = new double(); double ywj = new double(); double zwj = new double(); // Bodypoint in Kamera 2
					double  mx = new double(); double  my = new double(); double  mz = new double(); // Schnittpunkt
					
					vector3f wi;
					vector3f wj;
					vector3f ci;
					vector3f cj;

					double quality ; //Stores Quality of Intersection
					
					//Alle Schnittpunktkombinationen berechnen
					for (int i=0; i<_cameras.Count-1; i++)
					{
						if(curArray[i]!=null)
						{
							wi = _cameras.GetByIndex(i).image_coord_to_world_coord(curArray[i].X, curArray[i].Y);
							ci = _cameras.GetByIndex(i).Position;
//							((clsCamera)_cameras.GetByIndex(i)).image_coord_to_world_coord(curArray[i].X, curArray[i].Y, ref wi.Z, ref wi.X, ref wi.Y);
//							((clsCamera)_cameras.GetByIndex(i)).get_camera_position(ref ci.X, ref ci.Y, ref ci.Z);
							for (int j=i+1; j<_cameras.Count; j++)
							{
								if(curArray[j]!=null)
								{
									cj = _cameras.GetByIndex(j).Position;
									wj = _cameras.GetByIndex(j).image_coord_to_world_coord(curArray[j].X, curArray[j].Y); 
//									((clsCamera)_cameras.GetByIndex(j)).get_camera_position(ref cj.X, ref cj.Y, ref cj.Z);	// Kameraposition 
//									((clsCamera)_cameras.GetByIndex(j)).image_coord_to_world_coord(curArray[j].X, curArray[j].Y, ref wj.Z, ref wj.X, ref wj.Y); 
									quality = intersect(  ci.X,	ci.Y,	ci.Z,							// Schnittpunkt berechnen
										cj.X,	cj.Y,	cj.Z,
										wi.X,	wi.Y,	wi.Z,
										wj.X,	wj.Y,	wj.Z,
										ref mx, ref my,	ref mz);
									_intersectionpoints.Add(m, new vector3f((float)mx, (float)my, (float)mz)); //Schnittpunkte zur Liste hinzufgen
									
									if (quality > _QualityThreshold)
									{
										_badIntersections[i]++;
										_badIntersections[j]++;

										strInfo += "#";
									}
									
									strInfo += String.Format("{0:f0} und {1:f0}: {2:f0}: {3:f0}, {4:f0} Q:{5:f2} mm2.\n", ((clsCamera)_cameras.GetByIndex(i)).Name, ((clsCamera)_cameras.GetByIndex(j)).Name, mx, my, mz, quality);
									//Console.WriteLine("Schnittpunkt Kamera {0:f0} und {1:f0}: {2:f0}: {3:f0}, {4:f0}, {5:f0}", ((clsCamera)_cameras.GetByIndex(i)).Name, ((clsCamera)_cameras.GetByIndex(i)).Name, _name, mx, my, mz);
									m++;
								}
							}
						}
					}

					vector3f avg = CalcAverageIntersection();

					if (avg.Type == eVecType.Valid)
					{
						strInfo += String.Format("Durchschnitt: {0:f0}, {1:f0}, {2:f0}\n", avg.X, avg.Y, avg.Z);
					}

					if (_BodyConnectors.Count ==1)
					{
						foreach (BodyConnector BC in _BodyConnectors.Values)
						{
							strInfo += String.Format("Lnge des Stabes: {0}\n", BC.calcLength(FrameID));
						}
					}

					result = k; // k Kameras haben den Bodypoint erkannt
				}
			}
			

			OnTrackerInformation(strInfo);


			return result;
		}

		public void OnTrackerInformation(string strInfo)
		{
			if (TrackerInformation!=null){TrackerInformation(this, new TrackerEventArgs(strInfo));}
		}


		public int getWorstCamera()
		{
			int max=-1;
			int maxIndex=-1;

			for(int i =0; i<_badIntersections.Length; i++)
			{
				if (_badIntersections[i]>max)
				{
					max = _badIntersections[i];	
					maxIndex = i;
				}
			}

			//Return Camers with most bad Intersections if it has at least 2 bad ones.
			if (max>1)
			{
				return maxIndex;
			}
			else
			{
				return -1;
			}
		}

		private vector3f CalcAverageIntersection()
		{
			vector3f vec;
			if (_intersectionpoints.Count >= 1)
			{

				vec = new vector3f(0,0,0); // dies muss ein valid vektor sein
				foreach(DictionaryEntry DE in _intersectionpoints)
				{
					vec += (vector3f)DE.Value;
				}
				vec /= _intersectionpoints.Keys.Count;
			}
			else
				vec = new vector3f();

			return vec;

		}

		public void commit(int FrameID)
		{
			string strInfo="";

			if ( _dirty ) calc(FrameID); //Commit muss unmittelbar nach calc aufgerufen werden.

			if (_intersectionpoints.Count >= 1)
			{
				vector3f vec = CalcAverageIntersection(); // dies muss ein valid vektor sein
//				foreach(DictionaryEntry DE in _intersectionpoints)
//				{
//					vec += (vector3f)DE.Value;
//				}
//				vec /= _intersectionpoints.Keys.Count;

//				strInfo += String.Format("Durchschnitt: {0:f0}, {1:f0}, {2:f0}\n", vec.X, vec.Y, vec.Z);
				if (vec.Type == eVecType.Valid)
				{
				_BPStream.Add(new BodyPoint(vec.X,vec.Y,vec.Z,FrameID));

				FeaturePoint[] curArray = (FeaturePoint[])_featurepoints[FrameID];
					for (int i=0; i<curArray.GetLength(0)-1; i++)
					{
						if ( curArray[i] != null && curArray[i].Type == eFPType.calculated)//curArray[i] == null ||
						{
							if(_featurepoints.ContainsKey(FrameID-1) == true)
							{
								FeaturePoint[] prevArray = (FeaturePoint[])_featurepoints[FrameID-1];
								if (prevArray[i] != null)
								{
									curArray[i].previous = prevArray[i];
									prevArray[i].next    = curArray[i];
									curArray[i].tid		 = prevArray[i].tid;
									//								curArray[i].fid = _FPdata[i].GetFrame(FrameID).Count+1000;
									//								_FPdata[i].GetFrame(FrameID).Add(curArray[i].fid,curArray[i]);

									// ---Korrektur des berechneten Punktes---
									if (prevArray[i].Type == eFPType.measured)
									{						
										if(_featurepoints.ContainsKey(FrameID-2) == true)
										{
											FeaturePoint[] preprevArray = (FeaturePoint[])_featurepoints[FrameID-2];
											if (preprevArray[i] != null)
											{
												_correctionvector[i].X = (float)(2*prevArray[i].X - preprevArray[i].X - curArray[i].X);
												_correctionvector[i].Y = (float)(2*prevArray[i].Y - preprevArray[i].Y - curArray[i].Y);
									
											}
										}
									}
									curArray[i].X = (float)(curArray[i].X + _correctionvector[i].X);
									curArray[i].Y = (float)(curArray[i].Y + _correctionvector[i].Y);
									// ---------------------------------------
						
								}
							}
							//Console.WriteLine("Korrekturvektor                 : {0:f}, {1:f}", _correctionvector[i].X, _correctionvector[i].Y);
							strInfo += String.Format("Korrekturvektor                 : {0:f}, {1:f}\n", _correctionvector[i].X, _correctionvector[i].Y);
							if(curArray[i]!=null)
							{
								//Console.WriteLine("Korrigierte Bildkoordinaten     : {0:f}, {1:f}", curArray[i].X, curArray[i].Y);
								strInfo += String.Format("Korrigierte Bildkoordinaten     : {0:f}, {1:f}\n", curArray[i].X, curArray[i].Y);
							}
						}
					}
				}
			}
			//Calculation Phase finishes, therefore reset all intersectionpoints
			_intersectionpoints.Clear();
			_calculating = -1;		

			OnTrackerInformation(strInfo);
		}


		public double intersect(double	 xk1, double     yk1, double	 zk1,
							  double	 xk2, double     yk2, double	 zk2,
							  double	 xb1, double     yb1, double	 zb1,
							  double	 xb2, double     yb2, double	 zb2,
							  ref double mx,  ref double my,  ref double mz)
		{
			string strInfo="";

			double a,   b,   c,   d,   e,   f;
			double t1,  t2;
			double s1x, s1y, s1z, s2x, s2y, s2z;
			double dist2;
	
			a  = (xb1-xk1)*(xb1-xk1) + (yb1-yk1)*(yb1-yk1) + (zb1-zk1)*(zb1-zk1);
			b  = (xb1-xk1)*(xb2-xk2) + (yb1-yk1)*(yb2-yk2) + (zb1-zk1)*(zb2-zk2);
			c  = (xb1-xk1)*(xk2-xk1) + (yb1-yk1)*(yk2-yk1) + (zb1-zk1)*(zk2-zk1);

			d  = (xb1-xk1)*(xb2-xk2) + (yb1-yk1)*(yb2-yk2) + (zb1-zk1)*(zb2-zk2);
			e  = (xb2-xk2)*(xb2-xk2) + (yb2-yk2)*(yb2-yk2) + (zb2-zk2)*(zb2-zk2);
			f  = (xb2-xk2)*(xk2-xk1) + (yb2-yk2)*(yk2-yk1) + (zb2-zk2)*(zk2-zk1);
	
			t2 = (f*a-d*c)/(d*b-a*e);
			t1 = (f*b-e*c)/(d*b-a*e);

			s1x = xk1 + t1*(xb1-xk1);
			s1y = yk1 + t1*(yb1-yk1);
			s1z = zk1 + t1*(zb1-zk1);
			s2x = xk2 + t2*(xb2-xk2);
			s2y = yk2 + t2*(yb2-yk2);
			s2z = zk2 + t2*(zb2-zk2);

			mx = s1x + 0.5*(s2x-s1x);
			my = s1y + 0.5*(s2y-s1y);
			mz = s1z + 0.5*(s2z-s1z);

			dist2 = Math.Pow((s2x-s1x),2) + Math.Pow((s2y-s1y),2) + Math.Pow((s2z-s1z),2);

			//Console.Write("Distanz: {0:f2} mm. ", (Math.Abs(s2x-s1x)));
			//strInfo += String.Format("{0:f2} mm. ", (Math.Abs(s2x-s1x)));

			OnTrackerInformation(strInfo);

			return dist2;
		}


		public bool isFull(int FrameID, int CameraID)
		{
			FeaturePoint[] curArray = (FeaturePoint[])_featurepoints[FrameID];

			if (curArray[CameraID] == null) {return false;}
			else							{return ((FeaturePoint)curArray[CameraID]).Type==eFPType.measured;}
		}


		public void Feed(int FrameID, int CameraID, FeaturePoint FPoint)
		{
			if(_featurepoints.ContainsKey(FrameID) == false)
			{
				_featurepoints.Add(FrameID, new FeaturePoint[_cameras.Count]);
			}
			FeaturePoint[] curArray = (FeaturePoint[])_featurepoints[FrameID];

			curArray[CameraID] = FPoint;
			_lastFedFrameID[CameraID] = FrameID;
			_dirty = true;

			if(_featurepoints.ContainsKey(FrameID-1) == false)
			{
				_featurepoints.Add(FrameID-1, new FeaturePoint[_cameras.Count]);
			}
			FeaturePoint[] prevArray = (FeaturePoint[])_featurepoints[FrameID-1];

			if (prevArray[CameraID] != null && curArray[CameraID] != null)
			if (prevArray[CameraID].next == null && curArray[CameraID].previous == null)
			{
				prevArray[CameraID].next = curArray[CameraID];
				curArray[CameraID].previous = prevArray[CameraID];
			}
		}

		public FeaturePoint Unfeed(int FrameID, int CameraID)
		{
			if(_featurepoints.ContainsKey(FrameID) == false)
			{
				throw new Exception("Unfeed on not-existing Frame is not allowed. Check TUKController Logic for Bugs.");
			}
			FeaturePoint[] curArray = (FeaturePoint[])_featurepoints[FrameID];

			FeaturePoint FPoint = curArray[CameraID];

			_dirty = true;

			//Disconnect trajectories between Featurepoints
			if(_featurepoints.ContainsKey(FrameID-1) == false)
			{
				_featurepoints.Add(FrameID-1, new FeaturePoint[_cameras.Count]);
			}
			FeaturePoint[] prevArray = (FeaturePoint[])_featurepoints[FrameID-1];

			if (prevArray[CameraID] != null && curArray[CameraID] != null)
				if (prevArray[CameraID].next != null || curArray[CameraID].previous != null)
				{
					prevArray[CameraID].next = null;
					curArray[CameraID].previous = null;
				}

			curArray[CameraID]=null;

			return FPoint;
		}


		public FeaturePoint LastKnownPosition(int CameraID)
		{
			FeaturePoint FP = new FeaturePoint();

			if(_featurepoints.ContainsKey(_lastFedFrameID[CameraID]) == true)
			{
				FeaturePoint[] curArray = (FeaturePoint[])_featurepoints[_lastFedFrameID[CameraID]];
				if (curArray[CameraID] != null)
				{
					FP = curArray[CameraID];
				}
			}
			return FP;
		}


		public void paint(int FrameID)
		{
			vector3f vec = get3DCoord(FrameID)/1000.0f;
//			vector3f vec = getLatest3DCoord()/1000.0f;

			vector3f BPcol = new vector3f();
			switch (this._color)
			{
				case MT.DataStructures.eColorCode.R:
					BPcol.X = 1;
					BPcol.Y = 0;
					BPcol.Z = 0;
					break;
				case MT.DataStructures.eColorCode.G:
					BPcol.X = 0;
					BPcol.Y = 1;
					BPcol.Z = 0;
					break;
				case MT.DataStructures.eColorCode.B:
					BPcol.X = 0;
					BPcol.Y = 0;
					BPcol.Z = 1;
					break;
				default:
					BPcol.X = 0;
					BPcol.Y = 0;
					BPcol.Z = 0;
					break;
			}
			GL.glColor3f(BPcol.X, BPcol.Y ,BPcol.Z);

			if (vec.Type==eVecType.Valid) //&& vec.X >= 0 && vec.X >= 0 && vec.X >= 0)
			{
				GL.glTranslatef( vec.X, vec.Y, vec.Z );
				WGL.drawSphere(0.06f, 12, 12); //Pingpongball
				
				FeaturePoint[] curArray = (FeaturePoint[])_featurepoints[FrameID];
				FeaturePoint P;

				GL.glTranslatef( -vec.X, -vec.Y, -vec.Z );

//				//Pendel
//				GL.glColor3f(0,0,0);
//				WGL.drawLine(new vector3f(0.6f, 0.6f, 2.6f), vec);

//				for(int i = 0;i<_cameras.Count;i++)
//				{
//					//WGL.fillCircle(new vector3f(vec.X+0.12f,vec.Y,vec.Z),0.1f,4);
//
//					
//					P = curArray[i];
//					//Debugging: Draw cam indicators
//					if (P!=null)
//						if (P.Type == eFPType.measured)
//							WGL.fillCircleYZ(new vector3f(vec.X,vec.Y+0.12f,vec.Z+0.03f*(i-1)),0.02f,4);
//				}

				int i=0;
				vector3f color = new vector3f();
				foreach (clsCamera camera in _cameras.Values)
				{
					switch (camera.Name)
					{
						case "MT1":
							color.X = 0;
							color.Y = 0;
							color.Z = 1;
							break;
						case "MT2":
							color.X = 0;
							color.Y = 1;
							color.Z = 0;
							break;
						case "MT3":
							color.X = 0;
							color.Y = 1;
							color.Z = 1;
							break;
						case "MT4":
							color.X = 1;
							color.Y = 0;
							color.Z = 0;
							break;
						case "MT5":
							color.X = 1;
							color.Y = 0;
							color.Z = 1;
							break;
						case "MT6":
							color.X = 1;
							color.Y = 1;
							color.Z = 0;
							break;
						default:
							color.X = 0;
							color.Y = 0;
							color.Z = 0;
							break;
					}
					GL.glColor3f(color.X, color.Y ,color.Z);
										
					P = curArray[i];
					//Debugging: Draw cam indicators
					if (P!=null)
						if (P.Type == eFPType.measured)
							WGL.fillCircleYZ(new vector3f(vec.X,vec.Y+0.12f,vec.Z+0.03f*(i-1)),0.02f,4);
					i++;
				}
				
				//GL.glColor3f(0,0,0);
				//WGL.drawdottedLine(new vector3f(vec.X, vec.Y, vec.Z), new vector3f(vec.X, vec.Y, 0.0f), 0.05f); //Projektionslinie
				//WGL.fillCircle(new vector3f(vec.X, vec.Y, 0.001f), 0.10f, 32); //Schatten
			}
		}


		public FeaturePoint get2DCoord(int FrameID, int CameraID)
		{
			FeaturePoint result = null;

			//Frame wurde schon bearbeitet? -> es existiert im _featurepoints
			if(_featurepoints.ContainsKey(FrameID) == true)
			{
				FeaturePoint[] curArray = (FeaturePoint[])_featurepoints[FrameID];

				if (curArray[CameraID] == null)
				{

					//If we have a calculated positions
					if (_calculating==FrameID)
					{
						vector3f pos3D = CalcAverageIntersection();

						if (pos3D.Type == eVecType.Valid)
						{
							vector2f i;
//							double ix = new double();
//							double iy = new double();
							i = _cameras.GetByIndex(CameraID).world_coord_to_image_coord(pos3D.X, pos3D.Y, pos3D.Z);
							curArray[CameraID] = new FeaturePoint((float)i.X, (float)i.Y, _color, eFPType.calculated);

							//Console.WriteLine("Berechnete Bildkoordinaten: {0:f}, {1:f}", ix, iy);
//							_Console.WriteLine("Berechnete Bildkoordinaten      : {0:f}, {1:f}", ix, iy);
						}
					}
				}

				//WP: curArray[CameraID] existiert nun
				result = curArray[CameraID];

			}	

			return result;
		}


		public vector3f getLatest3DCoord()
		{
			int FrameID = _BPStream.Count-1;

			vector3f vec = new vector3f();

			if(_BPStream.ContainsKey(FrameID) == true)
			{
				vec = (vector3f)_BPStream[FrameID];
				/*	vector3f curvec = (vector3f)_BPStream[FrameID];
					vec.X = curvec.X;
					vec.Y = curvec.Y;
					vec.Z = curvec.Z;
				*/
			}
			return vec;
		}
        public vector3f get3DCoord(int FrameID)
		{
			vector3f vec = new vector3f();

			if(_BPStream.ContainsKey(FrameID) == true)
			{
				vec = (vector3f)_BPStream[FrameID];
			/*	vector3f curvec = (vector3f)_BPStream[FrameID];
				vec.X = curvec.X;
				vec.Y = curvec.Y;
				vec.Z = curvec.Z;
			*/
			}
			return vec;
		}

		public vector3f get3DCoordRaw(int FrameID)
		{
			vector3f vec = new vector3f();

			if(_BPStreamRaw.ContainsKey(FrameID) == true)
			{
				vec = (vector3f)_BPStreamRaw[FrameID];
				/*	vector3f curvec = (vector3f)_BPStream[FrameID];
					vec.X = curvec.X;
					vec.Y = curvec.Y;
					vec.Z = curvec.Z;
				*/
			}
			return vec;
		}

		// Todo: Remove, this function is now implemented in BPStream
		public void		newConnector(BodyConnector BC)
		{
			_BodyConnectors.Add(BC.Name,BC);
		}

		/// <summary>
		/// Returns the worst Bodyconnector Value when using the given Food and therfore gives us a hint about 
		/// the correct use of the Food. No Result possible: -1
		/// </summary>
		/// <param name="FrameID"></param>
		/// <param name="CameraID"></param>
		/// <param name="FPoint"></param>
		/// <returns>Returns a quality indicator. -1: No Indication possible</returns>
		public float	quality(int FrameID)
		{
			float r = -1;
			float q;

			//Feed(FrameID, CameraID, P);
			//calc(FrameID);

			BodyConnector BC;
			string name ="";

			foreach (DictionaryEntry DE in _BodyConnectors)
			{	BC = (BodyConnector)DE.Value;
				
				q = BC.ConnectorQuality(FrameID);
				if (q!=-1) 
					r = Math.Max(q,r);
					if (r==q)
						name=BC.Name;

			}
//			if(_name == "Huefte_Rechts")
//			Console.WriteLine("Frame Nr: {0}, Name: {1}, Wert: {2}",FrameID, name, r);
			return r;
		}

		/// <summary>
		/// Returns the Distance of 2 intersecionpoints which are most distant to each other
		/// This gives us a good quality indicator when we compare different hypothetical fitting Featurepoint
		/// </summary>
		/// <param name="FrameID">Frame Number</param>
		/// <returns>Greatest Distance between 2 Interseciont points, 0 when no or 1 intersection</returns>
		public float	intersectionQuality(int FrameID)
		{
			float r = 0;
			float t = 0;
			foreach(vector3f v1 in _intersectionpoints.Values)
			{
				foreach(vector3f v2 in _intersectionpoints.Values)
				{
					t = (v1-v2).length;
					if (t>r) {r=t;}
				}
			}

			return r;

		}

		public float	moveQuality(int FrameID)
		{
			float r = 0.0f;

			vector3f Pos1 = get3DCoord(FrameID-2);
			vector3f Pos2 = get3DCoord(FrameID-1);
			vector3f Pos3 = get3DCoord(FrameID);

			if (Pos1.Type == eVecType.Valid 
			 && Pos2.Type == eVecType.Valid 
			 && Pos3.Type == eVecType.Valid)
				r = BPHypothesis.gamma(Pos1, Pos2, Pos3);

			return r;
		}


		public void		save(StreamWriter output)
		{
			vector3f v;

			output.WriteLine(_name);

			foreach(DictionaryEntry DE in _BPStream)
			{
				v = (vector3f)DE.Value;
				output.WriteLine("{0}\t{1}\t{2}\t{3}",DE.Key,v.X,v.Y,v.Z);
			}

			//Empty Line indicates end of Bodypoint Save
			output.WriteLine("");

		}

//		public void		load(StreamReader input)
//		{	//Load part of a File structured as follow:
//			//Bodypoint_Name	
//			//NumberofFrames
//			//fid0	X0	Y0	Z0
//			//fid1	X1	Y1	Z1
//			//
//			string[] t = new string[4];
//
//			_name = input.ReadLine();
//
//			do
//			{
//				t = input.ReadLine().Split(new char[1] {'\t'},3);
//				if (t[0] != "") 
//					_BPStream.Add(t[0],new BodyPoint(float.Parse(t[1]),float.Parse(t[2]),float.Parse(t[3])));
//
//			}while  (t[0]!="");
//		}		
	}
}
