using System;

namespace MT.MathBase
{
	public enum eVecType
	{
		Invalid = -1,
		Valid = 0
	}
	/// <summary>
	/// Defines Basic 3-dimensional Vector Arithemtic Methods
	/// </summary>
	public class vector3f
	{
		protected float		_X = -1;
		protected float		_Y = -1;
		protected float		_Z = -1;

		protected eVecType _Type = eVecType.Invalid;
		
		public vector3f(){}
		public vector3f(vector3f vec)
		{
			_X = vec._X;
			_Y = vec._Y;
			_Z = vec._Z;
		}
		public vector3f(float X, float Y, float Z)
		{
			_X = X;
			_Y = Y;
			_Z = Z;
			_Type = eVecType.Valid;
		}
		public vector3f(float X, float Y, float Z, eVecType Type)
		{
			_X = X;
			_Y = Y;
			_Z = Z;
			_Type = Type;
		}

		public eVecType Type
		{
			get	{return _Type;}
		}

		public	float		X
		{
			get { return _X;  }
			set 
			{	_X = value; 
				_Type = eVecType.Valid;
			}
		}

		public	float		Y
		{
			get { return _Y;  }
			set 
			{
				_Y = value;
				_Type = eVecType.Valid;
			}
		}

		public	float		Z
		{
			get { return _Z;  }
			set 
			{ 
				_Type = eVecType.Valid;
				_Z = value; 
			}
		}


		public	float		dot(vector3f v)
		{
			return (float)_X*v._X + _Y*v._Y + _Z*v._Z; 
		}

		public vector3f		cross(vector3f v)
		{
			return new vector3f(_Y*v._Z-_Z*v._Y,-(_X*v._Z-_Z*v._X),_X*v._Y-_Y*v._X,(eVecType)Math.Min((int)_Type,(int)v._Type));
		}

		public	float		length
		{
			get { return  (float)Math.Sqrt(_X*_X + _Y*_Y + _Z*_Z); }
			set { vector3f temp = this/this.length * value;
				  this._X = temp._X;
				  this._Y = temp._Y;
				  this._Z = temp._Z;
				}
		}

		public vector3f scale(float scale)
		{
			return this*scale;
		}


		public	static float	operator *(vector3f v1, vector3f v2)
		{
			return v1.dot(v2);
		}

		public	static vector3f operator *(vector3f v1, float f)
		{
			return new vector3f(v1.X*f, v1.Y*f, v1.Z*f,v1._Type);
		}
	
		public	static vector3f operator -(vector3f v1, vector3f v2)
		{
			return new vector3f(v1._X - v2._X, v1._Y - v2._Y, v1._Z - v2._Z,(eVecType)Math.Min((int)v1._Type,(int)v2._Type));
		}

		public	static vector3f operator +(vector3f v1, vector3f v2)
		{
			return new vector3f(v1._X + v2._X, v1._Y + v2._Y, v1._Z + v2._Z,(eVecType)Math.Min((int)v1._Type,(int)v2._Type));
		}

		public	static vector3f operator /(vector3f v1, float div)
		{
			return new vector3f(v1._X/div,v1._Y/div,v1._Z/div,v1._Type);
		}


	}
}
