using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

namespace MT.Network
{
	/// <summary>
	/// A Client that listens for a server signal on a given port and automatically
	/// connects to a server which announces itself to it.
	/// Use Enable/Disable to activate or deactivate this listening mode.
	/// Disable will also disconnect Clients from an already running connection.
	/// </summary>
	public class clsNetAutoClient: clsNetClient
	{
//		public delegate void ListeningDelegate(string conID);
		public event NetEventHandler Listening;

		private static readonly IPAddress	GroupAddress	= IPAddress.Parse("224.168.100.2");
		private static readonly int			GroupPort		= 2346;

		private					Thread		_thrReceiver;

		private					bool		_Enabled = false;

		public clsNetAutoClient(string ConID)
		{
			base._conID = ConID;
			base.ClosedConnection += new NetEventHandler(_base_ClosedConnection);

		}

		/// <summary>
		/// Activates the Client Listener
		/// </summary>
		public void Enable()
		{
			_Enabled = true;
			StartListener();

		}

		/// <summary>
		/// Deactivates the Listener and/or disconnects
		/// the Client if already connected.
		/// </summary>
		public void Disable()
		{
			_Enabled = false;
		}

		public override bool Connect()
		{
			Enable();

			return true;
		}

		private void StartListener()
		{
			_thrReceiver = new Thread(new ThreadStart(Listen));
			_thrReceiver.IsBackground = true;
			_thrReceiver.Start();
		}

		private void Listen() 
		{
			UdpClient client= null;
			
			while (client==null)
				try
				{	//Try to create UdpClient on GroupPort
					client= new UdpClient(GroupPort);
					Thread.Sleep(500);
				}
				catch{}

			client.JoinMulticastGroup(GroupAddress);
			IPEndPoint iep = new IPEndPoint(IPAddress.Any, 0);
			
			//Announce ListeningState
			OnListening();

			byte[] data;
			data = client.Receive(ref iep);
			
			//Configure and start Client with Broadcast Parameters
			base._host = iep.Address.ToString();
			base._port = int.Parse(Encoding.ASCII.GetString(data));
			base.Connect();

			//Shutdown Broadcast Receiver Client
			client.DropMulticastGroup(GroupAddress);
			client.Close();
		}

		private void _base_ClosedConnection(object sender, NetEventArgs e)
		{
			if (_Enabled)
				StartListener();
		}

		#region "EventRaisers"

		private void OnListening()
		{
			if (Listening!=null){Listening(this, new NetEventArgs(base._conID));}
		}

		#endregion

		#region IDisposable Members

		public override void Dispose()
		{
			if (_thrReceiver!=null)
			{
//				_thrReceiver.Suspend();
//				_thrReceiver.Interrupt();

				_thrReceiver.Abort();
//				_thrReceiver.Join();

				_thrReceiver = null;
			}

			base.Dispose();

		}

		#endregion
	
	}
}
