// SWFilter.cpp: implementation of CSWFilter class.
// 
//////////////////////////////////////////////////////////////////////


#include "stdafx.h"

#include <fstream>
#include <iomanip>
#include <iostream>

//#include <strstream>

#include <streams.h>     // DirectShow
#include <initguid.h>    // DEFINE_GUID to declare an EXTERN_C const.
#include <stdio.h>

#include "SWFilter.h"
#include "iSWFilter.h"
#include "SWFilterProp.h"
//#include "midlswfilter.h"



using namespace std;

#include <cassert>


#define TRANSFORM_NAME L"SWFilter Filter"

// setup data - allows the self-registration to work.
const AMOVIESETUP_MEDIATYPE sudPinTypes =
{ &MEDIATYPE_NULL        // clsMajorType
, &MEDIASUBTYPE_NULL };  // clsMinorType

const AMOVIESETUP_PIN psudPins[] =
{ { L"Input"            // strName
  , FALSE               // bRendered
  , FALSE               // bOutput
  , FALSE               // bZero
  , FALSE               // bMany
  , &CLSID_NULL         // clsConnectsToFilter
  , L""                 // strConnectsToPin
  , 1                   // nTypes
  , &sudPinTypes        // lpTypes
  }
, { L"Output"           // strName
  , FALSE               // bRendered
  , TRUE                // bOutput
  , FALSE               // bZero
  , FALSE               // bMany
  , &CLSID_NULL         // clsConnectsToFilter
  , L""                 // strConnectsToPin
  , 1                   // nTypes
  , &sudPinTypes        // lpTypes
  }
};




const AMOVIESETUP_FILTER sudSWFilter =
{ &CLSID_SWFilter              // clsID
, TRANSFORM_NAME                    // strName
, MERIT_DO_NOT_USE                  // dwMerit
, 2                                 // nPins
, psudPins };                       // lpPin

// Needed for the CreateInstance mechanism
CFactoryTemplate g_Templates[]=
    {   { TRANSFORM_NAME
        , &CLSID_SWFilter
        , CSWFilter::CreateInstance
        , NULL
        , &sudSWFilter },
        { TRANSFORM_NAME L" Properties"
        , &CLSID_SWFilterProp
        , CSWFilterProp::CreateInstance }
    };
int g_cTemplates = sizeof(g_Templates)/sizeof(g_Templates[0]);

const int cintDefaultFilter = true;
const int cintDefaultNetwork = true;
const int cintDefaultPort = 2345;
const int cintDefaultThreshold = 100;
const int cintDefaultZoneThreshold = 10;
const int cblnDefaultDrawFeaturePoints = true;

///////////////////////////////////////////////////////////////////////
// CreateInstance: for COM to create a CSWFilter object
///////////////////////////////////////////////////////////////////////
CUnknown * WINAPI CSWFilter::CreateInstance(LPUNKNOWN punk, HRESULT *phr) {

    CSWFilter *pNewObject = new CSWFilter(NAME("SWFilter"), punk, phr );
    if (pNewObject == NULL) {
        *phr = E_OUTOFMEMORY;
    }

    return pNewObject;
}

///////////////////////////////////////////////////////////////////////
// CSWFilter: Constructor
///////////////////////////////////////////////////////////////////////
CSWFilter::CSWFilter(TCHAR *tszName, LPUNKNOWN punk, HRESULT *phr)
    : CTransformFilter (tszName, punk, CLSID_SWFilter),
      CPersistStream(punk, phr)
{ 
    readConfig();                  // read default value from registry
    SetParams(&m_SWFilterParams); // Set Filter parameters

	frameNr = 100000;
//	Synchro = TRUE;
	firstframe = TRUE;
	firstframehelligkeit = 0;
	//datei2.open( "C://Listen//liste.txt", ios::out);	//Dateiname zusammenfgen	
	//cout.setf(ios::fixed);
	//m_SWFilterParams.param7=0;
	//m_SWFilterParams.param5=0;

	_picFirstFrame = 0; 

	m_client = 0;
	//_ZoneMAX = 20;

}

///////////////////////////////////////////////////////////////////////
// ~CSWFilter: Destructor
///////////////////////////////////////////////////////////////////////
CSWFilter::~CSWFilter()
{

}

///////////////////////////////////////////////////////////////////////
// saveConfig: write the parameters into the registry
///////////////////////////////////////////////////////////////////////
void CSWFilter::saveConfig()
{
    char sz[STR_MAX_LENGTH];
    
	// write chkFilter into registry
    sprintf(sz, "%d", m_SWFilterParams.intFilter);
    WriteProfileStringA("SWFilter", "chkFilter", sz);

    // Write double value parameters
    sprintf(sz, "%d", m_SWFilterParams.intNet);
    WriteProfileStringA("SWFilter", "chkNetwork", sz);

	// Write double value parameters
    sprintf(sz, "%d", m_SWFilterParams.intPort);
    WriteProfileStringA("SWFilter", "intPort", sz);

	sprintf(sz, "%d", m_SWFilterParams.intThreshold);
    WriteProfileStringA("SWFilter", "intThreshold", sz);

	// write int into registry
    sprintf(sz, "%d", m_SWFilterParams.intZoneThreshold);
    WriteProfileStringA("SWFilter", "intZoneThreshold", sz);

	sprintf(sz, "%d", m_SWFilterParams.blnDrawFeaturePoints);
    WriteProfileStringA("SWFilter", "blnDrawFeaturePoints", sz);




	
}

///////////////////////////////////////////////////////////////////////
// readConfig: read the parameters from the registry
///////////////////////////////////////////////////////////////////////
void CSWFilter::readConfig()
{
    m_SWFilterParams.intFilter = GetProfileInt("SWFilter", "chkFilter", cintDefaultFilter);

    m_SWFilterParams.intNet = GetProfileInt("SWFilter", "chkNetwork", cintDefaultNetwork);	
    
	m_SWFilterParams.intPort = GetProfileInt("SWFilter", "intPort", cintDefaultPort);

	m_SWFilterParams.intThreshold = GetProfileInt("SWFilter", "intThreshold", cintDefaultThreshold);

	m_SWFilterParams.intZoneThreshold = GetProfileInt("SWFilter", "intZoneThreshold", cintDefaultZoneThreshold);

	m_SWFilterParams.blnDrawFeaturePoints = GetProfileInt("SWFilter", "blnDrawFeaturePoints", cblnDefaultDrawFeaturePoints);

// Example: How to store a String into the Registry
//	char *tmp = new char[1];
//  m_SWFilterParams.param4 = GetProfileStringA("SWFilter", "Parameters4", tmp, tmp, 7);
	
    
}

///////////////////////////////////////////////////////////////////////
// CheckInputType: Check if the input type can be done
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilter::CheckInputType(const CMediaType *mtIn)
{
    if (canPerformTransform(mtIn))
        return S_OK;
    else
        return VFW_E_TYPE_NOT_ACCEPTED;
}

///////////////////////////////////////////////////////////////////////
// Checktransform: Check a transform can be done between these formats
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilter::CheckTransform(const CMediaType *mtIn, const CMediaType *mtOut)
{
    if (canPerformTransform(mtIn)) {
        if (*mtIn == *mtOut) {
            return NOERROR;
        }
    }
    return E_FAIL;
}

///////////////////////////////////////////////////////////////////////
// DecideBufferSize: Tell the output pin's allocator what size buffers 
// we require. Can only do this when the input is connected
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilter::DecideBufferSize(IMemAllocator *pAlloc,
                             ALLOCATOR_PROPERTIES *pProperties)
{
    // Is the input pin connected
    if (m_pInput->IsConnected() == FALSE) {
        return E_UNEXPECTED;
    }

    ASSERT(pAlloc);
    ASSERT(pProperties);
    HRESULT hr = NOERROR;

    pProperties->cBuffers = 1;

    // Get input pin's allocator size and use that
    ALLOCATOR_PROPERTIES InProps;
    IMemAllocator * pInAlloc = NULL;
    hr = m_pInput->GetAllocator(&pInAlloc);
    if (SUCCEEDED (hr))
    {
        hr = pInAlloc->GetProperties (&InProps);
        if (SUCCEEDED (hr))
        {
            pProperties->cbBuffer = InProps.cbBuffer;
        }
        pInAlloc->Release();
    }

    if (FAILED(hr))
        return hr;

    ASSERT(pProperties->cbBuffer);

    // Ask the allocator to reserve us some sample memory, NOTE the function
    // can succeed (that is return NOERROR) but still not have allocated the
    // memory that we requested, so we must check we got whatever we wanted

    ALLOCATOR_PROPERTIES Actual;
    hr = pAlloc->SetProperties(pProperties,&Actual);
    if (FAILED(hr)) {
        return hr;
    }

    ASSERT( Actual.cBuffers == 1 );

    if ( pProperties->cBuffers > Actual.cBuffers ||
         pProperties->cbBuffer > Actual.cbBuffer) {
                return E_FAIL;
    }
    return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// GetMediaType: I support one type, namely the type of the input pin
// This type is only available if my input is connected
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilter::GetMediaType(int iPosition, CMediaType *pMediaType)
{
    // Is the input pin connected
    if (m_pInput->IsConnected() == FALSE) {
        return E_UNEXPECTED;
    }
    // This should never happen
    if (iPosition < 0) {
        return E_INVALIDARG;
    }
    // Do we have more items to offer
    if (iPosition > 0) {
        return VFW_S_NO_MORE_ITEMS;
    }

    *pMediaType = m_pInput->CurrentMediaType();
    return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// Transform (for CTransformFitler)
//
// Copy the input sample into the output sample - then transform the 
// output sample 'in place'. If we have all keyframes, then we should
// not do a copy. If we have cinepak or indeo and are decompressing 
// frame N it needs frame decompressed frame N-1 available to calculate
// it, unless we are at a keyframe. So with keyframed codecs, you can't 
// get away with applying the transform to change the frames in place, 
// because you'll mess up the next frames decompression. The runtime 
// MPEG decoder does not have keyframes in the same way so it can be 
// done in place. We know if a sample is key frame as we transform 
// because the sync point property will be set on the sample
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilter::Transform(IMediaSample *pIn, IMediaSample *pOut)
{
    // Copy the properties across
    HRESULT hr = copyMediaSample(pIn, pOut);
    if (FAILED(hr)) {
        return hr;
    }
    // Transform the output media sample in-place
    return Transform(pOut);
}

///////////////////////////////////////////////////////////////////////
// copyMediaSample: Make destination an identical copy of source
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilter::copyMediaSample(IMediaSample *pSource, IMediaSample *pDest) const
{
    // Copy the sample data
    BYTE *pSourceBuffer, *pDestBuffer;
    long lSourceSize = pSource->GetActualDataLength();
    long lDestSize  = pDest->GetSize();

    ASSERT(lDestSize >= lSourceSize);

    pSource->GetPointer(&pSourceBuffer);
    pDest->GetPointer(&pDestBuffer);

    CopyMemory( (PVOID) pDestBuffer,(PVOID) pSourceBuffer,lSourceSize);

    // Copy the sample times
    REFERENCE_TIME TimeStart, TimeEnd;
    if (NOERROR == pSource->GetTime(&TimeStart, &TimeEnd)) {
        pDest->SetTime(&TimeStart, &TimeEnd);
    }

    LONGLONG MediaStart, MediaEnd;
    if (pSource->GetMediaTime(&MediaStart,&MediaEnd) == NOERROR) {
        pDest->SetMediaTime(&MediaStart,&MediaEnd);
    }

    // Copy the Sync point property
    HRESULT hr = pSource->IsSyncPoint();
    if (hr == S_OK) {
        pDest->SetSyncPoint(TRUE);
    }
    else if (hr == S_FALSE) {
        pDest->SetSyncPoint(FALSE);
    }
    else {  // an unexpected error has occured...
        return E_UNEXPECTED;
    }

    // Copy the media type
    AM_MEDIA_TYPE *pMediaType;
    pSource->GetMediaType(&pMediaType);
    pDest->SetMediaType(pMediaType);
    DeleteMediaType(pMediaType);

    // Copy the preroll property
    hr = pSource->IsPreroll();
    if (hr == S_OK) {
        pDest->SetPreroll(TRUE);
    }
    else if (hr == S_FALSE) {
        pDest->SetPreroll(FALSE);
    }
    else {  // an unexpected error has occured...
        return E_UNEXPECTED;
    }

    // Copy the discontinuity property
    hr = pSource->IsDiscontinuity();
    if (hr == S_OK) {
    pDest->SetDiscontinuity(TRUE);
    }
    else if (hr == S_FALSE) {
        pDest->SetDiscontinuity(FALSE);
    }
    else {  // an unexpected error has occured...
        return E_UNEXPECTED;
    }

    // Copy the actual data length
    long lDataLength = pSource->GetActualDataLength();
    pDest->SetActualDataLength(lDataLength);

    return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// canPerformTransform: We support RGB24 and RGB32 input
///////////////////////////////////////////////////////////////////////
BOOL CSWFilter::canPerformTransform(const CMediaType *pMediaType) const
{
    // we accept the following image type: (RGB24, ARGB32 or RGB32)
    if (IsEqualGUID(*pMediaType->Type(), MEDIATYPE_Video)) {
        if (IsEqualGUID(*pMediaType->Subtype(), MEDIASUBTYPE_RGB24)) {
            VIDEOINFOHEADER *pvi = (VIDEOINFOHEADER *) pMediaType->Format();
            return (pvi->bmiHeader.biBitCount == 24);
        }
        if (IsEqualGUID(*pMediaType->Subtype(), MEDIASUBTYPE_ARGB32)) {
            VIDEOINFOHEADER *pvi = (VIDEOINFOHEADER *) pMediaType->Format();
            return (pvi->bmiHeader.biBitCount == 32);
        }
        if (IsEqualGUID(*pMediaType->Subtype(), MEDIASUBTYPE_RGB32)) {
            VIDEOINFOHEADER *pvi = (VIDEOINFOHEADER *) pMediaType->Format();
            return (pvi->bmiHeader.biBitCount == 32);
        }
    }
    return FALSE;
} 

/*
"suche" ist eine rekursive Funktion, welche die Pixel zu einer Zone hinzufgt, welche die 3 folgenden Bedingungen erfllen:
1. Der Pixel muss ber dem Schwellwert liegen.
   Dies geschieht ber eine Umformung des Farbbildes in ein Grauwertbild.
2. Der Pixel muss auf der gleichen Zeile sein wie die Maske zeigt.
   Damit wird verhindert, dass sich Zonen am linken und rechten Bildrand zu einer verschmelzen.
3. Der neue Pixel muss die gleiche Farbe haben wie der vorhergehende Pixel der Zone.

*/

void CSWFilter::suche( int		   cxImage,
					   int		   cyImage,
					   int		   aktZoneNr,		
					   BYTE*	   prgb,			
					   int		   iPixelSize,		
					   Zonenarray* Liste,			
					   BYTE*	   prgbsave,
					   int		   tiefe)		
{
	if (tiefe < 50)
	{
		tiefe++;
	    int Pixel  = (prgb-prgbsave)/iPixelSize;						// Aktuelle Position im Frame berechnen
		
		if( Pixel > (cxImage + 1) && Pixel < ((cxImage * cyImage) - cxImage - 1) ) // Ist der Pixel innerhalb des Bildes?
		{
	        char color = maxFarbe( *(prgb + 2), *(prgb + 1), *prgb);	// Farbe des Pixels bestimmen
	        Liste->Nr[aktZoneNr]  = aktZoneNr;
	    	Liste->X[aktZoneNr]  += (int)Pixel%cxImage;
	    	Liste->Y[aktZoneNr]  += (int)Pixel/cxImage;
	        Liste->R[aktZoneNr]  += *(prgb + 2) - rotanteil;			// Durchschnittlicher Rotanteil des ersten Frame.
	    	Liste->G[aktZoneNr]  += *(prgb + 1) - gruenanteil;
	    	Liste->B[aktZoneNr]  += *(prgb    ) - blauanteil; 
	    	Liste->Gw[aktZoneNr]++;										//   Anzahl Pixel, die an einer Zone beteiligt sind
	    	*(prgb    ) = 0;											//  |
	    	*(prgb + 1) = 0;											//   >Pixel schwarz frben
	    	*(prgb + 2) = 0;											//  |
	
	        
	    	//unten links
	    	if ( (0.11 * *(prgb-((cxImage+1)*iPixelSize)     ) + 
	    		  0.69 * *(prgb-((cxImage+1)*iPixelSize)  + 1) +
	    		  0.4  * *(prgb-((cxImage+1)*iPixelSize)  + 2))  > m_SWFilterParams.intThreshold &&
				  (Pixel-cxImage+1) / cxImage  == (Pixel / cxImage) + 1 &&
	              color == maxFarbe(*(prgb-((cxImage+1)*iPixelSize) + 2 ),
	                                *(prgb-((cxImage+1)*iPixelSize) + 1 ),
	                                *(prgb-((cxImage+1)*iPixelSize)     ) ) )
	
	    	{ suche( cxImage, cyImage, aktZoneNr, prgb-((cxImage+1)*iPixelSize), iPixelSize, Liste, prgbsave, tiefe);} 
	   
	        
	    	//links
	    	if ( (0.11 * *(prgb-iPixelSize    ) +
	    		  0.69 * *(prgb-iPixelSize + 1) +
	    		  0.4  * *(prgb-iPixelSize + 2))  > m_SWFilterParams.intThreshold &&
				  (Pixel-iPixelSize)/ cxImage == ( Pixel / cxImage) &&
	              color == maxFarbe(*(prgb-iPixelSize + 2),
	                                *(prgb-iPixelSize + 1),
	                                *(prgb-iPixelSize    ) ) )
	
	    	{ suche( cxImage, cyImage, aktZoneNr, prgb-(1*iPixelSize)          , iPixelSize, Liste, prgbsave, tiefe);} 
	 
	        
	    	//oben links
	    	if ( (0.11 * *(prgb+((cxImage-1)*iPixelSize)    ) + 
	    		  0.69 * *(prgb+((cxImage-1)*iPixelSize) + 1) + 
	    		  0.4  * *(prgb+((cxImage-1)*iPixelSize) + 2))  > m_SWFilterParams.intThreshold &&
				  (Pixel+cxImage-1) / cxImage == (Pixel / cxImage) - 1 &&
	              color == maxFarbe(*(prgb+((cxImage-1)*iPixelSize) + 2),
	                                *(prgb+((cxImage-1)*iPixelSize) + 1),
	                                *(prgb+((cxImage-1)*iPixelSize)    ) ) )
	  
	        { suche( cxImage, cyImage, aktZoneNr, prgb+((cxImage-1)*iPixelSize), iPixelSize, Liste, prgbsave, tiefe);} 
	 
	        
	    	//oben
	    	if ( (0.11 * *(prgb+(cxImage*iPixelSize)    ) + 
	    	   	  0.69 * *(prgb+(cxImage*iPixelSize) + 1) + 
	    		  0.4  * *(prgb+(cxImage*iPixelSize) + 2))  > m_SWFilterParams.intThreshold &&
	              color == maxFarbe(*(prgb+(cxImage*iPixelSize) + 2),
	                                *(prgb+(cxImage*iPixelSize) + 1),
	                                *(prgb+(cxImage*iPixelSize)    ) ) )
	 
	        { suche( cxImage, cyImage, aktZoneNr, prgb+((cxImage)*iPixelSize),   iPixelSize, Liste, prgbsave, tiefe);} 
	  
	        
	    	//oben rechts
	    	if ( (0.11 * *(prgb+((cxImage+1)*iPixelSize)    ) + 
	    		  0.69 * *(prgb+((cxImage+1)*iPixelSize) + 1) + 
	    		  0.4  * *(prgb+((cxImage+1)*iPixelSize) + 2))  > m_SWFilterParams.intThreshold &&
				  (Pixel+cxImage+1) / cxImage == (Pixel / cxImage) +1 &&
	              color == maxFarbe(*(prgb+((cxImage+1)*iPixelSize) + 2),
	                                *(prgb+((cxImage+1)*iPixelSize) + 1),
	                                *(prgb+((cxImage+1)*iPixelSize)    ) ) )
	  
	        { suche( cxImage, cyImage, aktZoneNr, prgb+((cxImage+1)*iPixelSize), iPixelSize, Liste, prgbsave, tiefe);} 
	  
	        
	    	//rechts
	    	if ( (0.11 * *(prgb+iPixelSize    ) + 
	    	      0.69 * *(prgb+iPixelSize + 1) + 
	    		  0.4  * *(prgb+iPixelSize + 2))  > m_SWFilterParams.intThreshold &&
				  (Pixel+iPixelSize)/ cxImage == ( Pixel / cxImage) &&
	              color == maxFarbe(*(prgb+iPixelSize + 2),
	                                *(prgb+iPixelSize + 1),
	                                *(prgb+iPixelSize    ) ) )
	  
	        { suche( cxImage, cyImage, aktZoneNr, prgb+(1*iPixelSize)          , iPixelSize, Liste, prgbsave, tiefe);} 
	  
	        
	    	//unten rechts
	    	if ( (0.11 * *(prgb-((cxImage-1)*iPixelSize)    ) + 
	    		  0.69 * *(prgb-((cxImage-1)*iPixelSize) + 1) + 
	    		  0.4  * *(prgb-((cxImage-1)*iPixelSize) + 2))  > m_SWFilterParams.intThreshold &&
				  (Pixel-cxImage-1) / cxImage  == (Pixel / cxImage) +1 &&
	              color == maxFarbe(*(prgb-((cxImage-1)*iPixelSize) + 2),
	                                *(prgb-((cxImage-1)*iPixelSize) + 1),
	                                *(prgb-((cxImage-1)*iPixelSize)    ) ) )
	 
	        { suche( cxImage, cyImage, aktZoneNr, prgb-((cxImage-1)*iPixelSize), iPixelSize, Liste, prgbsave, tiefe);} 
	  
	        
	    	//unten
	    	if ( (0.11 * *(prgb-(cxImage*iPixelSize)    ) + 
	    		  0.69 * *(prgb-(cxImage*iPixelSize) + 1) +
	    		  0.4  * *(prgb-(cxImage*iPixelSize) + 2))  > m_SWFilterParams.intThreshold &&
	              color == maxFarbe(*(prgb-(cxImage*iPixelSize) + 2),
	                                *(prgb-(cxImage*iPixelSize) + 1),
	                                *(prgb-(cxImage*iPixelSize)    ) ) )
	 
	        { suche( cxImage, cyImage, aktZoneNr, prgb-((cxImage)*iPixelSize),   iPixelSize, Liste, prgbsave, tiefe);} 
		}
	}
}



char CSWFilter::maxFarbe( float rot, float gruen, float blau)
{
	// rot = 0, gruen = 1, blau = 2, weiss = 3
	if (rot>gruen && rot>blau)   {return 'R';}
	if (gruen>rot && gruen>blau) {return 'G';}
    else                         { if (blau-gruen > 60) {return 'B';}
								   else					{return 'B';}}	// weiss
}


char* CSWFilter::concat(char* a, char* b, char* c)
{
	char *r  = new char[256];
	char *ri = r;
	while (*a!='\0') {*ri++ = *a++;}
	while (*b!='\0') {*ri++ = *b++;}
	while (*c!='\0') {*ri++ = *c++;}
	*ri='\0';
	return r;
}


void CSWFilter::drawCross(BYTE* prgb, int cxImage, int cyImage, int iPixelSize, int xCoord, int yCoord, int Gewicht, char Farbe)
{
	int rot, gruen, blau = 0;
    if (Farbe == 'R') {rot = 255; gruen =   0; blau =   0;}
    if (Farbe == 'G') {rot =   0; gruen = 255; blau =   0;}
    if (Farbe == 'B') {rot =   0; gruen =   0; blau = 255;}
    if (Farbe == 'W') {rot = 255; gruen = 255; blau = 255;}

	int limit = Gewicht/20;
	for (int x=-limit; x<=limit; x++)
    {
		// Horizontale Linie
        if ( (prgb + iPixelSize*((yCoord*cxImage) + xCoord + x)) > prgb &&
             (prgb + iPixelSize*((yCoord*cxImage) + xCoord + x)) < prgb + iPixelSize*cxImage*cyImage &&
			 yCoord == yCoord + ((xCoord + x) / cxImage)  &&
			 (xCoord + x) >= 0 )
        {
            *(prgb + iPixelSize*((yCoord*cxImage) + xCoord + x)    ) = blau;
            *(prgb + iPixelSize*((yCoord*cxImage) + xCoord + x) + 1) = gruen;
            *(prgb + iPixelSize*((yCoord*cxImage) + xCoord + x) + 2) = rot;
   /*     
            *(prgb + iPixelSize*(((yCoord+1)*cxImage) + xCoord + x)    ) = blau;
            *(prgb + iPixelSize*(((yCoord+1)*cxImage) + xCoord + x) + 1) = gruen;
            *(prgb + iPixelSize*(((yCoord+1)*cxImage) + xCoord + x) + 2) = rot;

			*(prgb + iPixelSize*(((yCoord-1)*cxImage) + xCoord + x)    ) = blau;
            *(prgb + iPixelSize*(((yCoord-1)*cxImage) + xCoord + x) + 1) = gruen;
            *(prgb + iPixelSize*(((yCoord-1)*cxImage) + xCoord + x) + 2) = rot;
*/
        }

		// Vertikale Linie
		if ( (prgb + iPixelSize*((yCoord*cxImage) + xCoord + x*cxImage)) > prgb &&
             (prgb + iPixelSize*((yCoord*cxImage) + xCoord + x*cxImage)) < prgb + iPixelSize*cxImage*cyImage)
        {
            *(prgb + iPixelSize*((yCoord*cxImage) + xCoord + x*cxImage)    ) = blau;
            *(prgb + iPixelSize*((yCoord*cxImage) + xCoord + x*cxImage) + 1) = gruen;
            *(prgb + iPixelSize*((yCoord*cxImage) + xCoord + x*cxImage) + 2) = rot;
/*
			*(prgb + iPixelSize*((yCoord*cxImage+1) + xCoord + x*cxImage)    ) = blau;
            *(prgb + iPixelSize*((yCoord*cxImage+1) + xCoord + x*cxImage) + 1) = gruen;
            *(prgb + iPixelSize*((yCoord*cxImage+1) + xCoord + x*cxImage) + 2) = rot;

			*(prgb + iPixelSize*((yCoord*cxImage-1) + xCoord + x*cxImage)    ) = blau;
            *(prgb + iPixelSize*((yCoord*cxImage-1) + xCoord + x*cxImage) + 1) = gruen;
            *(prgb + iPixelSize*((yCoord*cxImage-1) + xCoord + x*cxImage) + 2) = rot;
*/
		}     
	}
}


///////////////////////////////////////////////////////////////////////
// Transform: Transforms each image frame in-place
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilter::Transform(IMediaSample *pMediaSample)
{

    CAutoLock cAutolock(&m_SWFilterLock);
    // get current media type
    AM_MEDIA_TYPE* pmt   = &(m_pInput->CurrentMediaType());
    VIDEOINFOHEADER *pvi = (VIDEOINFOHEADER *) pmt->pbFormat;
    
    BYTE *pData;                // Pointer to the actual image buffer
    long lDataLen;              // Holds length of any given sample
    int iPixel;                 // Used to loop through the image pixels
    pMediaSample->GetPointer(&pData);
    lDataLen = pMediaSample->GetSize();

    // Get the image properties from the BITMAPINFOHEADER
    int iPixelSize    = pvi->bmiHeader.biBitCount / 8;
    int cxImage       = pvi->bmiHeader.biWidth;
    int cyImage       = pvi->bmiHeader.biHeight;
    int cbImage       = cyImage * cxImage * iPixelSize;
    int numPixels     = cxImage * cyImage;

	Zonenarray Liste;
	int  aktZoneNr    = 0;
	int  i, j         = 0;
	int  AnzahlZonen  = 0;
	BYTE *prgb        = (BYTE*) pData;
	BYTE *prgbsave    = (BYTE*) pData;
	int	 Helligkeit	  = 0;
	int Sync		  = m_SWFilterParams.intSync;
	BYTE* pFirst = _picFirstFrame;
	static bool firstdarkframe = true;


	//Initialise Network Client if not done yet
	if (m_client==0 && m_SWFilterParams.intNet == TRUE) 
	{	m_client = new ClientSocket("127.0.0.1", m_SWFilterParams.intPort);
		//ToDo: Set Connection ID by using properties
		//Send connectionID
		(*m_client) << "SWFilter_MT3\n";
		m_client->flush();
	}
	else if (m_client != 0 && m_SWFilterParams.intNet == FALSE)
	{
		//ToDo: eventually close connection first
		delete m_client;
		m_client=0;
	}

	//Initialize Zonelist
	for (i=0; i<50; i++) {Liste.Nr[i] = 0; Liste.X[i] = 0; Liste.Y[i] = 0; Liste.R[i]  = 0;
						  Liste.G[i]  = 0; Liste.B[i] = 0; Liste.F[i][0] = 0; Liste.F[i][1] = '\0'; Liste.Gw[i] = 0;}
	{
		//ToDo: moegliche Problemquelle
		//m_SWFilterParams.param1 = 1;
	}

	// Transform
    if (m_SWFilterParams.intFilter)
    {

		//Helle Pixel mit keiner klaren Farbvarianz deaktivieren
		// ToDo: Grauwert Threshold als Parameter implementieren
		//for (iPixel=0; iPixel < numPixels; iPixel++ ) 
		//{
			//if (    prgb[iPixelSize * iPixel    ] < 130 &&
			//		prgb[iPixelSize * iPixel + 1] < 130 &&
			//		prgb[iPixelSize * iPixel + 2] < 130 )
			//{
			//	prgb[iPixelSize * iPixel    ] = 0;
			//	prgb[iPixelSize * iPixel + 1] = 0;
			//	prgb[iPixelSize * iPixel + 2] = 0;
			//}
		//}

		
		// erstes Frame berprfen
		// ***********************
		// Dies ist das erste Frame im Dunkeln. Es wird jeweils der Mittelwert der drei Farbkanle gebildet,
		// und anschliessend im Algorithmus bercksichtigt resp. abgezogen. Dies war ntig, da der Blaukanal
		// bei einem dunklen Bild bereits einen Wert von etwa 40 hatte, whrend Rot und Grn zwischen 0 und 5
		// schwankten. 
		if (firstdarkframe)
		{
			firstdarkframe= false;

			_picFirstFrame = (BYTE *)malloc(cbImage*sizeof(BYTE));
			pFirst = _picFirstFrame;

			blackpixel   = 0;
			rotanteil    = 0;
			gruenanteil  = 0;
			blauanteil   = 0; 

			for (iPixel=0; iPixel < numPixels; iPixel++ ) 
			{
				pFirst[iPixelSize * iPixel    ] = prgb[iPixelSize * iPixel    ];
				pFirst[iPixelSize * iPixel + 1] = prgb[iPixelSize * iPixel + 1];
				pFirst[iPixelSize * iPixel + 2] = prgb[iPixelSize * iPixel + 2];

				if((0.11 * prgb[iPixelSize * iPixel    ] + 
					0.69 * prgb[iPixelSize * iPixel + 1] + 
					0.4  * prgb[iPixelSize * iPixel + 2] ) < m_SWFilterParams.intThreshold)
				{
					blackpixel ++;
					rotanteil   += prgb[iPixelSize * iPixel + 2];
					gruenanteil += prgb[iPixelSize * iPixel + 1];
					blauanteil  += prgb[iPixelSize * iPixel    ];
				}
			}
			rotanteil   /= blackpixel;
			gruenanteil /= blackpixel;
			blauanteil  /= blackpixel;

			//rotanteil   =0;
			//gruenanteil =0;
			//blauanteil  =0;
			pFirst = _picFirstFrame;
		}


		// ToDo: Moegliche Hintergrundsubratktion aktivieren um Lichtkomponenten auszuschliessen
		//for (iPixel=0; iPixel < numPixels; iPixel++ ) 
		//{	if (abs(pFirst[iPixelSize * iPixel    ] - prgb[iPixelSize * iPixel    ]) < 20 &&
		//		abs(pFirst[iPixelSize * iPixel + 1] - prgb[iPixelSize * iPixel + 1]) < 20 &&
		//		abs(pFirst[iPixelSize * iPixel + 2] - prgb[iPixelSize * iPixel + 2]) < 20 )
		//	{
		//		prgb[iPixelSize * iPixel    ] = 0;
		//		prgb[iPixelSize * iPixel + 1] = 0;
		//		prgb[iPixelSize * iPixel + 2] = 0;
		//	}
		//}
		// 1. Zonensuche
		// ****************

	    // erste Zeile schwarz frben

	    for (iPixel=0; iPixel<cxImage+1; iPixel ++)
		{
			prgb[iPixelSize * iPixel    ] = 0;
	    	prgb[iPixelSize * iPixel + 1] = 0;
			prgb[iPixelSize * iPixel + 2] = 0;
		}
	    // letzte Zeile schwarz frben

		for (iPixel=numPixels-cxImage-1; iPixel<numPixels; iPixel ++)
		{
			prgb[iPixelSize * iPixel    ] = 0;
	    	prgb[iPixelSize * iPixel + 1] = 0;
			prgb[iPixelSize * iPixel + 2] = 0;
		}

		prgb+=(cxImage+1)*iPixelSize;
		for (iPixel=cxImage+1; iPixel < numPixels-cxImage-1; iPixel++, prgb+=iPixelSize) 
		{
			if((0.11 * *(prgb    ) +
				0.69 * *(prgb + 1) +
				0.4  * *(prgb + 2)   ) > m_SWFilterParams.intThreshold)	
	        {
			
				//if (aktZoneNr < 13)
				//if (aktZoneNr < 49)
				if (aktZoneNr < _ZoneMAX-1)
				{
					aktZoneNr++;
					suche( cxImage, cyImage, aktZoneNr, prgb, iPixelSize, &Liste, prgbsave, 0);
	
					//Falls weniger als "Parameter 3" Pixel an einer Zone beteiligt sind, wird die Zone gelscht
					if ( Liste.Gw[aktZoneNr] < m_SWFilterParams.intZoneThreshold)
					{
						Liste.Nr[aktZoneNr] = 0;
						Liste.X[aktZoneNr]  = 0;
						Liste.Y[aktZoneNr]  = 0;
						Liste.R[aktZoneNr]  = 0;
						Liste.G[aktZoneNr]  = 0;
						Liste.B[aktZoneNr]  = 0;
						Liste.F[aktZoneNr][0] = 0;
						Liste.Gw[aktZoneNr] = 0;
						aktZoneNr--;
					}
					
					else
					{
						Liste.X[aktZoneNr] = Liste.X[aktZoneNr] / Liste.Gw[aktZoneNr];	// Durchschnitt bilden
						Liste.Y[aktZoneNr] = Liste.Y[aktZoneNr] / Liste.Gw[aktZoneNr];
						Liste.R[aktZoneNr] = Liste.R[aktZoneNr] / Liste.Gw[aktZoneNr];
						Liste.G[aktZoneNr] = Liste.G[aktZoneNr] / Liste.Gw[aktZoneNr];
						Liste.B[aktZoneNr] = Liste.B[aktZoneNr] / Liste.Gw[aktZoneNr];
						Liste.F[aktZoneNr][0] = maxFarbe(Liste.R[aktZoneNr], Liste.G[aktZoneNr], Liste.B[aktZoneNr]);
					}
				}
			}
	        *(prgb    ) = 0;
	    	*(prgb + 1) = 0;
			*(prgb + 2) = 0;
		}


		// 2. Benachbarte Zonen innerhalb 10 Pixel und gleicher Farbe zusammenfassen
		// *************************************************************************
		AnzahlZonen = aktZoneNr;
		for (i=1; i<=aktZoneNr; i++)
		{
			for (j=i+1; j<=aktZoneNr; j++)
			{
				if ((abs((int)(Liste.X[i] - Liste.X[j]))) < 10 &&
					(abs((int)(Liste.Y[i] - Liste.Y[j]))) < 10 &&
						Liste.F[i][0] == Liste.F[j][0]            &&
						i != j)
				{
					Liste.X [i] = (Liste.X[i]*Liste.Gw[i] + Liste.X[j]*Liste.Gw[j]) / (Liste.Gw[i]+Liste.Gw[j]);
					Liste.Y [i] = (Liste.Y[i]*Liste.Gw[i] + Liste.Y[j]*Liste.Gw[j]) / (Liste.Gw[i]+Liste.Gw[j]);
					Liste.R [i] = (Liste.R[i] + Liste.R[j]) / 2;
					Liste.G [i] = (Liste.G[i] + Liste.G[j]) / 2;
					Liste.B [i] = (Liste.B[i] + Liste.B[j]) / 2;
					Liste.Gw[i] = Liste.Gw[i] + Liste.Gw[j];

					Liste.Nr[j] = 0;
					Liste.X [j] = 0;
					Liste.Y [j] = 0;
					Liste.R [j] = 0;
					Liste.G [j] = 0;
					Liste.B [j] = 0;
					Liste.F [j][0] = 0;
					Liste.Gw[j] = 0;
					AnzahlZonen--;
				}
			}
		}

		// Kreuze zeichnen
		// ***************
		if (m_SWFilterParams.blnDrawFeaturePoints)
		{
			for (i=0; i<=aktZoneNr; i++)
			{
				if (Liste.Nr[i] != 0)
				{
					prgb = prgbsave;
					drawCross( prgb, cxImage, cyImage, iPixelSize, (int)Liste.X[i], (int)Liste.Y[i], Liste.Gw[i], Liste.F[i][0]);
				}
			}
		}
		
		char buffer[20];

		if(m_client!=0)
		{
			//(*m_client) << "Frame:";
			(*m_client) << itoa(frameNr - 100000,buffer,10);
			(*m_client) << ";";	
			//
			//(*m_client) << "Anzahl Zonen :";
			(*m_client) << itoa(AnzahlZonen,buffer,10);
			(*m_client) << ";";	
			
			//Sync Signal
			(*m_client) << itoa(Sync,buffer,10);
			(*m_client) << ";";	
			
			//(*m_client) << "Zone\tX\tY\tR\tG\tB\tF\tGw;";	

			//(*m_client) << "-------------------------------------------------------------------;";	

			for (i=0; i<=aktZoneNr; i++)
			{
				if (Liste.Nr[i] != 0)
				{
					(*m_client) <<  _gcvt(Liste.Nr[i],5,buffer);
					(*m_client) << "\t";
					(*m_client) << _gcvt(Liste.X[i],5,buffer); 
					(*m_client) << "\t";
					(*m_client) << _gcvt(Liste.Y[i],5,buffer);
					(*m_client) << "\t";
					(*m_client) << _gcvt(Liste.R[i],5,buffer);
					(*m_client) << "\t";
					(*m_client) << _gcvt(Liste.G[i],5,buffer);
					(*m_client) << "\t";
					(*m_client) << _gcvt(Liste.B[i],5,buffer);
					(*m_client) << "\t";
					(*m_client) << (const char *)&Liste.F[i];
					(*m_client) << "\t";
					(*m_client) << _gcvt(Liste.Gw[i],5,buffer);
					(*m_client) << ";";
				}
			}
			(*m_client) << "\n";	
			m_client->flush();
		}

		//m_SWFilterParams.param4[0] = 1;
		//SetParams(&m_SWFilterParams);

		//Param5 veraltet - wurde zur Synchronisation mit Grafs MT3Force verwendet um die Aktivierung
		//des Filters mitzuteilen
		//if(firstdarkframe == true) {m_SWFilterParams.param5=1; SetParams(&m_SWFilterParams);}

	}

	frameNr++;
	
	//Param7 veraltet - wurde zur Mitteilung der Nummer des verarbeiteten Frames an die MT3Force bentigt.
	//m_SWFilterParams.param7++;
	return NOERROR;
}


///////////////////////////////////////////////////////////////////////
// NonDelegatingQueryInterface:
//   Reveals ISWFilter, ISpecifyPropertyPages, IPersistStream
///////////////////////////////////////////////////////////////////////
STDMETHODIMP CSWFilter::NonDelegatingQueryInterface(REFIID riid, void **ppv)
{
    CheckPointer(ppv,E_POINTER);

    if (riid == IID_ISWFilter) {
        return GetInterface((ISWFilter *) this, ppv);
    } else if (riid == IID_IPersistStream) {
        return GetInterface((IPersistStream *) this, ppv);
    } else if (riid == IID_ISpecifyPropertyPages) {
        return GetInterface((ISpecifyPropertyPages *) this, ppv);
    } else {
        return CTransformFilter::NonDelegatingQueryInterface(riid, ppv);
    }
}

///////////////////////////////////////////////////////////////////////
// GetParams: Get the filter parameters to a given destination
///////////////////////////////////////////////////////////////////////
STDMETHODIMP CSWFilter::GetParams(SWFilterParams *irp)
{
    CAutoLock cAutolock(&m_SWFilterLock);
    CheckPointer(irp,E_POINTER);

    *irp = m_SWFilterParams;

    return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// SetParams: Set the filter parameters
///////////////////////////////////////////////////////////////////////
STDMETHODIMP CSWFilter::SetParams(SWFilterParams *irp)
{
    CAutoLock cAutolock(&m_SWFilterLock);
    m_SWFilterParams = *irp;
	if (m_SWFilterParams.intThreshold>255) {m_SWFilterParams.intThreshold=255;}

    SetDirty(TRUE);

    // Save the config whenever the user changes it
    saveConfig();

    return NOERROR;
} 

#define WRITEOUT(var)  hr = pStream->Write(&var, sizeof(var), NULL); \
               if (FAILED(hr)) return hr;

#define READIN(var)    hr = pStream->Read(&var, sizeof(var), NULL); \
               if (FAILED(hr)) return hr;

///////////////////////////////////////////////////////////////////////
// GetClassID: This is the only method of IPersist
///////////////////////////////////////////////////////////////////////
STDMETHODIMP CSWFilter::GetClassID(CLSID *pClsid)
{
    return CBaseFilter::GetClassID(pClsid);
}

///////////////////////////////////////////////////////////////////////
// WriteToStream: called when save the filter to stream
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilter::WriteToStream(IStream *pStream)
{
    HRESULT hr;
    WRITEOUT(m_SWFilterParams.intFilter);
    WRITEOUT(m_SWFilterParams.intNet);
	WRITEOUT(m_SWFilterParams.intPort);
    WRITEOUT(m_SWFilterParams.intThreshold);
	WRITEOUT(m_SWFilterParams.intZoneThreshold);
	WRITEOUT(m_SWFilterParams.blnDrawFeaturePoints);
	WRITEOUT(m_SWFilterParams.intSync);
	return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// ReadFromStream: called when read from stream
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilter::ReadFromStream(IStream *pStream)
{
    HRESULT hr;
    READIN(m_SWFilterParams.intFilter);
    READIN(m_SWFilterParams.intNet);
	READIN(m_SWFilterParams.intPort);
    READIN(m_SWFilterParams.intThreshold);
	READIN(m_SWFilterParams.intZoneThreshold);
	READIN(m_SWFilterParams.blnDrawFeaturePoints);
	READIN(m_SWFilterParams.intSync);
    return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// GetPages: Returns the clsid's of the property pages we support
///////////////////////////////////////////////////////////////////////
STDMETHODIMP CSWFilter::GetPages(CAUUID *pPages)
{
    pPages->cElems = 1;
    pPages->pElems = (GUID *) CoTaskMemAlloc(sizeof(GUID));
    if (pPages->pElems == NULL) {
        return E_OUTOFMEMORY;
    }
    *(pPages->pElems) = CLSID_SWFilterProp;
    return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// DllRegisterServer
///////////////////////////////////////////////////////////////////////
STDAPI DllRegisterServer()
{
    return AMovieDllRegisterServer2( TRUE );
}

///////////////////////////////////////////////////////////////////////
// DllUnregisterServer
///////////////////////////////////////////////////////////////////////
STDAPI DllUnregisterServer()
{
    return AMovieDllRegisterServer2( FALSE );
}
