// SWFilterProp.h: implementation of CSWFilterProp class
// 
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"

#include <streams.h>
#include <commctrl.h>
#include <olectl.h>
#include <memory.h>
#include <stdlib.h>
#include <stdio.h>
#include <tchar.h>

#include "resource.h"
#include "iSWFilter.h"
#include "SWFilter.h"
#include "SWFilterProp.h"


///////////////////////////////////////////////////////////////////////
// CreateInstance: Used by the DirectShow base classes to create 
// instances
///////////////////////////////////////////////////////////////////////
CUnknown *CSWFilterProp::CreateInstance(LPUNKNOWN lpunk, HRESULT *phr)
{
    CUnknown *punk = new CSWFilterProp(lpunk, phr);
    if (punk == NULL) {
    *phr = E_OUTOFMEMORY;
    }
    return punk;

}

///////////////////////////////////////////////////////////////////////
// Constructor
///////////////////////////////////////////////////////////////////////
CSWFilterProp::CSWFilterProp(LPUNKNOWN pUnk, HRESULT *phr) :
    CBasePropertyPage(NAME("SWFilter Property Page"),
                      pUnk,IDD_PROPERTIES,IDS_TITLE),
    m_pISWFilter(NULL),
    m_bIsInitialized(FALSE)
{
    ASSERT(phr);
}

///////////////////////////////////////////////////////////////////////
// OnReceiveMessage: Handles the messages for our property window
///////////////////////////////////////////////////////////////////////
BOOL CSWFilterProp::OnReceiveMessage(HWND hwnd,
                                          UINT uMsg,
                                          WPARAM wParam,
                                          LPARAM lParam)
{
    switch (uMsg)
    {
        case WM_INITDIALOG:  // init property page
            break;

        case WM_COMMAND:
        {
            if (m_bIsInitialized) {
                // Set dirty to indicate the setting has changed
                m_bDirty = TRUE;
                if (m_pPageSite)
                    m_pPageSite->OnStatusChange(PROPPAGESTATUS_DIRTY);
            }
            return (LRESULT) 1;
        }
    }
    return CBasePropertyPage::OnReceiveMessage(hwnd,uMsg,wParam,lParam);
}

///////////////////////////////////////////////////////////////////////
// OnConnect: Called when we connect to a transform filter
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilterProp::OnConnect(IUnknown *pUnknown)
{
    ASSERT(m_pISWFilter == NULL);

    HRESULT hr = pUnknown->QueryInterface(IID_ISWFilter, (void **) &m_pISWFilter);
    if (FAILED(hr)) {
        return E_NOINTERFACE;
    }

    ASSERT(m_pISWFilter);
    m_bIsInitialized = FALSE ;

    return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// OnDisconnect: Likewise called when we disconnect from a filter
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilterProp::OnDisconnect()
{
    if (m_pISWFilter == NULL) {
        return E_UNEXPECTED;
    }

    m_pISWFilter->Release();
    m_pISWFilter = NULL;
    return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// OnActivate: We are being activated
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilterProp::OnActivate()
{
    ASSERT(m_pISWFilter);
    // Get the current setup of the filter
    m_pISWFilter->GetParams(&m_SWFilterParams);
    // Set the control to the current setup
    SetControlValues();
    m_bIsInitialized = TRUE;

    return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// OnDeactivate: We are being deactivated
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilterProp::OnDeactivate(void)
{
    ASSERT(m_pISWFilter);
    m_bIsInitialized = FALSE;
    
    return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// OnApplyChanges: Apply any changes so far made
///////////////////////////////////////////////////////////////////////
HRESULT CSWFilterProp::OnApplyChanges()
{
    ASSERT(m_pISWFilter);
    // Update the setup from controls and set it into the filter
    GetControlValues();
    m_pISWFilter->SetParams(&m_SWFilterParams);

    return NOERROR;
}

///////////////////////////////////////////////////////////////////////
// GetControlValues: read the setup from current controls
///////////////////////////////////////////////////////////////////////
void CSWFilterProp::GetControlValues()
{
    // Is Filter turned on?
    m_SWFilterParams.intFilter = IsDlgButtonChecked( m_Dlg, chkFilter ) == BST_CHECKED;
    
    // Is Network turned on?
    m_SWFilterParams.intNet = IsDlgButtonChecked( m_Dlg, chkNet ) == BST_CHECKED;

	//Get Portnumber from Property Dialog
    TCHAR   sz[STR_MAX_LENGTH];
    Edit_GetText(GetDlgItem(m_Dlg, txtPort), sz, STR_MAX_LENGTH);
    m_SWFilterParams.intPort = atoi(sz);

	// Get Threshold from Property Dialog
	Edit_GetText(GetDlgItem(m_Dlg, txtThreshold), sz, STR_MAX_LENGTH);
    m_SWFilterParams.intThreshold = atoi(sz);

	// Get Zonethreshold from Property Dialog
	Edit_GetText(GetDlgItem(m_Dlg, txtZoneThreshold), sz, STR_MAX_LENGTH);
    m_SWFilterParams.intZoneThreshold = atoi(sz);

    // param4
//    Edit_GetText(GetDlgItem(m_Dlg, IDC_PARAMETERS4), sz, STR_MAX_LENGTH);
//    memcpy(m_SWFilterParams.param4, sz, sizeof(int)*STR_MAX_LENGTH);

	// param4
//    Edit_GetText(GetDlgItem(m_Dlg, IDC_PARAMETERS4), sz, STR_MAX_LENGTH);
//    memcpy(m_SWFilterParams.param4, sz, sizeof(TCHAR)*STR_MAX_LENGTH);


    m_SWFilterParams.blnDrawFeaturePoints = IsDlgButtonChecked( m_Dlg, chkDrawFeaturePoints ) == BST_CHECKED;

	// param6
    //Edit_GetText(GetDlgItem(m_Dlg, IDC_PARAMETERS6), sz, STR_MAX_LENGTH);
    //m_SWFilterParams.param6 = atoi(sz);

	// param7
    //Edit_GetText(GetDlgItem(m_Dlg, IDC_PARAMETERS7), sz, STR_MAX_LENGTH);
    //m_SWFilterParams.param7 = atoi(sz);

}

///////////////////////////////////////////////////////////////////////
// SetControlValues: set the current setup to the controls
///////////////////////////////////////////////////////////////////////
void CSWFilterProp::SetControlValues()
{
    // param1
    CheckDlgButton(m_Dlg, chkFilter, m_SWFilterParams.intFilter);
 
    CheckDlgButton(m_Dlg, chkNet, m_SWFilterParams.intNet);

    TCHAR   sz[STR_MAX_LENGTH];
    _stprintf(sz, TEXT("%d"), m_SWFilterParams.intPort);
    Edit_SetText(GetDlgItem(m_Dlg, txtPort), sz);

	_stprintf(sz, TEXT("%d"), m_SWFilterParams.intThreshold);
    Edit_SetText(GetDlgItem(m_Dlg, txtThreshold), sz);

    _stprintf(sz, TEXT("%d"), m_SWFilterParams.intZoneThreshold);
    Edit_SetText(GetDlgItem(m_Dlg, txtZoneThreshold), sz);


    CheckDlgButton(m_Dlg, chkDrawFeaturePoints, m_SWFilterParams.blnDrawFeaturePoints);

}

