/* A fairly crude Implementation of the Socket class
 * for Windows and Unix-Systems
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Author: Ronald Tanner                   Created: 2004-03-10
 */

//#ifndef WIN32
//#define WIN32
//#endif 

#include "Socket.h"
#include "string.h"
#include <string.h>
#ifdef WIN32
bool Socket::initialized=false;
typedef int socklen_t;
#define MSG_NOSIGNAL 0
#else
#include <fcntl.h>
#endif

#include <errno.h>
#include <iostream>

#ifdef WIN32
char *geterror(){
  LPVOID lpMsgBuf=0;
  FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER |
		FORMAT_MESSAGE_FROM_SYSTEM |
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT ),
		(LPTSTR) &lpMsgBuf, 0, NULL );
  return (char*)lpMsgBuf;
}
#endif

void Socket::init(){
#ifdef WIN32
    if( !Socket::initialized ) {
	WSADATA info;
	if (WSAStartup(MAKEWORD(2,0), &info)) {
	    throw "Could not start WSA";
	}
	Socket::initialized = true;
    }
#endif
}

Socket::Socket() :
  m_sock ( 0 ) {
    init();
    memset ( &m_addr,
	     0,
	     sizeof ( m_addr ) );
    
}

Socket::~Socket(){
  if ( is_valid() )
#ifdef WIN32
    ::closesocket( m_sock );
#else
    ::close ( m_sock );
#endif
}

bool Socket::create() {
    m_errmsg="";
    m_sock = socket ( AF_INET,
		      SOCK_STREAM,
		      0 );
    
    if ( ! is_valid() ){
	m_errmsg=std::string("socket: ") + strerror( errno );
	return false;
    }

  return true;

}



bool Socket::bind ( const int port )
{
  if ( ! is_valid() )    {
      return false;
  }
  m_errmsg="";
#ifdef WIN32
  m_addr.sin_family = PF_INET;
#else
  m_addr.sin_family = AF_INET;
#endif
  m_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  m_addr.sin_port = htons ( port );

  int bind_return = ::bind ( m_sock,
			     ( struct sockaddr * ) &m_addr,
			     sizeof ( m_addr ) );
  

  if ( bind_return == -1 )  {
      m_errmsg=std::string("bind: ") + strerror( errno );
      return false;
  }

  return true;
}


bool Socket::listen()
{
  if ( ! is_valid() )    {
      return false;
  }
  m_errmsg="";
  int listen_return = ::listen ( m_sock, MAXCONNECTIONS );


  if ( listen_return == -1 ) {
      m_errmsg=std::string("listen: ") + strerror( errno );
      return false;
  }

  return true;
}


bool Socket::accept ( Socket& new_socket ) {
    if( !is_valid() ){
	return false;
    }
    m_errmsg="";
    int addr_length = sizeof ( m_addr );
    new_socket.m_sock = ::accept ( m_sock, ( sockaddr * ) &m_addr, 
				   ( socklen_t * ) &addr_length );
    
    if ( new_socket.m_sock <= 0 ){
	m_errmsg=std::string("accept: ") + strerror( errno );
	return false;
    }
    return true;
}


bool Socket::send ( const std::string s ) {
    if( !is_valid() ){
	return false;
    }
    m_errmsg="";
    int status = ::send ( m_sock, s.c_str(), s.size(), MSG_NOSIGNAL );
    if ( status == -1 ){
	m_errmsg=std::string("send: ") + strerror( errno );
	return false;
    }
    return true;
}


int Socket::recv ( std::string& s ) {
  char buf [ MAXRECV + 1 ];

  s = "";

  if( !is_valid() ){
      return false;
  }
  m_errmsg="";
  memset ( buf, 0, MAXRECV + 1 );
  int status = ::recv ( m_sock, buf, MAXRECV, 0 );

  if ( status == -1 ) {
      m_errmsg= std::string("recv: ") + strerror( errno );
      return -1;
  }
  else if ( status == 0 ) {
      return 0;
  }
  else {
      s = buf;
      return status;
  }
}



bool Socket::connect ( const std::string host, const int port ) {
    if ( ! is_valid() ) return false;
    m_errmsg="";
  
  hostent *he;
  if ((he = gethostbyname(host.c_str())) == NULL) {
    m_errmsg = std::string( "gethostbyname: " ) + host + " " +strerror(errno);
    return false;
  }

  m_addr.sin_family = AF_INET;
  m_addr.sin_port = htons(port);
  m_addr.sin_addr = *((in_addr *)he->h_addr);
  memset(&(m_addr.sin_zero), 0, 8); 

  int status = ::connect ( m_sock, ( sockaddr * ) &m_addr, sizeof ( m_addr ) );

  if ( status == 0 )
      return true;
  m_errmsg = "connect: " + host + " " + 
#ifdef WIN32
    geterror()
#else
    strerror(errno)
#endif
;
  return false;
}
