/*  
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Author: Ronald Tanner                   Created: 2004-03-10
 */

#ifndef Socket_class
#define Socket_class

#ifdef WIN32
#include <WINSOCK.h>
#else
//#include <sys/types.h>
//#include <sys/socket.h>
//#include <netinet/in.h>
//#include <netdb.h>
//#include <unistd.h>
//#include <string>
//#include <arpa/inet.h>
#endif

#include <string>

const int MAXHOSTNAME = 200;
const int MAXCONNECTIONS = 5;
const int MAXRECV = 500;
//const int MSG_NOSIGNAL = 0; // defined by dgame

class Socket
{
 public:
  Socket();
  virtual ~Socket();

  // Server initialization
  bool create();
  bool bind ( const int port );
  bool listen();
  bool accept ( Socket& );

  // Client initialization
  bool connect ( const std::string host, const int port );

  // Data Transimission
  bool send ( const std::string );
  int recv ( std::string& );

  bool is_valid() const { return m_sock != 0; }

  std::string getError()const { return m_errmsg; }

 private:
  void init();

#ifdef WIN32
  SOCKET m_sock;
  static bool initialized;
#else
  int m_sock;
#endif
  sockaddr_in m_addr;
  std::string m_errmsg;
};


#endif
