using System;
using System.Threading;

using System.ComponentModel;
using System.ComponentModel.Design;

namespace MT.Synchronization
{
	public abstract class SyncInvoker: ISynchronizeInvoker
	{
		private delegate void Invoker(Delegate del, object[] arguments);
		
		public void SyncInvoke(Delegate del, object sender, EventArgs e)
		{			
			// Make sure that the delegate is instanciated
			if (del != null) 
			{
				// Create an object array which will hold
				// the arguments sent to the event handler 
				// function
				object[] arguments;			
				arguments = new object[2];
				arguments[0] = sender;
				arguments[1] = e;
				// --
				
				// Loop through all delegates in the invocationlist to 
				// examine each target object
				foreach(Delegate d in del.GetInvocationList()) 
				{
					// Try to cast the target object to a variable of the type
					// ISynchronizeInvoke. 
					// If this succeeds a synchronization is possible.
					ISynchronizeInvoke syncer = d.Target as ISynchronizeInvoke;
					if (syncer == null) 
					{
						// Cast failed, invoke delegate the regular way.
						d.DynamicInvoke(arguments);							
					} 
					else 
					{
						// Cast succeeded, the target object is a control
						// post the delegate to the main thread
						syncer.BeginInvoke(d, arguments);
					}
				}			
			} // if (del != null)
		} // SyncInvoke	} // class
	}
}
