using System;
using System.Collections;
using System.Diagnostics;
using System.ComponentModel;
using System.Threading;
using System.Security.Permissions;

namespace MT.Synchronization
{
	/// <summary>
	/// Zusammenfassung fr SynchronizerSortedList.
	/// </summary>
	[SecurityPermission(SecurityAction.Demand,ControlThread = true)]
	public class SynchronizerSortedList: SortedList, ISynchronizeInvoke, ISynchronizeInvoker, IDisposable
	{
		protected WorkerThread m_WorkerThread;

		internal WorkerThread workerThread
		{
			get { return m_WorkerThread; }
		}

		public bool InvokeRequired 
		{
			get
			{
				bool res = Thread.CurrentThread.GetHashCode() == m_WorkerThread.GetHashCode();
				return  ! res;
			}
		}
		public IAsyncResult BeginInvoke(Delegate method, object[] args)
		{
			WorkItem result = new WorkItem(null,method,args);
			m_WorkerThread.QueueWorkItem(result);
			return result;
		}
		public object EndInvoke(IAsyncResult result)
		{
			result.AsyncWaitHandle.WaitOne();
			WorkItem workItem = (WorkItem)result;
			return  workItem.MethodReturnedValue;
		}
		public object Invoke(Delegate method, object[] args)
		{
			IAsyncResult asyncResult;
			asyncResult = BeginInvoke(method,args);
			return EndInvoke(asyncResult);
		}
		public SynchronizerSortedList()
		{ 
			m_WorkerThread = new WorkerThread(this);
		}
		protected SynchronizerSortedList(WorkerThread workerThread)
		{
			m_WorkerThread = workerThread;
		}
		~SynchronizerSortedList()
		{
		}
		public void Dispose()
		{
			if (m_WorkerThread.Synchronizer == this)
				m_WorkerThread.Kill();
		}

		private delegate void Invoker(Delegate del, object[] arguments);
		
		public void SyncInvoke(Delegate del, object sender, EventArgs e)
		{			
			// Make sure that the delegate is instanciated
			if (del != null) 
			{
				// Create an object array which will hold
				// the arguments sent to the event handler 
				// function
				object[] arguments;			
				arguments = new object[2];
				arguments[0] = sender;
				arguments[1] = e;
				// --
				
				// Loop through all delegates in the invocationlist to 
				// examine each target object
				foreach(Delegate d in del.GetInvocationList()) 
				{
					// Try to cast the target object to a variable of the type
					// ISynchronizeInvoke. 
					// If this succeeds a synchronization is possible.
					ISynchronizeInvoke syncer = d.Target as ISynchronizeInvoke;
					if (syncer == null) 
					{
						// Cast failed, invoke delegate the regular way.
						d.DynamicInvoke(arguments);							
					} 
					else 
					{
						// Cast succeeded, the target object is a control
						// post the delegate to the main thread
						syncer.BeginInvoke(d, arguments);
					}
				}			
			} // if (del != null)
		}
		}
}
